@@ -1,3 +1,3 @@
-from nuauth_command.client import Client
+from nuauth_command.client import NuauthError, Client
 from nuauth_command.version import VERSION as __revision__
 
