@@ -4,11 +4,11 @@
  **            Vincent Deffontaines <vincent@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: checks.h 2795 2007-02-26 08:00:17Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,14 +23,11 @@
 #ifndef CHECKS_H
 #define CHECKS_H
 
-void *recv_message(void *session);
+nu_error_t recv_message(nuauth_session_t *session, nuclient_error_t *err);
 
 int nu_client_check(nuauth_session_t * session, nuclient_error * err);
 void *nu_client_thread_check(void *session);
 
 int nu_client_real_check(nuauth_session_t * session, nuclient_error * err);
 
-int secure_snprintf(char *buffer, unsigned int buffer_size, char *format,
-		    ...);
-
 #endif
