@@ -1,7 +1,8 @@
 /*
- ** Copyright 2004-2009 - INL
+ ** Copyright 2004-2010 - EdenWall Technologies
  ** Written by Eric Leblond <regit@inl.fr>
  **            Vincent Deffontaines <vincent@inl.fr>
+ **            Pierre Chifflier <chifflier@edenwall.com>
  ** INL http://www.inl.fr/
  **
  ** $Id$
@@ -138,9 +139,11 @@ int mysasl_negotiate(nuauth_session_t * session, sasl_conn_t * conn,
 {
 	char buf[8192];
 	const char *data;
+	const char *mechlist = session->sasl_mechlist;
 	const char *chosenmech;
 	unsigned len;
 	int result;
+	int retry_auth = 1;
 	/* gnutls_session session = session->tls; */
 
 	memset(buf, 0, sizeof buf);
@@ -150,8 +153,48 @@ int mysasl_negotiate(nuauth_session_t * session, sasl_conn_t * conn,
 		return SASL_FAIL;
 	}
 
-	result = sasl_client_start(conn,
-				   buf, NULL, &data, &len, &chosenmech);
+	if (mechlist == NULL)
+		mechlist = buf;
+
+	while (retry_auth > 0) {
+		retry_auth--;
+
+		if (session->verbose) {
+			log_printf(DEBUG_LEVEL_DEBUG, "Server mechanisms %s", buf);
+			log_printf(DEBUG_LEVEL_DEBUG, "Client mechanisms %s", mechlist);
+		}
+
+		result = sasl_client_start(conn,
+					   mechlist, NULL, &data, &len, &chosenmech);
+		if (result == -1) {
+			char * smech = strcasestr(mechlist, chosenmech);
+			int strclen = strlen(chosenmech);
+			/* remove chosenmech */
+			if (smech == mechlist) {
+				if (strcmp(mechlist, chosenmech) == 0) {
+					break;
+				}
+				/* need only to offset mechlist */
+				mechlist += strclen + 1;
+			} else {
+				int i = 0;
+				/* comma need to be suppressed */
+				smech--;
+				/* copy string with offset */
+				while (smech[i + strclen + 1]) {
+					smech[i] = smech[i + strclen + 1];
+					i++;
+				}
+				/* nullify end of string */
+				smech[i] = 0;
+			}
+			/* try again with updated mechlist */
+			log_printf(DEBUG_LEVEL_WARNING, "SASL negotiation with mechanism %s failed, retrying with %s",
+				   chosenmech,
+				   mechlist);
+			retry_auth = 1;
+		}
+	}
 
 	if (session->verbose) {
 		log_printf(DEBUG_LEVEL_INFO, "Using mechanism %s", chosenmech);
