@@ -3,11 +3,11 @@
  ** Written by Eric Leblond <regit@inL.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: proto.h 5223 2008-11-19 14:15:21Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -47,11 +47,14 @@ enum proto_type_t {
 };
 
 enum proto_client_version_t {
+	PROTO_VERSION_NONE,
 	PROTO_VERSION_V20 = 3,
-	PROTO_VERSION_V22
+	PROTO_VERSION_V22,
+	PROTO_VERSION_V22_1,
+	PROTO_VERSION_V24
 };
 
-#define PROTO_VERSION PROTO_VERSION_V22
+#define PROTO_VERSION PROTO_VERSION_V24
 
 enum proto_nufw_version_t {
 	PROTO_VERSION_NUFW_V20 = 3,
@@ -107,6 +110,7 @@ typedef struct {
  * (possible value of the option member of ::nuv2_authfield)
  */
 #define OS_SRV 0x1
+#define CLIENT_SRV 0x1
 
 #define APP_TYPE_NAME 0x1 /** application is defined by full path.  */
 
@@ -138,7 +142,8 @@ typedef enum {
 	USER_HELLO,
 	AUTH_CONN_DESTROY,
 	AUTH_CONN_UPDATE,
-	AUTH_CONN_FIXED_TIMEOUT
+	AUTH_CONN_FIXED_TIMEOUT,
+	EXTENDED_PROTO,
 } nufw_message_t;
 
 
@@ -147,8 +152,11 @@ typedef enum {
 	IPV6_FIELD,
 	APP_FIELD,
 	OS_FIELD,
-	USERNAME_FIELD,
-	HELLO_FIELD
+	VERSION_FIELD,
+	HELLO_FIELD,
+	CAPA_FIELD,
+	EXTENDED_PROTO_FIELD,
+	HASH_FIELD,
 } nu_field_identifier_t;
 
 struct nu_authreq {
@@ -161,7 +169,7 @@ struct nu_authreq {
  * See also the header of the whole packet: ::nu_authreq
  */
 struct nu_authfield {
-	uint8_t type;		/*!< Field type identifier: see ::nuv4_field_identifier_t */
+	uint8_t type;		/*!< Field type identifier: see ::nuv_field_identifier_t */
 	uint8_t option;		/*!< Option: equals to 0 to #OS_SRV */
 	uint16_t length;	/*!< Length of one field */
 };
@@ -220,15 +228,31 @@ typedef enum {
 	SRV_TYPE = 1,		/*!< Send server mode: #SRV_TYPE_PUSH or #SRV_TYPE_POLL */
 	SRV_REQUIRED_PACKET,
 	SRV_REQUIRED_DISCONNECT,
-	SRV_REQUIRED_HELLO
+	SRV_REQUIRED_HELLO,
+	SRV_REQUIRED_INFO,
+	SRV_EXTENDED_PROTO,
+	SRV_INIT,
 } nu_type_t;
 
 /** Server mode, value of with #SRV_TYPE (::nuv2_srv_message) message type */
 typedef enum {
 	SRV_TYPE_POLL = 0,	/*!< Server works in POLL mode (default) */
-	SRV_TYPE_PUSH		/*!< Server works in PUSH mode */
+	SRV_TYPE_PUSH,		/*!< Server works in PUSH mode */
+	SRV_HASH_TYPE		/*!< Server hash function for app sig */
 } nu_server_mode_t;
 
+typedef enum {
+	OS_VERSION = 0,
+	CLIENT_VERSION,
+	CLIENT_CAPA
+} nu_client_info_t;
+
+typedef enum {
+	INIT_NOK = 0,
+	INIT_OK
+} nu_srv_init_t;
+
+
 struct nu_srv_message {
 	uint8_t type;
 	uint8_t option;
