@@ -2,11 +2,11 @@
  ** Copyright(C) 2004-2007 INL
  ** Written by Eric Leblond <regit@inl.fr>
  **
- ** $Id: acls.c 5196 2008-11-18 15:28:35Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,11 +25,12 @@
 
 /*! \file acls.c
  *  \brief Acls manipulations and cache
- *
+
  *  It implements the functions needed to activate cache for acls and basic
  *  acl manipulations tasks
- *
- */
+
+
+*/
 
 
 
@@ -54,11 +55,7 @@ struct acl_key {
 	 * application full path
 	 */
 	gchar *appname;
-	/** application md5sum.
-	 *
-	 * md5sum of the binary which send the packet
-	 */
-	gchar *appmd5;
+	gchar *appsig;
 	/*
 	 * interfaces
 	 */
@@ -108,8 +105,6 @@ gboolean compare_acls(gconstpointer a, gconstpointer b)
 	}
 	if (strcmp_null(acl_key1->appname, acl_key2->appname))
 		return FALSE;
-	if (strcmp_null(acl_key1->appmd5, acl_key2->appmd5))
-		return FALSE;
 	if (strcmp_null(acl_key1->sysname, acl_key2->sysname))
 		return FALSE;
 	if (strcmp_null(acl_key1->release, acl_key2->release))
@@ -130,7 +125,6 @@ void free_acl_key(gpointer datas)
 	g_free(kdatas->release);
 	g_free(kdatas->version);
 	g_free(kdatas->appname);
-	g_free(kdatas->appmd5);
 	g_free(kdatas);
 }
 
@@ -174,7 +168,7 @@ gpointer acl_create_and_alloc_key(connection_t * kdatas)
 	key.release = kdatas->os_release;
 	key.version = kdatas->os_version;
 	key.appname = kdatas->app_name;
-	key.appmd5 = kdatas->app_md5;
+	key.appsig = kdatas->app_sig;
 	duplicate_iface_nfo(&(key.iface_nfo), &(kdatas->iface_nfo));
 	return acl_duplicate_key(&key);
 }
@@ -194,7 +188,6 @@ gpointer acl_duplicate_key(gpointer datas)
 	key->release = g_strdup(kdatas->release);
 	key->version = g_strdup(kdatas->version);
 	key->appname = g_strdup(kdatas->appname);
-	key->appmd5 = g_strdup(kdatas->appmd5);
 	duplicate_iface_nfo(&key->iface_nfo, &kdatas->iface_nfo);
 	return key;
 }
