@@ -5,11 +5,11 @@
  **            Pierre Chifflier <chifflier@inl.fr>
  **     INL : http://www.inl.fr/
  **
- ** $Id: auth_common.c 5486 2009-02-03 12:30:56Z pollux $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,8 +24,15 @@
 #include "auth_srv.h"
 #include <netinet/ip.h>
 #include <netinet/ip6.h>
+#include <sys/socket.h>
+#include <sys/un.h>
 #include <ctype.h>         /* isspace() */
 
+#include <nubase.h>
+
+/* See <sys/un.h for details, value is hardcoded */
+#define UNIX_MAX_PATH 108
+
 /**
  * \ingroup Nuauth
  * \defgroup NuauthCore Nuauth Core
@@ -128,35 +135,6 @@ void free_buffer_read(struct tls_buffer_read *datas)
 	g_free(datas);
 }
 
-/**
- * Function snprintf() which check buffer overflow, and always write a '\\0'
- * to the end of the buffer.
- *
- * \param buffer Buffer where characters are written
- * \param buffer_size Buffer size (in bytes), usually equals to sizeof(buffer)
- * \param format Format string (see printf() documentation)
- * \return Returns FALSE if a buffer overflow occurs, TRUE is everything goes fine.
- */
-gboolean secure_snprintf(char *buffer, unsigned int buffer_size,
-			 char *format, ...)
-{
-	va_list args;
-	int ret;
-	va_start(args, format);
-#ifdef DEBUG_ENABLE
-	memset(buffer, 0, buffer_size);
-#else
-	buffer[0] = 0;
-#endif
-	ret = g_vsnprintf(buffer, buffer_size, format, args);
-	va_end(args);
-	buffer[buffer_size - 1] = '\0';
-	if (0 <= ret && ret <= ((int) buffer_size - 1))
-		return TRUE;
-	else
-		return FALSE;
-}
-
 /**
  * Check Protocol version agains supported one
  *
@@ -190,6 +168,8 @@ nu_error_t check_protocol_version(enum proto_type_t type, int version)
 				case PROTO_VERSION_V20:
 					return NU_EXIT_OK;
 				case PROTO_VERSION_V22:
+				case PROTO_VERSION_V22_1:
+				case PROTO_VERSION_V24:
 					return NU_EXIT_OK;
 				default:
 					log_message(CRITICAL,
@@ -213,82 +193,6 @@ char* int_to_str(int value)
 	return g_strdup_printf("%i", value);
 }
 
-/**
- * Convert a string to a signed long integer number.
- * Skip spaces before first digit.
- * Return 0 on error, 1 otherwise.
- */
-int str_to_long(const char *text, long *value)
-{
-	char *err = NULL;
-	long longvalue;
-
-	/* skip spaces */
-	while (isspace(*text))
-		text++;
-
-	/* call strtol */
-	longvalue = strtol(text, &err, 10);
-	if (err == NULL || *err != 0)
-		return 0;
-	*value = longvalue;
-	return 1;
-}
-
-/**
- * Convert a string to an unsigned long integer number.
- * Skip spaces before first digit.
- * Return 0 on error, 1 otherwise.
- */
-int str_to_ulong(const char *text, unsigned long *value)
-{
-	char *err = NULL;
-	unsigned long ulongvalue;
-
-	/* skip spaces */
-	while (isspace(*text))
-		text++;
-
-	/* call strtol */
-	ulongvalue = strtoul(text, &err, 10);
-	if (err == NULL || *err != 0)
-		return 0;
-	*value = ulongvalue;
-	return 1;
-}
-
-/**
- * Convert a string to integer number (value in INT_MIN..INT_MAX).
- * Skip spaces before number value if any.
- * Return 0 on error, 1 otherwise.
- */
-int str_to_int(const char *text, int *value)
-{
-	long longvalue;
-	if (!str_to_long(text, &longvalue))
-		return 0;
-	if (longvalue < INT_MIN || INT_MAX < longvalue)
-		return 0;
-	*value = (int)longvalue;
-	return 1;
-}
-
-/**
- * Convert a string to a 32-bit unsigned integer (value in 0..4294967295).
- * Skip spaces before number value if any.
- * Returns 0 on error, 1 otherwise.
- */
-int str_to_uint32(const char *text, uint32_t * value)
-{
-	unsigned long ulongvalue;
-	if (!str_to_ulong(text, &ulongvalue))
-		return 0;
-	if (4294967295UL < ulongvalue)
-		return 0;
-	*value = (uint32_t)ulongvalue;
-	return 1;
-}
-
 /**
  * Wrapper to g_thread_pool_push(): block on server reload.
  */
@@ -311,6 +215,7 @@ int nuauth_bind(char **errmsg, const char *addr, const char *port, char *context
 	hints.ai_flags = AI_PASSIVE;
 	hints.ai_socktype = SOCK_STREAM;
 	hints.ai_family = PF_UNSPEC;
+
 	ecode = getaddrinfo(addr, port,
 			&hints, &res);
 	if (ecode != 0) {
@@ -336,6 +241,7 @@ int nuauth_bind(char **errmsg, const char *addr, const char *port, char *context
 	else
 		log_message(DEBUG, DEBUG_AREA_USER | DEBUG_AREA_MAIN,
 			    "Creating server (any) socket");
+
 	sck_inet = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
 	if (sck_inet == -1) {
 		*errmsg = g_strdup("Socket creation failed.");
@@ -373,6 +279,36 @@ int nuauth_bind(char **errmsg, const char *addr, const char *port, char *context
 	return sck_inet;
 }
 
+int nuauth_bind_unix(char **errmsg, const char *unix_path)
+{
+	struct sockaddr_un s_addr;
+	int sck_unix;
+	socklen_t len;
+
+	log_message(DEBUG, DEBUG_AREA_USER | DEBUG_AREA_MAIN,
+		    "Creating server (unix socket) on %s", unix_path);
+
+	sck_unix = socket(AF_UNIX, SOCK_STREAM, 0);
+	if (sck_unix == -1) {
+		*errmsg = g_strdup("Socket creation failed.");
+		return -1;
+	}
+
+	s_addr.sun_family = AF_UNIX;
+	strncpy(s_addr.sun_path, unix_path, UNIX_MAX_PATH-1);
+	/* ignore errors, socket may not exist */
+	unlink(s_addr.sun_path);
+	len = strlen(s_addr.sun_path) + sizeof(s_addr.sun_family);
+	if (bind(sck_unix, (struct sockaddr *)&s_addr, len) == -1) {
+		*errmsg = g_strdup_printf("Unable to bind socket to %s.",
+					  unix_path);
+		close(sck_unix);
+		return -1;
+	}
+
+	return sck_unix;
+}
+
 /**
  * Parse "[ipv6]:port", "[ipv6]", "ipv4:port" or "ipv4" string
  */
