@@ -1,12 +1,10 @@
 /*
- ** Copyright(C) 2005-2008 INL
+ ** Copyright(C) 2005-2009 INL
  ** Written by  Eric Leblond <regit@inl.fr>
  **
- ** $Id: client_mngr.c 5454 2008-12-20 21:20:57Z regit $
- **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,62 +39,73 @@ GMutex *client_mutex;
 GHashTable *client_conn_hash = NULL;
 GHashTable *client_ip_hash = NULL;
 
+typedef struct {
+	GSList *sessions;
+	int proto_version;
+	struct timeval last_message;
+} ip_sessions_t;
+
 static uint32_t hash_ipv6(struct in6_addr *addr)
 {
 	return jhash2(addr->s6_addr32, sizeof(*addr) / 4, 0);
 }
 
+/**
+ * Log and free structure relative to a user_session_t
+ *
+ * Used as destroy function for #client_conn_hash
+ */
+void log_clean_session(user_session_t * c_session)
+{
+	log_user_session(c_session, SESSION_CLOSE);
+	clean_session(c_session);
+}
+
 void clean_session(user_session_t * c_session)
 {
-	if (c_session->tls) {
-		gnutls_deinit(*(c_session->tls));
-		g_free(c_session->tls);
-	}
-	g_free(c_session->user_name);
-	g_slist_free(c_session->groups);
+	if (c_session->nussl)
+		nussl_session_destroy(c_session->nussl);
 
-	g_free(c_session->sysname);
-	g_free(c_session->release);
-	g_free(c_session->version);
+	if (c_session->user_name)
+		g_free(c_session->user_name);
+	if (c_session->groups)
+		g_slist_free(c_session->groups);
+
+	if (c_session->sysname)
+		g_free(c_session->sysname);
+	if (c_session->release)
+		g_free(c_session->release);
+	if (c_session->version)
+		g_free(c_session->version);
+	if (c_session->client_name)
+		g_free(c_session->client_name);
+	if (c_session->client_version)
+		g_free(c_session->client_version);
 
 	g_mutex_free(c_session->tls_lock);
 
 	g_free(c_session);
 }
 
-/**
- * Destroy function for #client_conn_hash
- */
-
-static void hash_clean_session(user_session_t * c_session)
-{
-	int socket = GPOINTER_TO_INT(gnutls_transport_get_ptr(*c_session->tls));
-	log_user_session(c_session, SESSION_CLOSE);
-	clean_session(c_session);
-	shutdown(socket, SHUT_RDWR);
-	close(socket);
-}
-
-
 void init_client_struct()
 {
 	client_mutex = g_mutex_new();
 	/* build client hash */
 	client_conn_hash = g_hash_table_new_full(NULL, NULL, NULL,
 						 (GDestroyNotify)
-						 hash_clean_session);
+						 log_clean_session);
 
 	/* build client hash */
 	client_ip_hash = g_hash_table_new_full((GHashFunc)hash_ipv6,
 					  (GEqualFunc)ipv6_equal,
 					  (GDestroyNotify) g_free,
-					  NULL);
+					  (GDestroyNotify) g_free);
 }
 
 void add_client(int socket, gpointer datas)
 {
 	user_session_t *c_session = (user_session_t *) datas;
-	GSList *ipsockets;
+	ip_sessions_t *ipsessions;
 	gpointer key;
 
 	g_mutex_lock(client_mutex);
@@ -104,46 +113,99 @@ void add_client(int socket, gpointer datas)
 	g_hash_table_insert(client_conn_hash, GINT_TO_POINTER(socket),
 			    datas);
 
-	/* need to create entry in ip hash */
-	ipsockets =
+	/* need to create entry in ip hash ? */
+	ipsessions =
 	    g_hash_table_lookup(client_ip_hash,
 				&c_session->addr);
-	ipsockets = g_slist_prepend(ipsockets, c_session);
-
-	key = g_memdup(&c_session->addr, sizeof(c_session->addr));
-	g_hash_table_replace(client_ip_hash, key, ipsockets);
-
+	if (ipsessions == NULL) {
+		ipsessions = g_new0(ip_sessions_t, 1);
+		ipsessions->proto_version = c_session->proto_version;
+		ipsessions->sessions = NULL;
+		key = g_memdup(&c_session->addr, sizeof(c_session->addr));
+		g_hash_table_replace(client_ip_hash, key, ipsessions);
+	}
+	/* let's assume backward compatibility, older client wins */
+	/* TODO: Add a configuration variable for this choice */
+	if (c_session->proto_version < ipsessions->proto_version) {
+		char buffer[256];
+		format_ipv6(&c_session->addr, buffer, 256, NULL);
+		ipsessions->proto_version = c_session->proto_version;
+		log_message(WARNING, DEBUG_AREA_USER,
+			    "User %s on %s uses older version of client",
+			    c_session->user_name,
+			    buffer);
+	}
+	if (c_session->proto_version > ipsessions->proto_version) {
+		char buffer[256];
+		format_ipv6(&c_session->addr, buffer, 256, NULL);
+		log_message(WARNING, DEBUG_AREA_USER,
+				"User %s on %s uses newer version of client",
+				c_session->user_name,
+				buffer);
+	}
+	ipsessions->sessions = g_slist_prepend(ipsessions->sessions, c_session);
 	g_mutex_unlock(client_mutex);
 }
 
-static GSList *delete_ipsockets_from_hash(GSList *ipsockets,
+static ip_sessions_t *delete_session_from_hash(ip_sessions_t *ipsessions,
 					  user_session_t *session,
 					  int destroy)
 {
 	gpointer key;
 	key = g_memdup(&session->addr, sizeof(session->addr));
-	ipsockets = g_slist_remove(ipsockets, session);
-	if (ipsockets != NULL) {
-		g_hash_table_replace(client_ip_hash,
-				key, ipsockets);
-	} else {
+	ipsessions->sessions = g_slist_remove(ipsessions->sessions, session);
+	if (ipsessions->sessions == NULL) {
 		g_hash_table_remove(client_ip_hash, key);
 		g_free(key);
+		ipsessions = NULL;
 	}
 	if (destroy) {
 		/* remove entry from hash */
 		key = GINT_TO_POINTER(session->socket);
-		g_hash_table_steal(client_conn_hash, key);
-		log_user_session(session, SESSION_CLOSE);
-		clean_session(session);
+		g_hash_table_remove(client_conn_hash, key);
 	}
-	return ipsockets;
+	return ipsessions;
+}
+
+
+static nu_error_t cleanup_session(user_session_t * session)
+{
+	ip_sessions_t *ipsessions;
+	/* destroy entry in IP hash */
+	ipsessions =
+		g_hash_table_lookup(client_ip_hash,
+				    &session->addr);
+	if (ipsessions) {
+		delete_session_from_hash(ipsessions, session, 0);
+	} else {
+		log_message(CRITICAL, DEBUG_AREA_USER,
+			    "Could not find entry in ip hash");
+		return NU_EXIT_ERROR;
+	}
+
+	tls_user_remove_client(session->socket);
+
+	return NU_EXIT_OK;
+}
+
+static nu_error_t delete_client_by_session(user_session_t * session)
+{
+	nu_error_t ret;
+
+	ret = cleanup_session(session);
+
+	if (ret != NU_EXIT_OK) {
+		return ret;
+	}
+
+	return NU_EXIT_OK;
 }
 
 nu_error_t delete_client_by_socket_ext(int socket, int use_lock)
 {
-	GSList *ipsockets;
+	gpointer key;
 	user_session_t *session;
+	nu_error_t ret;
 
 
 	if (use_lock) {
@@ -156,27 +218,35 @@ nu_error_t delete_client_by_socket_ext(int socket, int use_lock)
 				     GINT_TO_POINTER(socket)));
 	if (!session) {
 		log_message(WARNING, DEBUG_AREA_USER,
-			    "Could not find user session in hash for socket %d",
-			    socket);
+				"Could not find user session in hash");
 		if (use_lock)
 			g_mutex_unlock(client_mutex);
 		return NU_EXIT_ERROR;
 	}
 
-	/* destroy entry in IP hash */
-	ipsockets =
-		g_hash_table_lookup(client_ip_hash,
-				    &session->addr);
-	delete_ipsockets_from_hash(ipsockets, session, 1);
 
-	tls_user_remove_client(socket);
+	ret = cleanup_session(session);
+
+	if (ret != NU_EXIT_OK) {
+		if (use_lock)
+			g_mutex_unlock(client_mutex);
+		return ret;
+	}
+
+	if (shutdown(socket, SHUT_RDWR) != 0) {
+		log_message(VERBOSE_DEBUG, DEBUG_AREA_USER,
+				"Could not shutdown socket: %s", strerror(errno));
+	}
+	if (close(socket) != 0) {
+		log_message(VERBOSE_DEBUG, DEBUG_AREA_USER,
+				"Could not close socket: %s", strerror(errno));
+	}
+
+
+	key = GINT_TO_POINTER(session->socket);
+	g_hash_table_remove(client_conn_hash, key);
+
 	if (use_lock) {
-		if (shutdown(socket, SHUT_RDWR) != 0)
-			log_message(VERBOSE_DEBUG, DEBUG_AREA_USER,
-					"Could not shutdown socket");
-		if (close(socket) != 0)
-			log_message(VERBOSE_DEBUG, DEBUG_AREA_USER,
-					"Could not close socket");
 		g_mutex_unlock(client_mutex);
 	}
 
@@ -201,10 +271,13 @@ user_session_t *get_client_datas_by_socket(int socket)
 
 GSList *get_client_sockets_by_ip(struct in6_addr * ip)
 {
-	void *ret;
+	ip_sessions_t *session;
+	GSList *ret = NULL;
 
 	g_mutex_lock(client_mutex);
-	ret = g_hash_table_lookup(client_ip_hash, ip);
+	session = g_hash_table_lookup(client_ip_hash, ip);
+	if (session)
+		ret = session->sessions;
 	g_mutex_unlock(client_mutex);
 	return ret;
 }
@@ -272,6 +345,39 @@ gboolean test_username_count_vs_max(const gchar * username, int maxcount)
 	}
 }
 
+/**
+ * Property check
+ */
+
+gboolean check_property_clients(struct in6_addr *addr, user_session_check_t *scheck, int mode, gpointer data)
+{
+	gboolean cst = FALSE;
+	ip_sessions_t *ipsessions = NULL;
+	GSList *ipsockets = NULL;
+
+	g_mutex_lock(client_mutex);
+	ipsessions = g_hash_table_lookup(client_ip_hash, addr);
+	if (ipsessions) {
+		for (ipsockets = ipsessions->sessions; ipsockets; ipsockets = ipsockets->next) {
+			user_session_t *session = (user_session_t *)ipsockets->data;
+			cst = scheck(session, data);
+			if (mode) {
+				if (cst == TRUE) {
+					g_mutex_unlock(client_mutex);
+					return TRUE;
+				}
+			}
+		}
+		g_mutex_unlock(client_mutex);
+		return cst;
+	} else {
+		g_mutex_unlock(client_mutex);
+		return FALSE;
+	}
+	g_mutex_unlock(client_mutex);
+	return FALSE;
+}
+
 /**
  * Ask each client of global_msg address set to send their new connections
  * (connections in stage "SYN SENT").
@@ -279,43 +385,59 @@ gboolean test_username_count_vs_max(const gchar * username, int maxcount)
  * \param global_msg Address set of clients
  * \return Returns 0 on error, 1 otherwise
  */
-char warn_clients(struct msg_addr_set *global_msg)
+char warn_clients(struct msg_addr_set *global_msg,
+		  user_session_check_t *scheck,
+		  gpointer data)
 {
-	GSList *start_ipsockets = NULL;
+	ip_sessions_t *ipsessions = NULL;
 	GSList *ipsockets = NULL;
 	GSList *badsockets = NULL;
+	struct timeval timestamp;
+	struct timeval interval;
 #if DEBUG_ENABLE
-	if (DEBUG_OR_NOT(DEBUG_LEVEL_VERBOSE_DEBUG, DEBUG_AREA_USER))
-	{
+	if (DEBUG_OR_NOT(DEBUG_LEVEL_VERBOSE_DEBUG, DEBUG_AREA_USER)) {
 		char addr_ascii[INET6_ADDRSTRLEN];
-		FORMAT_IPV6(&global_msg->addr, addr_ascii);
+		format_ipv6(&global_msg->addr, addr_ascii, INET6_ADDRSTRLEN, NULL);
 		g_message("Warn client(s) on IP %s", addr_ascii);
 	}
 #endif
 
 	g_mutex_lock(client_mutex);
-	start_ipsockets =
-	    g_hash_table_lookup(client_ip_hash,
-				&global_msg->addr);
-	if (start_ipsockets) {
+	ipsessions = g_hash_table_lookup(client_ip_hash, &global_msg->addr);
+	if (ipsessions) {
 		global_msg->found = TRUE;
-		for (ipsockets = start_ipsockets; ipsockets; ipsockets = ipsockets->next)
-		{
+
+		if ((!(data || scheck)) && ipsessions->proto_version >= PROTO_VERSION_V22_1) {
+			gettimeofday(&timestamp, NULL);
+			timeval_substract(&interval, &timestamp, &(ipsessions->last_message));
+			if (interval.tv_sec || ((unsigned)interval.tv_usec < nuauthconf->push_delay)) {
+				g_mutex_unlock(client_mutex);
+				return 1;
+			} else {
+				ipsessions->last_message.tv_sec = timestamp.tv_sec;
+				ipsessions->last_message.tv_usec = timestamp.tv_usec;
+			}
+		}
+		for (ipsockets = ipsessions->sessions; ipsockets; ipsockets = ipsockets->next) {
 			user_session_t *session = (user_session_t *)ipsockets->data;
-			gnutls_session tls = *session->tls;
-			int ret = gnutls_record_send(tls,
-					global_msg->msg,
-					ntohs(global_msg->msg->length));
-			if (ret < 0) {
-				/* Test on non critical error */
-				if (gnutls_error_is_fatal(ret)) {
+			int ret;
+
+			if ((!scheck) || scheck(session, data)) {
+				ret = nussl_write(session->nussl,
+						(char*)global_msg->msg,
+						ntohs(global_msg->msg->length));
+				if (ret < 0) {
 					log_message(WARNING, DEBUG_AREA_USER,
-							"Failed to send warning to client(s).");
+							"Failed to send warning to client(s): %s", nussl_get_error(session->nussl));
 					badsockets = g_slist_prepend(badsockets, GINT_TO_POINTER(ipsockets->data));
-				} else {
-					log_message(INFO, DEBUG_AREA_USER,
-							"Failed to send warning to client(s) (non fatal TLS error).");
 				}
+#if DEBUG_ENABLE
+				else {
+					log_message(VERBOSE_DEBUG, DEBUG_AREA_USER,
+							"Message sent to client.");
+
+				}
+#endif /* DEBUG_ENABLE */
 			}
 		}
 		if (badsockets) {
@@ -324,7 +446,7 @@ char warn_clients(struct msg_addr_set *global_msg)
 				nu_error_t ret = delete_client_by_socket_ext(sockno, 0);
 				if (ret != NU_EXIT_OK) {
 					log_message(WARNING, DEBUG_AREA_USER,
-						"Failed to destroy socket in hash.");
+						"Fails to destroy socket in hash.");
 				}
 			}
 			g_slist_free(badsockets);
@@ -341,7 +463,10 @@ char warn_clients(struct msg_addr_set *global_msg)
 gboolean hash_delete_client(gpointer key, gpointer value,
 			    gpointer userdata)
 {
-	g_slist_free(value);
+	ip_sessions_t *ipsessions = (ip_sessions_t *) value;
+	if (ipsessions->sessions) {
+		g_slist_free(ipsessions->sessions);
+	}
 	return TRUE;
 }
 
@@ -359,6 +484,12 @@ void close_clients(int signal)
 gboolean is_expired_client(gpointer key,
 			   gpointer value, gpointer user_data)
 {
+	if (! value) {
+		return FALSE;
+	}
+	if (((user_session_t *) value)->expire == -1) {
+		return FALSE;
+	}
 	if (((user_session_t *) value)->expire < *((time_t *) user_data)) {
 		return TRUE;
 	} else {
@@ -366,13 +497,17 @@ gboolean is_expired_client(gpointer key,
 	}
 }
 
+void clean_client_session_bycallback(GHRFunc cb, gpointer data)
+{
+	g_mutex_lock(client_mutex);
+	g_hash_table_foreach_remove(client_conn_hash, cb, data);
+	g_mutex_unlock(client_mutex);
+}
+
 void kill_expired_clients_session()
 {
 	time_t current_time = time(NULL);
-	g_mutex_lock(client_mutex);
-	g_hash_table_foreach_remove(client_conn_hash, is_expired_client,
-				    &current_time);
-	g_mutex_unlock(client_mutex);
+	clean_client_session_bycallback(is_expired_client, &current_time);
 }
 
 /**
@@ -390,7 +525,7 @@ gboolean kill_all_clients_cb(gpointer sock, user_session_t* session, gpointer da
 	if (session->activated == FALSE)
 		return FALSE;
 
-	if (delete_client_by_socket_ext(GPOINTER_TO_INT(sock), 0) == NU_EXIT_OK)
+	if (delete_client_by_session(session) == NU_EXIT_OK)
 		return TRUE;
 	else
 		return FALSE;
@@ -429,38 +564,4 @@ nu_error_t activate_client_by_socket(int socket)
 	return NU_EXIT_ERROR;
 }
 
-nu_error_t disconnect_client_by_socket(int sock)
-{
-	int ok = NU_EXIT_OK;
-	GSList *thread_p;
-	disconnect_user_msg_t msg;
-	msg.cond = g_cond_new();
-	msg.mutex = g_mutex_new();
-
-	/* iter on each server thread */
-	for (thread_p=nuauthdatas->tls_auth_servers; thread_p; thread_p = thread_p->next) {
-		struct tls_user_context_t *this =
-			((struct nuauth_thread_t *)thread_p->data)->data;
-		msg.socket = sock;
-		/* send query to disconnect users */
-		g_async_queue_push(this->cmd_queue, &msg);
-
-		/* wait until clients are disconnected */
-		g_cond_wait(msg.cond, msg.mutex);
-
-		/* write answer */
-		if (msg.result == NU_EXIT_OK) {
-			ok = NU_EXIT_OK;
-			break;
-		} else {
-			ok = NU_EXIT_ERROR;
-		}
-	}
-	g_cond_free(msg.cond);
-	g_mutex_free(msg.mutex);
-	return ok;
-}
-
-
-
 /** @} */
