@@ -2,11 +2,11 @@
  ** Copyright(C) 2005-2007 INL
  ** Written by Eric Leblond <regit@inl.fr>
  **
- ** $Id: client_mngr.h 5452 2008-12-20 21:00:59Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -40,6 +40,7 @@ user_session_t *look_for_username(const gchar * username);
 
 gboolean test_username_count_vs_max(const gchar * username, int maxcount);
 
+void log_clean_session(user_session_t *);
 void clean_session(user_session_t *);
 
 void foreach_session(GHFunc callback, void *data);
@@ -50,8 +51,14 @@ struct msg_addr_set {
 	gboolean found;
 };
 
+typedef gboolean user_session_check_t(user_session_t * session, gpointer data);
 
-char warn_clients(struct msg_addr_set *global_msg);
+char warn_clients(struct msg_addr_set *global_msg, user_session_check_t *scheck,
+		  gpointer data);
+
+gboolean check_property_clients(struct in6_addr *addr, user_session_check_t *scheck, int mode, gpointer data);
+
+void clean_client_session_bycallback(GHRFunc cb, gpointer data);
 
 void close_clients();
 
@@ -59,8 +66,6 @@ nu_error_t kill_all_clients();
 void kill_expired_clients_session();
 nu_error_t activate_client_by_socket(int socket);
 
-nu_error_t disconnect_client_by_socket(int socket);
-
 guint get_number_of_clients();
 
 struct username_counter {
