@@ -1,13 +1,13 @@
 /*
- ** Copyright(C) 2007-2008 INL
+ ** Copyright(C) 2007,2008 INL
  ** Written by Victor Stinner <victor.stinner@inl.fr>
  **	       Eric Leblond <eric@inl.fr>
  **
- ** $Id: command.c 5452 2008-12-20 21:00:59Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,29 +19,46 @@
  ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "auth_srv.h"
+
 #include "command.h"
+
+#ifdef BUILD_NUAUTH_COMMAND
+#include "auth_srv.h"
 #include "command_enc.h"
 #include "security.h"
+
+#include <errno.h>
 #include <sys/un.h>		/* unix socket */
 #include <sys/stat.h>		/* fchmod() */
+#include <sys/types.h>		/* mode_t */
 
-#define SOCKET_FILENAME LOCAL_STATE_DIR "/run/nuauth/nuauth-command.socket"
+#include <nubase.h>
+
+#include "nuauthconf.h"
+
+#define SOCKET_PATH LOCAL_STATE_DIR "/run/nuauth/"
+#define SOCKET_FILENAME "nuauth-command.socket"
+#define SOCKET_TARGET SOCKET_PATH SOCKET_FILENAME
+
+GHashTable *client_conn_hash;
 
 const char* COMMAND_HELP =
 "version: display nuauth version\n"
+"confdump: dump configuration\n"
 "users: list connected users\n"
 "firewalls: list connected nufw firewalls\n"
 "packets count: display number of decision waiting packets\n"
+"user count: display number of connected clients\n"
 "refresh cache: refresh all caches\n"
 "refresh crl: refresh the TLS crl file\n"
-"disconnect ID: disconnect a user with his session identifier\n"
+"disconnect ID: disconnect an user with his session identifier\n"
 "disconnect all: disconnect all users\n"
 "uptime: display nuauth starting time and uptime\n"
-"reload: reload the configuration and reload the modules\n"
+"reload: reload nuauth configuration\n"
 "reload periods: reload the time periods\n"
 "display debug_level\n"
 "display debug_areas\n"
+"display threads\n"
 "debug_level LEVEL\n"
 "debug_areas AREAS\n"
 "help: display this help\n"
@@ -62,7 +79,7 @@ int command_new(command_t * this)
 {
 	struct sockaddr_un addr;
 	int len;
-	int res;
+	int ret;
 	int on = 1;
 
 	this->start_timestamp = time(NULL);
@@ -70,20 +87,27 @@ int command_new(command_t * this)
 	this->client = -1;
 	this->select_max = 0;
 
+	/* Create socket dir */
+	ret = mkdir(SOCKET_PATH, S_IRWXU);
+	if ( ret != 0 ) {
+		log_area_printf(DEBUG_AREA_AUTH, DEBUG_LEVEL_FATAL,
+				"Cannot create socket directory %s: %s", SOCKET_PATH, strerror(errno));
+	}
+
 	/* Remove socket file */
-	(void) unlink(SOCKET_FILENAME);
+	(void) unlink(SOCKET_TARGET);
 
 	/* set address */
 	addr.sun_family = AF_UNIX;
-	SECURE_STRNCPY(addr.sun_path, SOCKET_FILENAME, sizeof(addr.sun_path));
+	SECURE_STRNCPY(addr.sun_path, SOCKET_TARGET, sizeof(addr.sun_path));
 	addr.sun_path[sizeof(addr.sun_path) - 1] = 0;
 	len = strlen(addr.sun_path) + sizeof(addr.sun_family);
 
 	/* create socket */
 	this->socket = socket(AF_UNIX, SOCK_STREAM, 0);
 	if (this->socket == -1) {
-		g_warning("[%i] Command server: unable to create UNIX socket %s: %s",
-			    getpid(), addr.sun_path, g_strerror(errno));
+		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_CRITICAL, "Command server: unable to create UNIX socket %s: %s",
+				addr.sun_path, g_strerror(errno));
 		return 0;
 	}
 	this->select_max = this->socket + 1;
@@ -92,15 +116,19 @@ int command_new(command_t * this)
 	(void)fchmod(this->socket, 0600);
 
 	/* set reuse option */
-	res =
+	ret =
 	    setsockopt(this->socket, SOL_SOCKET, SO_REUSEADDR,
 		       (char *) &on, sizeof(on));
+	if ( ret != 0 ) {
+		log_area_printf(DEBUG_AREA_AUTH, DEBUG_LEVEL_FATAL,
+				"Cannot set sockets options: %s.",  strerror(errno));
+	}
 
 	/* bind socket */
-	res = bind(this->socket, (struct sockaddr *) &addr, len);
-	if (res == -1) {
+	ret = bind(this->socket, (struct sockaddr *) &addr, len);
+	if (ret == -1) {
 		g_warning("[%i] Command server: UNIX socket bind(%s) error: %s",
-			    getpid(), SOCKET_FILENAME, g_strerror(errno));
+			    getpid(), SOCKET_TARGET, g_strerror(errno));
 		return 0;
 	}
 
@@ -116,7 +144,7 @@ int command_new(command_t * this)
 void command_client_close(command_t * this)
 {
 	log_message(WARNING, DEBUG_AREA_MAIN,
-		    "Command server: closing client connection");
+		    "Command server: close client connection");
 	close(this->client);
 	this->client = -1;
 	this->select_max = this->socket + 1;
@@ -146,7 +174,7 @@ int command_client_accept(command_t * this)
 	ret = recv(this->client, buffer, sizeof(buffer)-1, 0);
 	if (ret < 0) {
 		log_message(CRITICAL, DEBUG_AREA_MAIN,
-			    "Command server: client didn't send version");
+			    "Command server: client doesn't send version");
 		command_client_close(this);
 		return 0;
 	}
@@ -215,11 +243,51 @@ void command_servers(command_t *this, encoder_t *encoder)
 }
 
 /**
- * Disconnect all clients
+ * Internal function do disconnect a client
+ */
+int command_do_disconnect(int sock)
+{
+	int ok = 1;
+	GSList *thread_p;
+
+	/* iter on each server thread */
+	for (thread_p=nuauthdatas->tls_auth_servers; thread_p; thread_p = thread_p->next) {
+		struct tls_user_context_t *this =
+			((struct nuauth_thread_t *)thread_p->data)->data;
+		/* send query to disconnect all users */
+		disconnect_user_msg_t *msg = g_new(disconnect_user_msg_t, 1);
+		msg->socket = sock;
+		msg->mutex = g_mutex_new();
+		g_async_queue_push(this->cmd_queue, msg);
+
+		/* wait until clients are disconnected */
+		g_mutex_lock(msg->mutex);
+		g_mutex_lock(msg->mutex);
+		g_mutex_unlock(msg->mutex);
+		g_mutex_free(msg->mutex);
+
+		/* write answer */
+		if (msg->result == NU_EXIT_OK) {
+			ok = 1;
+			g_free(msg);
+			break;
+		} else {
+			ok = 0;
+		}
+		/* return in case we've just send a global disconnect message */
+		if (sock == -1) {
+			break;
+		}
+	}
+	return ok;
+}
+
+/**
+ * Disconnect all client
  **/
 int command_disconnect_all(command_t *this, encoder_t *encoder)
 {
-	if (disconnect_client_by_socket(-1) == NU_EXIT_OK) {
+	if (command_do_disconnect(-1)) {
 		encoder_add_string(encoder, "users disconnected");
 		return 1;
 	} else {
@@ -240,7 +308,7 @@ int command_disconnect(command_t *this, encoder_t *encoder, char *command)
 		return 0;
 	}
 
-	if (disconnect_client_by_socket(sock) == NU_EXIT_OK) {
+	if (command_do_disconnect(sock)) {
 		encoder_add_string(encoder, "users disconnected");
 		return 1;
 	} else {
@@ -268,6 +336,11 @@ const char* fortune()
 	return FORTUNES[(int)index];
 }
 
+static void conf_server_side_print(void *encoder, char *buffer)
+{
+	encoder_add_string(encoder, buffer);
+}
+
 void command_execute(command_t * this, char *command)
 {
 	encoder_t *encoder, *answer;
@@ -291,6 +364,8 @@ void command_execute(command_t * this, char *command)
 		command_servers(this, encoder);
 	} else if (strcmp(command, "version") == 0) {
 		encoder_add_string(encoder, NUAUTH_FULL_VERSION);
+	} else if (strcmp(command, "confdump") == 0) {
+		nuauth_config_table_print(encoder, conf_server_side_print);
 	} else if (strcmp(command, "disconnect all") == 0) {
 		ok = command_disconnect_all(this, encoder);
 		force_refresh_crl_file();
@@ -298,8 +373,12 @@ void command_execute(command_t * this, char *command)
 		ok = command_disconnect(this, encoder, command+10);
 		force_refresh_crl_file();
 	} else if (strcmp(command, "reload") == 0) {
-		nuauth_reload(0);
-		encoder_add_string(encoder, "Configuration reloaded");
+		gboolean restart = nuauth_reload(0);
+		if (restart) {
+			encoder_add_string(encoder, "Configuration change requires Nuauth restart");
+		} else {
+			encoder_add_string(encoder, "Configuration reloaded");
+		}
 	} else if (strcmp(command, "reload periods") == 0) {
 		reload_periods(&nuauthconf->periods);
 		encoder_add_string(encoder, "Periods reloaded");
@@ -341,8 +420,32 @@ void command_execute(command_t * this, char *command)
 		encoder_add_int32(encoder, nuauthconf->debug_level);
 	} else if (strcmp(command, "display debug_areas") == 0) {
 		encoder_add_int32(encoder, nuauthconf->debug_areas);
+	} else if (strcmp(command, "display threads") == 0) {
+		char buffer [450];
+		snprintf (buffer, sizeof(buffer),
+				"User threads: %u\n"
+				"User session threads: %u\n"
+				"Unprocessed user threads: %u\n"
+				"Unprocessed user session threads: %u\n"
+				"Max user threads: %u\n"
+				"Max user session threads: %u\n"
+				"logger user pool full: %s\n"
+				"logger session pool full: %s\n",
+				g_thread_pool_get_num_threads(nuauthdatas->user_loggers),
+				g_thread_pool_get_num_threads(nuauthdatas->user_session_loggers),
+				g_thread_pool_unprocessed(nuauthdatas->user_loggers),
+				g_thread_pool_unprocessed(nuauthdatas->user_session_loggers),
+				g_thread_pool_get_max_threads(nuauthdatas->user_loggers),
+				g_thread_pool_get_max_threads(nuauthdatas->user_session_loggers),
+				(nuauthdatas->loggers_pool_full)?"true":"false",
+				(nuauthdatas->session_loggers_pool_full)?"true":"false"
+			);
+		buffer[sizeof(buffer)-1] = '\0';
+		encoder_add_string(encoder,buffer);
 	} else if (strcmp(command, "packets count") == 0) {
 		encoder_add_int32(encoder, g_hash_table_size(conn_list));
+	} else if (strcmp(command, "user count") == 0) {
+		encoder_add_int32(encoder, g_hash_table_size(client_conn_hash));
 	} else {
 		/* unknown command => disconnect */
 	}
@@ -390,7 +493,7 @@ void command_client_run(command_t * this)
 	{
 		log_message(WARNING, DEBUG_AREA_MAIN,
 			    "Command server: client command is too long, "
-			    "disconnecting him.");
+			    "disconnect him.");
 		command_client_close(this);
 	}
 	buffer[ret] = 0;
@@ -455,3 +558,6 @@ void *command_server(GMutex * mutex)
 	}
 	return NULL;
 }
+
+#endif
+
