@@ -2,11 +2,11 @@
  ** Copyright(C) 2007 INL
  ** Written by Victor Stinner <victor.stinner@inl.fr>
  **
- ** $Id: command.h 4126 2007-12-18 13:20:29Z haypo $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,6 +22,9 @@
 #define COMMAND_SERVER_H
 
 #include "auth_srv.h"
+#ifdef BUILD_NUAUTH_COMMAND
+
 void *command_server(GMutex * mutex);
 
+#endif				/* BUILD_NUAUTH_COMMAND */
 #endif				/* COMMAND_SERVER_H */
