@@ -4,11 +4,11 @@
  **            Vincent Deffontaines <vincent@gryzor.com>
  **     INL : http://www.inl.fr/
  **
- ** $Id: connections.c 5406 2008-12-07 23:25:07Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,8 +27,6 @@
  * @{
  */
 
-/** global lock for the conn list. */
-GStaticMutex insert_mutex;
 
 void free_connection_callback(gpointer conn, gpointer unused)
 {
@@ -144,7 +142,7 @@ void free_connection(connection_t * conn)
 	g_slist_free(conn->packet_id);
 	g_slist_free(conn->user_groups);
 	g_free(conn->app_name);
-	g_free(conn->app_md5);
+	g_free(conn->app_sig);
 	g_free(conn->os_sysname);
 	g_free(conn->os_release);
 	g_free(conn->os_version);
@@ -193,7 +191,7 @@ connection_t *duplicate_connection(connection_t * element)
 	}
 	conn_copy->username = g_strdup(element->username);
 	conn_copy->app_name = g_strdup(element->app_name);
-	conn_copy->app_md5 = g_strdup(element->app_md5);
+	conn_copy->app_sig = g_strdup(element->app_sig);
 	conn_copy->os_sysname = g_strdup(element->os_sysname);
 	conn_copy->os_release = g_strdup(element->os_release);
 	conn_copy->os_version = g_strdup(element->os_version);
@@ -207,6 +205,11 @@ connection_t *duplicate_connection(connection_t * element)
 	conn_copy->packet_id = NULL;
 	conn_copy->cacheduserdatas = NULL;
 	conn_copy->state = AUTH_STATE_DONE;
+
+	/* copy payload */
+	conn_copy->payload_len = element->payload_len;
+	memcpy(conn_copy->payload, element->payload, conn_copy->payload_len);
+
 	return conn_copy;
 }
 
@@ -222,7 +225,7 @@ connection_t *duplicate_connection(connection_t * element)
  * \return Returns 1 if success, 0 if it fails
  */
 
-inline int conn_cl_remove(gconstpointer conn)
+int conn_cl_remove(gconstpointer conn)
 {
 	if (!g_hash_table_steal(conn_list,
 				&(((connection_t *) conn)->tracking))) {
@@ -347,6 +350,7 @@ void clean_connections_list()
 			element->decision = DECISION_DROP;
 		if (element->state == AUTH_STATE_AUTHREQ) {
 			apply_decision(element);
+			element->state = AUTH_STATE_DONE;
 		}
 		free_connection(element);
 	}
@@ -381,8 +385,8 @@ static char * str_print_tracking_t(tracking_t *tracking)
 	char dst_ascii[INET6_ADDRSTRLEN];
 	char * ip_header, * proto_header, *message;
 
-	FORMAT_IPV6(&tracking->saddr, src_ascii);
-	FORMAT_IPV6(&tracking->daddr, dst_ascii);
+	format_ipv6(&tracking->saddr, src_ascii, INET6_ADDRSTRLEN, NULL);
+	format_ipv6(&tracking->daddr, dst_ascii, INET6_ADDRSTRLEN, NULL);
 
 	ip_header = g_strdup_printf(" src=%s dst=%s proto=%u",
 			src_ascii, dst_ascii, tracking->protocol);
@@ -409,24 +413,14 @@ nu_error_t print_tracking_t(tracking_t *tracking)
 
 	if (! tracking_display)
 		return NU_EXIT_ERROR;
-	
+
 	g_message("%s", tracking_display);
 
 	g_free(tracking_display);
 	return NU_EXIT_OK;
 }
 
-/**
- * Display connection parameters using g_message(): IP+TCP/UDP headers,
- * OS name, OS release and OS version, and application name.
- *
- * Only display the connection if ::debug_level is #DEBUG_LEVEL_VERBOSE_DEBUG
- * or greater.
- *
- * \return Returns -1 if an error occurs, 1 else.
- */
-
-gint print_connection(gpointer data, gpointer userdata)
+gint print_connection_wid(gpointer data, gpointer userdata, gboolean pid, uint32_t packet_id)
 {
 	connection_t *conn = (connection_t *) data;
 	char * prefix = userdata;
@@ -438,6 +432,8 @@ gint print_connection(gpointer data, gpointer userdata)
 	char * str_user = NULL;
 	char * str_os = NULL;
 	char * str_app = NULL;
+	char * str_sig = NULL;
+	char * str_exptime = NULL;
 	char * message = NULL;
 
 	str_tracking = str_print_tracking_t(&(conn->tracking));
@@ -461,15 +457,19 @@ gint print_connection(gpointer data, gpointer userdata)
 		str_iface = g_strdup("");
 	}
 
-	if (conn->packet_id) {
-		str_id = g_strdup_printf(", packet_id=%d",
-				GPOINTER_TO_UINT(conn->packet_id->data));
+	if (pid != TRUE) {
+		if (conn->packet_id) {
+			str_id = g_strdup_printf(", packet_id=%d",
+					GPOINTER_TO_UINT(conn->packet_id->data));
+		} else {
+			str_id = g_strdup("");
+		}
 	} else {
-		str_id = g_strdup("");
+		str_id = g_strdup_printf(", packet_id=%d", packet_id);
 	}
 
 	str_mark = g_strdup_printf(", mark=%d", conn->mark);
-	
+
 	if (conn->username) {
 		str_user = g_strdup_printf(", user=%s", conn->username);
 	} else {
@@ -487,17 +487,31 @@ gint print_connection(gpointer data, gpointer userdata)
 	} else {
 		str_app = g_strdup("");
 	}
+	if (conn->app_sig) {
+		str_sig = g_strdup_printf(", appsig=%s", conn->app_sig);
+	} else {
+		str_sig = g_strdup("");
+	}
+
+	if (conn->expire) {
+		str_exptime = g_strdup_printf(", exptime=%d", (int)conn->expire);
+	} else {
+		str_exptime = g_strdup("");
+	}
 
 	message = g_strconcat(prefix, ":", str_tracking, str_state, str_iface,
-			      str_id, str_mark, str_user, str_os, str_app, NULL);
-	g_free(str_tracking);	
+			      str_id, str_mark, str_user, str_os, str_app,
+			      str_sig, str_exptime, NULL);
+	g_free(str_tracking);
 	g_free(str_state);
-	g_free(str_iface);	
-	g_free(str_id);	
-	g_free(str_mark);	
-	g_free(str_user);	
-	g_free(str_os);	
-	g_free(str_app);	
+	g_free(str_iface);
+	g_free(str_id);
+	g_free(str_mark);
+	g_free(str_user);
+	g_free(str_os);
+	g_free(str_app);
+	g_free(str_sig);
+	g_free(str_exptime);
 
 	g_message("%s", message);
 
@@ -505,6 +519,21 @@ gint print_connection(gpointer data, gpointer userdata)
 	return 1;
 }
 
+/**
+ * Display connection parameters using g_message(): IP+TCP/UDP headers,
+ * OS name, OS release and OS version, and application name.
+ *
+ * Only display the connection if ::debug_level is #DEBUG_LEVEL_VERBOSE_DEBUG
+ * or greater.
+ *
+ * \return Returns -1 if an error occurs, 1 else.
+ */
+
+gint print_connection(gpointer data, gpointer userdata)
+{
+	return print_connection_wid(data, userdata, FALSE, 0);
+}
+
 /**
  * @}
  */
