@@ -1,11 +1,11 @@
 /*
 ** Copyright(C) 2005,2006,2007 Eric Leblond <regit@inl.fr>
 **
-** $Id: connections.h 5196 2008-11-18 15:28:35Z regit $
+** $Id$
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation, version 2 of the License.
+** the Free Software Foundation, version 3 of the License.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -38,10 +38,10 @@ typedef enum {
 	AUTH_STATE_USERPCKT,	/*!< Connection received from an user: see user_request() */
 	AUTH_STATE_READY,	/*!< (see search_and_fill_completing()) */
 
-	/**
-	 * State used when a connection is send to acl_checkers queue: read ACLs
-	 * from cache or external source. See acl_check_and_decide().
-	 */
+    /**
+     * State used when a connection is send to acl_checkers queue: read ACLs
+     * from cache or external source. See acl_check_and_decide().
+     */
 	AUTH_STATE_COMPLETING,
 	AUTH_STATE_DONE,	/*!< This state is set when the connection will be only used for logging purpose */
 	AUTH_STATE_HELLOMODE,	/*!< This connection is treated by the HELLO authentication mode */
@@ -72,6 +72,16 @@ typedef enum {
  */
 #define IPHDR_REJECT_LENGTH_BWORD 5
 
+typedef enum {
+	AUTHQ_NONE,
+	AUTHQ_BYIP,
+	AUTHQ_HELLO,
+	AUTHQ_SASL,
+	AUTHQ_SSL,
+	AUTHQ_SSLHARD,
+	AUTHQ_MAX = AUTHQ_SSLHARD,
+} auth_quality_t;
+
 /**
  * Used to store the acl that apply for a packet
  */
@@ -82,6 +92,7 @@ struct acl_group {
 	gchar *period;		/*!< Period linked to the acl */
 	gchar *log_prefix;	/*!< Log prefix used for the acl */
 	gint flags;		/*!< flags used to set some acl properties */
+	gint auth_quality;	/*!< Minimum authentication quality required for a match */
 };
 
 typedef struct {
@@ -91,6 +102,16 @@ typedef struct {
 	char physoutdev[IFNAMSIZ];	/*!< Output physical device set to "\0" if not available */
 } iface_nfo_t;
 
+typedef struct {
+	tracking_t header;
+	iface_nfo_t iface_nfo;
+} auth_pckt_t;
+
+/**
+ * \brief Size of payload we keep for parsing (must be > headers)
+ */
+#define STORED_PAYLOAD_SIZE	128
+
 /**
  * This is a packet blocked by NuFW and waiting for an authentication
  * of NuAuth. They are created in authpckt_new_connection().
@@ -111,11 +132,11 @@ typedef struct {
 	uint32_t mark;		/*!< Number used for marking set to user numeric identity at start */
 	char *username;		/*!< User name */
 
-	/**
-	 * ACL related groups.
-	 *
-	 * Contains the list of acl corresponding to the IPv4 header
-	 */
+ /**
+  * ACL related groups.
+  *
+  * Contains the list of acl corresponding to the IPv4 header
+  */
 	GSList *acl_groups;	/*!< ACL group list (of type ::acl_group) */
 	GSList *user_groups;	/*!< User groups */
 	struct user_cached_datas *cacheduserdatas;	/* Pointer to cache */
@@ -124,7 +145,7 @@ typedef struct {
 	gchar *os_release;	/*!< Operating system release */
 	gchar *os_version;	/*!< Operating system version */
 	gchar *app_name;	/*!< Application name (full path) */
-	gchar *app_md5;		/*!< Application binary MD5 checksum */
+	gchar *app_sig;		/*!< Application hash (SHA1 to SHA512) */
 
 	auth_state_t state;	/*!< State of the packet */
 
@@ -136,31 +157,38 @@ typedef struct {
 
 
 	int nufw_version;	/*!< Store the version of the nufw server which has sent the request */
-	int client_version;	/*!< Store version of the client which has sent the packet */
+	int proto_version;	/*!< Store protocol version of the client which has sent the packet */
+
+	int auth_quality;
+
 #ifdef PERF_DISPLAY_ENABLE
 	struct timeval arrival_time;	/*!< Performance datas */
 #endif
+
+	unsigned char payload[STORED_PAYLOAD_SIZE];
+	unsigned int payload_len;
+
 } connection_t;
 
 guint hash_connection(gconstpointer conn_p);
 /** hash table containing the connections. */
 GHashTable *conn_list;
 /** global lock for the conn list. */
-extern GStaticMutex insert_mutex;
+GStaticMutex insert_mutex;
 
 
+void duplicate_iface_nfo(iface_nfo_t * copy, iface_nfo_t * orig);
 nu_error_t compare_iface_nfo_t(iface_nfo_t *a, iface_nfo_t *b);
 
 gboolean get_old_conn(gpointer key, gpointer value, gpointer user_data);
-gboolean compare_connection(gconstpointer conn1, gconstpointer conn2);
 int conn_cl_remove(gconstpointer conn);
 int conn_cl_delete(gconstpointer conn);
 nu_error_t print_tracking_t(tracking_t *tracking);
 gint print_connection(gpointer data, gpointer userdata);
+gint print_connection_wid(gpointer data, gpointer userdata, gboolean pid, uint32_t pckt_id);
 void free_connection_list(GSList * list);
 connection_t *duplicate_connection(connection_t * element);
 void free_connection(connection_t * conn);
-void duplicate_iface_nfo(iface_nfo_t * copy, iface_nfo_t * orig);
 int lock_and_free_connection(connection_t * conn);
 void clean_connections_list();
 
