@@ -1,12 +1,12 @@
 /*
- ** Copyright(C) 2005-2008 INL
+ ** Copyright(C) 2005-2007 INL
  ** Written by Eric Leblond <regit@inl.fr>
  **
- ** $Id: conntrack.c 4948 2008-10-07 14:20:12Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -295,16 +295,15 @@ void *limited_connection_handler(GMutex * mutex)
 #ifdef DEBUG_ENABLE
 			else {
 				log_message(VERBOSE_DEBUG, DEBUG_AREA_PACKET,
-					    "connection not found: can not be destroyed");
+					    "connection not found can not be destroyed");
 			}
 #endif
 			g_free(message->datas);
 			break;
 
 		case UPDATE_MESSAGE:
-			/* here we get message from nufw kernel connection is ASSURED
-			 * we have to limit it if needed and log the state change if needed
-			 */
+		/** here we get message from nufw kernel connection is ASSURED
+                 * we have to limit it if needed and log the state change if needed */
 			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_GW,
 					  "received update message for a conntrack entry");
 			elt =
