@@ -1,11 +1,11 @@
 /*
- ** Copyright(C) 2004 Eric Leblond <regit@inl.fr>
- **
- ** $Id: gcrypt.c 2895 2007-03-05 16:49:03Z regit $
+ ** Copyright(C) 2004-2009 INL
+ ** Written by  Eric Leblond <regit@inl.fr>
+ **             Pierre Chifflier <chifflier@inl.fr>
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,14 +18,16 @@
  **
  ** Changelog :
  **  20/06/2005 : deal with seeded/unseeded cases. Patch from Julian Reich <jreich@epplehaus.de>
+ **  02/06/2009 : use hash functions from NuSSL. Recode hash format
  **
  */
 #include <auth_srv.h>
 #include <tls.h>
-#include <gcrypt.h>
 #include <sasl/saslutil.h>
 #include "../include/security.h"
 
+#include <nussl_hash.h>
+
 /**
  * \ingroup TLSUser
  * @{
@@ -41,12 +43,17 @@
  */
 int verify_user_password(const char *given, const char *ours)
 {
-	gcry_md_hd_t hd;
-	char *res;
+	int ret;
 	char *decoded;
-	size_t len;
+	unsigned int decoded_len;
 	char **splitted_secret;
 	int algo = 0;
+	int seeded = 0;
+	char seed[NUSSL_HASH_MAX_SIZE];
+	unsigned int seed_len;
+	char res[NUSSL_HASH_MAX_SIZE];
+	size_t res_len;
+
 
 	if (g_str_has_prefix(ours, "{")) {
 		splitted_secret = g_strsplit(ours, "}", 2);
@@ -63,20 +70,18 @@ int verify_user_password(const char *given, const char *ours)
 			}
 		}
 
-		int seeded = 0;
-
 		if (!(strcmp("{SSHA", splitted_secret[0]))) {	/* SHA1 (seeded) */
-			algo = GCRY_MD_SHA1;
+			algo = NUSSL_HASH_SHA1;
 			seeded = 1;
 		} else if (!(strcmp("{SMD5", splitted_secret[0]))) {	/* MD5 (seeded) */
-			algo = GCRY_MD_MD5;
+			algo = NUSSL_HASH_MD5;
 			seeded = 1;
 		} else if (!(strcmp("{SHA1", splitted_secret[0])))	/* SHA1 */
-			algo = GCRY_MD_SHA1;
+			algo = NUSSL_HASH_SHA1;
 		else if (!(strcmp("{SHA", splitted_secret[0])))	/* SHA1 */
-			algo = GCRY_MD_SHA1;
+			algo = NUSSL_HASH_SHA1;
 		else if (!(strcmp("{MD5", splitted_secret[0])))	/* MD5 */
-			algo = GCRY_MD_MD5;
+			algo = NUSSL_HASH_MD5;
 		else {
 			log_message(WARNING, DEBUG_AREA_AUTH,
 				    "verify_user_password() : Unsupported hash algorithm");
@@ -84,40 +89,29 @@ int verify_user_password(const char *given, const char *ours)
 			return SASL_BADAUTH;
 		}
 
-
-		gcry_md_open(&hd, algo, 0);
-		gcry_md_write(hd, given, strlen(given));
-
-		if ((algo == GCRY_MD_SHA1) && seeded) {
-			char temp[24];
-
-			if (sasl_decode64
-			    (splitted_secret[1],
-			     strlen(splitted_secret[1]), temp,
-			     sizeof(temp), &len) != SASL_OK) {
+		if (seeded) {
+			/* get seed */
+			if (sasl_decode64(splitted_secret[1], strlen(splitted_secret[1]),
+					seed, sizeof(seed), &seed_len) != SASL_OK) {
 				log_message(INFO, DEBUG_AREA_AUTH,
-					    "sasl_decode64 failed in gcrypt.c, where seeded SHA1 is used");
-				return (SASL_BADAUTH);
+					    "sasl_decode64 failed in gcrypt.c, where seeded is used");
+				g_strfreev(splitted_secret);
+				return SASL_BADAUTH;
 			}
-			gcry_md_write(hd, temp + 20, 4);
-		} else if ((algo == GCRY_MD_MD5) && seeded) {
-			char temp[20];
 
-			if (sasl_decode64
-			    (splitted_secret[1],
-			     strlen(splitted_secret[1]), temp,
-			     sizeof(temp), &len) != SASL_OK) {
-				log_message(INFO, DEBUG_AREA_AUTH,
-					    "sasl_decode64 failed in gcrypt.c, where seeded MD5 is used");
-				return (SASL_BADAUTH);
-			}
-			gcry_md_write(hd, temp + 16, 4);
+			ret = nussl_hash_compute_with_salt(algo, given, strlen(given), seed, seed_len, res, &res_len);
+		} else {
+			ret = nussl_hash_compute(algo, given, strlen(given), res, &res_len);
 		}
 
-		res = (char *) gcry_md_read(hd, algo);
 		/* alloc decoded to reasonnable length */
 		decoded = g_new0(char, 50);
-		sasl_encode64(res, strlen(res), decoded, 50, &len);
+		if (sasl_encode64(res, res_len, decoded, 50, &decoded_len) != SASL_OK) {
+				log_message(INFO, DEBUG_AREA_AUTH,
+					    "sasl_encode64 failed in gcrypt.c");
+				g_strfreev(splitted_secret);
+				return SASL_BADAUTH;
+		}
 
 		/* convert password from utf-8 to locale */
 		if (nuauthconf->uses_utf8) {
@@ -136,53 +130,22 @@ int verify_user_password(const char *given, const char *ours)
 			}
 		}
 		debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
-				  "given %s, hash %s, decoded %s, stored : %s",
-				  given, res, decoded, ours);
+				  "given %s, decoded %s, stored : %s",
+				  given, decoded, ours);
 
-		if (!seeded && !strcmp(decoded, splitted_secret[1])) {
-			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
-					  "%s == %s", decoded,
-					  splitted_secret[1]);
-
-			g_free(decoded);
-			g_strfreev(splitted_secret);
-			return SASL_OK;
-		} else if (seeded && (algo == GCRY_MD_SHA1)
-			   && !memcmp(decoded, splitted_secret[1], 20)) {
+		if (memcmp(decoded, splitted_secret[1+seeded], decoded_len) == 0) {
 #ifdef DEBUG_ENABLE
 			if (DEBUG_OR_NOT
 			    (DEBUG_LEVEL_VERBOSE_DEBUG, DEBUG_AREA_MAIN)) {
-				char temp_decoded[21];
-				char temp_stored[21];
-
-				SECURE_STRNCPY(temp_decoded, decoded,
-					       sizeof(temp_decoded));
-				SECURE_STRNCPY(temp_stored,
-					       splitted_secret[1],
-					       sizeof(temp_stored));
-
-				g_message("%s == %s (first 20 chars)",
-					  temp_decoded, temp_stored);
-			}
-#endif
-
-			g_strfreev(splitted_secret);
-			g_free(decoded);
-			return SASL_OK;
-		} else if (seeded && (algo == GCRY_MD_MD5)
-			   && !memcmp(decoded, splitted_secret[1], 16)) {
-#ifdef DEBUG_ENABLE
-			if (DEBUG_OR_NOT
-			    (DEBUG_LEVEL_VERBOSE_DEBUG, DEBUG_AREA_MAIN)) {
-				char temp_decoded[17];
-				char temp_stored[17];
+				char temp_decoded[NUSSL_HASH_MAX_SIZE];
+				char temp_stored[NUSSL_HASH_MAX_SIZE];
 
 				SECURE_STRNCPY(temp_decoded, decoded,
 					       sizeof(temp_decoded));
 				SECURE_STRNCPY(temp_stored, decoded,
 					       sizeof(temp_stored));
 
-				g_message("%s == %s (first 16 chars)",
+				g_message("%s == %s",
 					  temp_decoded, temp_stored);
 			}
 #endif
@@ -191,7 +154,8 @@ int verify_user_password(const char *given, const char *ours)
 			g_free(decoded);
 			return SASL_OK;
 		} else {
-			log_message(DEBUG, DEBUG_AREA_AUTH, "given != stored");
+			log_message(DEBUG, DEBUG_AREA_AUTH, "given (%s) != stored (%s)",
+				decoded, splitted_secret[1+seeded]);
 
 			g_strfreev(splitted_secret);
 			g_free(decoded);
@@ -232,6 +196,8 @@ int verify_user_password(const char *given, const char *ours)
 			return SASL_BADAUTH;
 		}
 	}
+
+	return SASL_BADAUTH;
 }
 
 /* @} */
