@@ -2,11 +2,11 @@
  ** Copyright(C) 2004 INL
  ** Written by Eric Leblond <regit@inl.fr>
  **
- ** $Id: ip_auth.c 5223 2008-11-19 14:15:21Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -59,10 +59,12 @@ void external_ip_auth(gpointer userdata, gpointer data)
 			connection->username = username;
 			connection->os_sysname = NULL;
 			connection->app_name = NULL;
+			connection->app_sig = NULL;
 			connection->flags = ACL_FLAGS_NONE;
+			connection->auth_quality = AUTHQ_BYIP;
 			/* copy ipv4 header */
 			memcpy(&(connection->tracking),
-			       (tracking_t *) userdata,
+			       &((auth_pckt_t *) userdata)->header,
 			       sizeof(tracking_t));
 			g_async_queue_push(nuauthdatas->connections_queue,
 					   connection);
