@@ -1,11 +1,10 @@
 /*
-** Copyright(C) 2005 Eric Leblond <regit@inl.fr>
-**
-** $Id: localid_auth.h 2738 2007-02-17 13:59:56Z regit $
+** Copyright(C) 2005,2009 INL
+** Written by Eric Leblond <eleblond@inl.fr>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation, version 2 of the License.
+** the Free Software Foundation, version 3 of the License.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,7 +20,7 @@
 #define LOCALID_AUTH_H
 
 /* from localid_auth */
-char localid_authenticated_protocol(int protocol);
+char localid_authenticated_protocol(connection_t *conn);
 void *localid_auth(GMutex * mutex);
 
 #endif
