@@ -1,12 +1,13 @@
 /*
- ** Copyright(C) 2005-2008 INL
+ ** Copyright(C) 2005,2006,2007,2008 INL
  ** Written by  Eric Leblond <regit@inl.fr>
+ **             Pierre Chifflier <chifflier@inl.fr>
  **
- ** $Id: modules.c 5470 2008-12-31 11:13:39Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -59,6 +60,8 @@
 #include <auth_srv.h>
 #include "modules_definition.h"
 
+#include "nuauthconf.h"
+
 /** This is a static variable to initiate all pointers to zero */
 static hook_t hooks[MOD_END] = {
 	{ "nuauth_user_check_module", NULL, NULL, "user_check", "user checking" },
@@ -73,6 +76,7 @@ static hook_t hooks[MOD_END] = {
 	{ "nuauth_periods_module", NULL, NULL, "define_periods", "define periods checking" },
 	{ "nuauth_certificate_check_module", NULL, NULL,  "certificate_check", "certificate check" },
 	{ "nuauth_certificate_to_uid_module", NULL, NULL, "certificate_to_uid", "certificate to uid" },
+	{ "nuauth_postauth_proto_module", NULL, NULL, "postauth_proto", "post auth proto" },
 	{ "nuauth_ip_authentication_module", NULL, NULL, "ip_authentication", "ip authentication" },
 };
 
@@ -81,7 +85,7 @@ static hook_t hooks[MOD_END] = {
  *  It returns the decision using SASL defined return value.
  */
 int modules_user_check(const char *user, const char *pass,
-		       unsigned passlen)
+		       unsigned passlen, user_session_t *session)
 {
 	/* iter through module list and stop when user is found */
 	GSList *walker = hooks[MOD_USER_CHECK].modules;
@@ -92,6 +96,7 @@ int modules_user_check(const char *user, const char *pass,
 		    (*(user_check_callback *)
 		     (((module_t *) walker->data))->func) (user, pass,
 							   passlen,
+							   session,
 							   ((module_t *)
 							    walker->data)->
 							   params);
@@ -174,7 +179,7 @@ GSList *modules_acl_check(connection_t * element)
 }
 
 /* ip auth */
-gchar *modules_ip_auth(tracking_t * header)
+gchar *modules_ip_auth(auth_pckt_t * header)
 {
 	/* iter through module list and stop when decision is made */
 	GSList *walker = hooks[MOD_IP_AUTH].modules;
@@ -259,11 +264,10 @@ void modules_parse_periods(GHashTable * periods)
 /**
  * Check certificate
  *
- * \param session TLS connection
- * \param cert x509 certificate
+ * \param nussl NuSSL connection
  * \return SASL_OK if certificate is correct
  */
-int modules_check_certificate(gnutls_session session, gnutls_x509_crt cert)
+int modules_check_certificate(nussl_session* nussl)
 {
 	/* iter through all modules list */
 	GSList *walker = hooks[MOD_CERT_CHECK].modules;
@@ -276,9 +280,7 @@ int modules_check_certificate(gnutls_session session, gnutls_x509_crt cert)
 		certificate_check_callback *handler =
 		    (certificate_check_callback *) ((module_t *) walker->
 						    data)->func;
-		ret =
-		    handler(session, cert,
-			    ((module_t *) walker->data)->params);
+		ret = handler(nussl, ((module_t *) walker->data)->params);
 		if (ret != SASL_OK) {
 			return ret;
 		}
@@ -289,12 +291,10 @@ int modules_check_certificate(gnutls_session session, gnutls_x509_crt cert)
 /**
  * certificate to uid
  *
- * \param session TLS connection
- * \param cert x509 certificate
+ * \param nussl NuSSL connection
  * \return uid
  */
-gchar *modules_certificate_to_uid(gnutls_session session,
-				  gnutls_x509_crt cert)
+gchar *modules_certificate_to_uid(nussl_session* nussl)
 {
 	/* iter through all modules list */
 	GSList *walker = hooks[MOD_CERT_TO_UID].modules;
@@ -305,9 +305,7 @@ gchar *modules_certificate_to_uid(gnutls_session session,
 		certificate_to_uid_callback *handler =
 		    (certificate_to_uid_callback *) ((module_t *) walker->
 						     data)->func;
-		uid =
-		    handler(session, cert,
-			    ((module_t *) walker->data)->params);
+		uid = handler(nussl, ((module_t *) walker->data)->params);
 		if (uid) {
 			return uid;
 		}
@@ -377,6 +375,29 @@ void modules_auth_error_log(user_session_t * session,
 	}
 }
 
+
+/**
+ * custom modification of post authentication exchange
+ */
+int modules_postauth_proto(user_session_t * user)
+{
+	/* iter through all modules list */
+	GSList *walker = hooks[MOD_POSTAUTH_PROTO].modules;
+	int ret;
+
+	block_on_conf_reload();
+	for (; walker != NULL; walker = walker->next) {
+		postauth_proto_callback *handler =
+		    (postauth_proto_callback *) ((module_t *) walker->
+						    data)->func;
+		ret = handler(user, ((module_t *) walker->data)->params);
+		if (ret != SASL_OK) {
+			return ret;
+		}
+	}
+	return SASL_OK;
+}
+
 void clean_module_t(module_t *module)
 {
 	if (module) {
@@ -522,11 +543,12 @@ static int load_modules_from(gchar * confvar, gchar * func,
 			    current_module->module_name,
 			    current_module->configfile);
 		if (current_module->module == NULL) {
-			g_warning("Unable to load module %s in %s\nError: %s",
-				  modules_list[i], MODULE_PATH,
-				  g_module_error ());
+			log_message(FATAL, DEBUG_AREA_MAIN,
+				    "Unable to load module %s in %s\nError: %s",
+				    modules_list[i], MODULE_PATH,
+				    g_module_error ());
 			free_module_t(current_module);
-			return 0;
+			exit(EXIT_FAILURE);
 		}
 
 		/* check module version */
@@ -539,11 +561,12 @@ static int load_modules_from(gchar * confvar, gchar * func,
 		if (!g_module_symbol
 		    (current_module->module, func,
 		     (gpointer *) & current_module->func)) {
-			g_warning("Unable to load function %s in %s", func,
-				  g_module_name(current_module->module));
+			log_message(FATAL, DEBUG_AREA_MAIN,
+				    "Unable to load function %s in %s", func,
+				    g_module_name(current_module->module));
 			free_module_t(current_module);
 			g_strfreev(params_list);
-			return 0;
+			exit(EXIT_FAILURE);
 		}
 
 		current_module->hook = hook;
@@ -591,93 +614,70 @@ static int load_modules_from(gchar * confvar, gchar * func,
 	return 1;
 }
 
+static char *module_default_value(int type)
+{
+	switch (type) {
+		case MOD_USER_CHECK:
+		case MOD_USER_ID:
+		case MOD_USER_GROUPS:
+			return DEFAULT_USERAUTH_MODULE;
+			break;
+		case MOD_USER_FAIL:
+		case MOD_POSTAUTH_PROTO:
+			return "";
+			break;
+		case MOD_ACL_CHECK:
+			return DEFAULT_ACLS_MODULE;
+			break;
+		case MOD_SESSION_MODIFY:
+			return DEFAULT_USER_SESSION_MODIFY_MODULE;
+			break;
+		case MOD_LOG_PACKETS:
+		case MOD_LOG_SESSION:
+			return DEFAULT_LOGS_MODULE;
+			break;
+		case MOD_FINALIZE_PACKET:
+			return DEFAULT_FINALIZE_PACKET_MODULE;
+			break;
+		case MOD_PERIOD:
+			return DEFAULT_PERIODS_MODULE;
+			break;
+		case MOD_CERT_CHECK:
+			return DEFAULT_CERTIFICATE_CHECK_MODULE;
+			break;
+		case MOD_CERT_TO_UID:
+			return DEFAULT_CERTIFICATE_TO_UID_MODULE;
+			break;
+		case MOD_IP_AUTH:
+			return DEFAULT_IPAUTH_MODULE;
+			break;
+		default:
+			return NULL;
+	}
+}
+
 /**
  * Load modules for user and acl checking as well as for user logging and ip authentication
  */
 int load_modules()
 {
-	confparams_t nuauth_vars[] = {
-		{"nuauth_user_check_module", G_TOKEN_STRING, 1,
-		 g_strdup(DEFAULT_USERAUTH_MODULE)},
-		{"nuauth_acl_check_module", G_TOKEN_STRING, 1,
-		 g_strdup(DEFAULT_ACLS_MODULE)},
-		{"nuauth_periods_module", G_TOKEN_STRING, 1,
-		 g_strdup(DEFAULT_PERIODS_MODULE)},
-		{"nuauth_user_logs_module", G_TOKEN_STRING, 1,
-		 g_strdup(DEFAULT_LOGS_MODULE)},
-		{"nuauth_user_session_logs_module", G_TOKEN_STRING, 1,
-		 g_strdup(DEFAULT_LOGS_MODULE)},
-		{"nuauth_ip_authentication_module", G_TOKEN_STRING, 1,
-		 g_strdup(DEFAULT_IPAUTH_MODULE)},
-		{"nuauth_certificate_check_module", G_TOKEN_STRING, 1,
-		 NULL},
-		{"nuauth_certificate_to_uid_module", G_TOKEN_STRING, 1,
-		 g_strdup(DEFAULT_CERTIFICATE_TO_UID_MODULE)},
-		{"nuauth_user_session_modify_module", G_TOKEN_STRING, 1,
-		 g_strdup(DEFAULT_USER_SESSION_MODIFY_MODULE)},
-		{"nuauth_finalize_packet_module", G_TOKEN_STRING, 1,
-		 g_strdup(DEFAULT_FINALIZE_PACKET_MODULE)},
-		{"nuauth_auth_error_log_module", G_TOKEN_STRING, 1,
-		 g_strdup("")}
-	};
 	int i;
-	char *configfile = DEFAULT_CONF_FILE;
-
-	/* parse conf file */
-	if (!parse_conffile(configfile,
-		       sizeof(nuauth_vars) / sizeof(confparams_t),
-		       nuauth_vars)) {
-	        log_message(FATAL, DEBUG_AREA_MAIN, "Failed to load config file %s", configfile);
-		return 0;
-	}
-
-#define READ_CONF(KEY) \
-	get_confvar_value(nuauth_vars, sizeof(nuauth_vars)/sizeof(confparams_t), KEY);
 
 	hooks[MOD_USER_CHECK].config =
-		(char *) READ_CONF(hooks[MOD_USER_CHECK].configstring);
-	for (i = MOD_SIMPLE; i < MOD_OPTIONNAL; i++) {
-		hooks[i].config = (char *) READ_CONF(hooks[i].configstring);
+		nuauth_config_table_get_or_default(hooks[MOD_USER_CHECK].configstring, module_default_value(MOD_USER_CHECK));
+	for (i = MOD_SIMPLE; i < MOD_OPTIONAL; i++) {
+		hooks[i].config = nuauth_config_table_get_or_default(hooks[i].configstring, module_default_value(i));
 	}
 
 	if (nuauthconf->do_ip_authentication) {
 		hooks[MOD_IP_AUTH].config =
-			(char *) READ_CONF(hooks[MOD_IP_AUTH].configstring);
+			nuauth_config_table_get_or_default(hooks[MOD_IP_AUTH].configstring, module_default_value(MOD_IP_AUTH));
 	}
 
-	/* free config struct */
-	free_confparams(nuauth_vars,
-			sizeof(nuauth_vars) / sizeof(confparams_t));
+	/* MOD_USER_CHECK is *always* set to something */
+	hooks[MOD_USER_ID].config = nuauth_config_table_get_or_default(hooks[MOD_USER_ID].configstring, hooks[MOD_USER_CHECK].config);
+	hooks[MOD_USER_GROUPS].config = nuauth_config_table_get_or_default(hooks[MOD_USER_GROUPS].configstring, hooks[MOD_USER_CHECK].config);
 
-
-#undef READ_CONF
-#define READ_CONF(KEY) \
-	get_confvar_value(deps_check_vars, sizeof(deps_check_vars)/sizeof(confparams_t), KEY);
-	if (hooks[MOD_USER_CHECK].config) {
-		confparams_t deps_check_vars[] = {
-			{"nuauth_get_user_groups_module", G_TOKEN_STRING, 1,
-				g_strdup(hooks[MOD_USER_CHECK].config)},
-			{"nuauth_get_user_id_module", G_TOKEN_STRING, 1,
-				g_strdup(hooks[MOD_USER_CHECK].config)},
-		};
-
-		/* parse conf file for user_check sub vars*/
-		if (!parse_conffile(configfile,
-				sizeof(deps_check_vars) / sizeof(confparams_t),
-				deps_check_vars)) {
-		        log_message(FATAL, DEBUG_AREA_MAIN, "Failed to load config file %s", configfile);
-			return 0;
-		}
-
-		hooks[MOD_USER_ID].config =
-			(char *) READ_CONF(hooks[MOD_USER_ID].configstring);
-		hooks[MOD_USER_GROUPS].config =
-			(char *) READ_CONF(hooks[MOD_USER_GROUPS].configstring);
-		/* free config struct */
-		free_confparams(deps_check_vars,
-				sizeof(deps_check_vars) / sizeof(confparams_t));
-
-	}
 	/* external auth module loading */
 	g_mutex_lock(modules_mutex);
 
@@ -691,7 +691,7 @@ int load_modules()
 
 
 	/* loading modules */
-	for (i = MOD_FIRST; i < MOD_OPTIONNAL; i++) {
+	for (i = MOD_FIRST; i < MOD_OPTIONAL; i++) {
 		LOAD_MODULE(i);
 	}
 
@@ -774,4 +774,59 @@ void block_on_conf_reload()
 	g_mutex_unlock(nuauthdatas->reload_cond_mutex);
 }
 
+/**
+ * \brief Register client capabilities (for plugin)
+ */
+
+nu_error_t register_client_capa(const char * name, unsigned int * index)
+{
+	int i;
+
+	for (i = 0; i < 32; i++) {
+		if (! capa_array[i]) {
+			capa_array[i] = g_strdup(name);
+			*index = i;
+			return NU_EXIT_OK;
+		}
+
+	}
+
+	return NU_EXIT_ERROR;
+}
+
+/**
+ * \brief Unregister client capabilities (for plugin)
+ */
+
+nu_error_t unregister_client_capa(int index)
+{
+	g_free(capa_array[index]);
+	capa_array[index] = NULL;
+
+	return NU_EXIT_OK;
+}
+
+/*
+ * protocol extension handling
+ */
+
+nu_error_t init_protocol_extension(struct nuauth_datas * ndatas)
+{
+	INIT_LLIST_HEAD(&(ndatas->ext_proto_l));
+	return NU_EXIT_OK;
+}
+
+nu_error_t register_protocol_extension(struct nuauth_datas * ndatas, struct proto_ext_t *extproto)
+{
+	INIT_LLIST_HEAD(&(extproto->list));
+	llist_add(&(ndatas->ext_proto_l), &(extproto->list));
+	return NU_EXIT_OK;
+}
+
+nu_error_t unregister_protocol_extension(struct proto_ext_t *extproto)
+{
+	llist_del(&(extproto->list));
+	return NU_EXIT_OK;
+}
+
 /* @} */
