@@ -1,13 +1,13 @@
 /*
  ** Copyright(C) 2003-2008 INL
- **     written by Eric Leblond <eric@regit.org>
+ **     written by Eric Leblond <eric@inl.fr>
  **                Vincent Deffontaines <vincent@gryzor.com>
  **
- ** $Id: ldap.c 5479 2009-01-10 13:02:31Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,6 +22,9 @@
 #include <auth_srv.h>
 #include <auth_ldap.h>
 #include "security.h"
+#include "nubase.h"
+
+#include "nuauthconf.h"
 
 /**
  * \ingroup NuauthModules
@@ -111,10 +114,8 @@ int number_add(number_t number, digit_t value)
 
 /**
  * Convert a "Base 10^n" number to decimal string.
- * \param number a number
- * \param ipaddr pre-allocated string that whill contain the stringified IP
- * 
- * \return NU_EXIT_ERROR on error, NU_EXIT_OK on success
+ *
+ * \return Returns new allocated string
  */
 static nu_error_t number_to_decimal(number_t number, char *ipaddr)
 {
@@ -137,24 +138,22 @@ static nu_error_t number_to_decimal(number_t number, char *ipaddr)
 	return NU_EXIT_OK;
 }
 
-#if SASL_VERSION_FULL > 0x020116
+
 static void ldap_conn_destroy(void * connection)
 {
 	if (connection) {
 		ldap_unbind_ext_s(connection, NULL, NULL);
 	}
 }
-#endif
-
 
 
 G_MODULE_EXPORT gboolean unload_module_with_params(gpointer params_p)
 {
 	struct ldap_params *params = (struct ldap_params *) params_p;
 	if (params) {
-		g_free(params->ldap_server);
 		g_free(params->binddn);
 		g_free(params->bindpasswd);
+		g_free(params->ldap_server);
 		g_free(params->ldap_acls_base_dn);
 		g_free(params->ldap_users_base_dn);
 	}
@@ -167,127 +166,33 @@ G_MODULE_EXPORT gboolean unload_module_with_params(gpointer params_p)
  */
 G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 {
-	char *configfile = NULL;
-	gpointer vpointer;
 	struct ldap_params *params = g_new0(struct ldap_params, 1);
-	gchar *ldap_base_dn = g_strdup(LDAP_BASE);
-	confparams_t ldap_nuauth_vars[] = {
-		{"ldap_server_addr", G_TOKEN_STRING, 0, g_strdup(LDAP_SERVER)},
-		{"ldap_server_port", G_TOKEN_INT, LDAP_SERVER_PORT, NULL},
-		{"ldap_base_dn", G_TOKEN_STRING, 0, g_strdup(LDAP_BASE)},
-		{"ldap_users_base_dn", G_TOKEN_STRING, 0, g_strdup(LDAP_BASE)},
-		{"ldap_acls_base_dn", G_TOKEN_STRING, 0, g_strdup(LDAP_BASE)},
-		{"ldap_bind_dn", G_TOKEN_STRING, 0, g_strdup(LDAP_USER)},
-		{"ldap_bind_password", G_TOKEN_STRING, 0, g_strdup(LDAP_CRED)},
-		{"ldap_request_timeout", G_TOKEN_INT, LDAP_REQUEST_TIMEOUT,
-		 NULL},
-		{"ldap_use_ipv4_schema", G_TOKEN_INT, 1, NULL},
-		{"ldap_filter_type", G_TOKEN_INT, 1, NULL}
-	};
-
+	char *ldap_base_dn = LDAP_BASE;
 
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "Ldap module ($Revision: 5479 $)");
-	if (!module->configfile) {
-		configfile = DEFAULT_CONF_FILE;
-	} else {
-		configfile = module->configfile;
-	}
+		    "Ldap module ($Revision$)");
 
-
-	/* parse conf file */
-	parse_conffile(configfile,
-		       sizeof(ldap_nuauth_vars) / sizeof(confparams_t),
-		       ldap_nuauth_vars);
 	/* set variables */
-	vpointer =
-	    get_confvar_value(ldap_nuauth_vars,
-			      sizeof(ldap_nuauth_vars) /
-			      sizeof(confparams_t), "ldap_server_addr");
-	params->ldap_server =
-	    (char *) (vpointer ? vpointer : params->ldap_server);
-	vpointer =
-	    get_confvar_value(ldap_nuauth_vars,
-			      sizeof(ldap_nuauth_vars) /
-			      sizeof(confparams_t), "ldap_server_port");
-	params->ldap_server_port =
-	    *(int *) (vpointer ? vpointer : &params->ldap_server_port);
-	vpointer =
-	    get_confvar_value(ldap_nuauth_vars,
-			      sizeof(ldap_nuauth_vars) /
-			      sizeof(confparams_t), "ldap_bind_dn");
-	params->binddn = (char *) (vpointer ? vpointer : params->binddn);
-	vpointer =
-	    get_confvar_value(ldap_nuauth_vars,
-			      sizeof(ldap_nuauth_vars) /
-			      sizeof(confparams_t), "ldap_base_dn");
-	ldap_base_dn = (char *) (vpointer ? vpointer : ldap_base_dn);
-	vpointer =
-	    get_confvar_value(ldap_nuauth_vars,
-			      sizeof(ldap_nuauth_vars) /
-			      sizeof(confparams_t), "ldap_users_base_dn");
-	params->ldap_users_base_dn =
-	    (char *) (vpointer ? vpointer : params->ldap_users_base_dn);
-	vpointer =
-	    get_confvar_value(ldap_nuauth_vars,
-			      sizeof(ldap_nuauth_vars) /
-			      sizeof(confparams_t), "ldap_acls_base_dn");
-	params->ldap_acls_base_dn =
-	    (char *) (vpointer ? vpointer : params->ldap_acls_base_dn);
-
+	params->ldap_server = nuauth_config_table_get_or_default("ldap_server_addr", LDAP_SERVER);
+	params->ldap_server_port = nuauth_config_table_get_or_default_int("ldap_server_port", LDAP_SERVER_PORT);
+	params->binddn = nuauth_config_table_get_or_default("ldap_bind_dn",LDAP_USER);
+	ldap_base_dn = nuauth_config_table_get_or_default("ldap_base_dn",LDAP_BASE);
+	params->ldap_users_base_dn = nuauth_config_table_get_or_default("ldap_users_base_dn",LDAP_BASE);
+	params->ldap_acls_base_dn = nuauth_config_table_get_or_default("ldap_acls_base_dn",LDAP_BASE);
 	if (!strcmp(params->ldap_acls_base_dn, LDAP_BASE)) {
-		params->ldap_acls_base_dn = g_strdup(ldap_base_dn);
+		params->ldap_acls_base_dn = ldap_base_dn;
 	}
 	if (!strcmp(params->ldap_users_base_dn, LDAP_BASE)) {
-		params->ldap_users_base_dn = g_strdup(ldap_base_dn);
+		params->ldap_users_base_dn = ldap_base_dn;
 	}
-
-	vpointer =
-	    get_confvar_value(ldap_nuauth_vars,
-			      sizeof(ldap_nuauth_vars) /
-			      sizeof(confparams_t), "ldap_bind_password");
-	params->bindpasswd =
-	    (char *) (vpointer ? vpointer : params->bindpasswd);
-	params->ldap_request_timeout = LDAP_REQUEST_TIMEOUT;
-	vpointer =
-	    get_confvar_value(ldap_nuauth_vars,
-			      sizeof(ldap_nuauth_vars) /
-			      sizeof(confparams_t), "ldap_request_timeout");
-	params->ldap_request_timeout =
-	    *(int *) (vpointer ? vpointer : &params->ldap_request_timeout);
-
-	vpointer =
-	    get_confvar_value(ldap_nuauth_vars,
-			      sizeof(ldap_nuauth_vars) /
-			      sizeof(confparams_t), "ldap_use_ipv4_schema");
-	params->ldap_use_ipv4_schema =
-	    *(int *) (vpointer ? vpointer : &params->ldap_use_ipv4_schema);
-
-	vpointer =
-	    get_confvar_value(ldap_nuauth_vars,
-			      sizeof(ldap_nuauth_vars) /
-			      sizeof(confparams_t), "ldap_filter_type");
-	params->ldap_filter_type =
-	    *(int *) (vpointer ? vpointer : &params->ldap_filter_type);
-
-
-	/* free config struct */
-
-	g_free(ldap_base_dn);
-
-	free_confparams(ldap_nuauth_vars,
-			sizeof(ldap_nuauth_vars) / sizeof(confparams_t));
+	params->bindpasswd = nuauth_config_table_get_or_default("ldap_bind_password",LDAP_CRED);
+	params->ldap_request_timeout = nuauth_config_table_get_or_default_int("ldap_request_timeout",LDAP_REQUEST_TIMEOUT);
+	params->ldap_use_ipv4_schema = nuauth_config_table_get_or_default_int("ldap_use_ipv4_schema", 1);
+	params->ldap_filter_type = nuauth_config_table_get_or_default_int("ldap_filter_type", 1);
 
 
 	/* init thread private stuff */
-	/* cyrus-sasl is buggy in version <= to 2.1.22
-	* See http://archives.free.net.ph/message/20080920.124141.b3d1fc95.fr.html
-	*/
-#if SASL_VERSION_FULL > 0x020116
 	params->ldap_priv = g_private_new((GDestroyNotify) ldap_conn_destroy);
-#else
-	params->ldap_priv = g_private_new(NULL);
-#endif
 
 	module->params = params;
 
@@ -329,6 +234,7 @@ static LDAP *ldap_conn_init(struct ldap_params *params)
 			    &version) == LDAP_OPT_SUCCESS) {
 		/* Goes to ssl if needed */
 
+#if 0
 #ifdef LDAP_OPT_X_TLS
 		if (params->ldap_server_port == LDAPS_PORT) {
 			int tls_option;
@@ -343,6 +249,7 @@ static LDAP *ldap_conn_init(struct ldap_params *params)
 			}
 		}
 #endif /* LDAP_OPT_X_TLS */
+#endif
 
 		password.bv_val = params->bindpasswd;
 		password.bv_len = strlen(password.bv_val);
@@ -382,6 +289,69 @@ static nu_error_t ipv6_to_base10(struct in6_addr *addr, gchar *ipaddr)
 	return number_to_decimal(number, ipaddr);
 }
 
+
+/**
+ * \brief Escape character to protect them in query
+ *
+ * \verbatim
+Abstract from RFC 2254
+	Character       ASCII value
+	---------------------------
+	*               0x2a
+	(               0x28
+	)               0x29
+	\               0x5c
+	NUL             0x00
+For example * is coded \2a
+\endverbatim
+ *
+ * \param basestring the string to convert
+ * \return a newly allocated string
+ */
+
+gchar *escape_string_for_ldap(const gchar * basestring)
+{
+	int length = strlen(basestring) + 1;
+	gchar *result = g_new0(gchar, length);
+	const gchar *c_char = basestring;
+	int i = 0;
+
+	while (*c_char) {
+		switch (*c_char) {
+		case '*':
+			length += 2;
+			result = g_realloc(result, length);
+			g_strlcat(result, "\\2a", length);
+			i += 3;
+			break;
+		case '(':
+			length += 2;
+			result = g_realloc(result, length);
+			g_strlcat(result, "\\28", length);
+			i += 3;
+			break;
+		case ')':
+			length += 2;
+			result = g_realloc(result, length);
+			g_strlcat(result, "\\29", length);
+			i += 3;
+			break;
+		case '\\':
+			length += 2;
+			result = g_realloc(result, length);
+			g_strlcat(result, "\\5c", length);
+			i += 3;
+			break;
+		default:
+			result[i] = *c_char;
+			i++;
+		}
+		c_char++;
+	}
+	result[length - 1] = 0;
+	return result;
+}
+
 struct weighted_acl {
 	struct acl_group *acl;
 	int weight;
@@ -389,8 +359,8 @@ struct weighted_acl {
 
 gint compare_acl_weight(gconstpointer data1, gconstpointer data2)
 {
-	return ((struct weighted_acl *)data2)->weight
-		- ((struct weighted_acl *)data1)->weight;
+	return ((struct weighted_acl *)data1)->weight
+		- ((struct weighted_acl *)data2)->weight;
 }
 
 static void local_free(gpointer data, gpointer userdata)
@@ -454,25 +424,25 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 	struct timeval timeout;
 	struct acl_group *this_acl;
 	struct weighted_acl *this = NULL;
-	LDAPMessage *res = NULL;
-	LDAPMessage *result;
-	int err, try;
+	LDAPMessage *res, *result;
+	int ok, err, try;
 	struct ldap_params *params = (struct ldap_params *) params_p;
 	LDAP *ld = g_private_get(params->ldap_priv);
 	gchar ip_src[IPADDR_STRLEN];
 	gchar ip_dst[IPADDR_STRLEN];
 
 	if (params->ldap_use_ipv4_schema) {
+		struct in_addr ipv4;
 		if (!is_ipv4(&element->tracking.saddr) ||
 				!is_ipv4(&element->tracking.daddr)) {
 			log_message(SERIOUS_WARNING, DEBUG_AREA_AUTH,
 				    "ldap: IPv4 schema but IPv6 address\n");
 			return NULL;
 		}
-		snprintf(ip_src, IPADDR_STRLEN, "%u",
-				ntohl(element->tracking.saddr.s6_addr32[3]));
-		snprintf(ip_dst, IPADDR_STRLEN, "%u",
-				ntohl(element->tracking.daddr.s6_addr32[3]));
+		ipv6_to_ipv4(&element->tracking.saddr, &ipv4);
+		snprintf(ip_src, IPADDR_STRLEN, "%u", ipv4.s_addr);
+		ipv6_to_ipv4(&element->tracking.daddr, &ipv4);
+		snprintf(ip_dst, IPADDR_STRLEN, "%u", ipv4.s_addr);
 	} else {
 		if (ipv6_to_base10(&element->tracking.saddr, ip_src) ==
 				NU_EXIT_ERROR) {
@@ -489,19 +459,17 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 	    || (element->tracking).protocol == IPPROTO_UDP) {
 		switch (params->ldap_filter_type) {
 		case 1:
-			if (snprintf(filter, LDAP_QUERY_SIZE - 1,
-#if USE_SOURCE_PORT
-				     "(&(objectClass=NuAccessControlList)(Proto=%d)(DstPort=%d)(SrcIPStart<=%s)(SrcIPEnd>=%s)(DstIPStart<=%s)(DstIPEnd>=%s)(SrcPortStart<=%d)(SrcPortEnd>=%d)",
-#endif
-				     "(&(objectClass=NuAccessControlList)(Proto=%d)(DstPort=%d)(SrcIPStart<=%s)(SrcIPEnd>=%s)(DstIPStart<=%s)(DstIPEnd>=%s)",
-				     (element->tracking).protocol,
-				     (element->tracking).dest,
-				     ip_src, ip_src, ip_dst, ip_dst
-#if USE_SOURCE_PORT
-				     , (element->tracking).source,
-				     (element->tracking).source
-#endif
-			    ) >= (LDAP_QUERY_SIZE - 1)) {
+			ok = secure_snprintf(filter, sizeof(filter),
+				"(&(objectClass=NuAccessControlList)"
+				"(Proto=%d)"
+				"(DstPort=%d)"
+				"(SrcIPStart<=%s)(SrcIPEnd>=%s)"
+				"(DstIPStart<=%s)(DstIPEnd>=%s)",
+				element->tracking.protocol,
+				element->tracking.dest,
+				ip_src, ip_src,
+				ip_dst, ip_dst);
+			if (!ok) {
 				log_message(WARNING, DEBUG_AREA_MAIN,
 					    "LDAP query too big (more than %d bytes)\n",
 					    LDAP_QUERY_SIZE);
@@ -509,22 +477,17 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 			}
 			break;
 		case 0:
-			if (snprintf(filter, LDAP_QUERY_SIZE - 1,
-#if USE_SOURCE_PORT
-				     "(&(objectClass=NuAccessControlList)(SrcIPStart<=%s)(SrcIPEnd>=%s)(DstIPStart<=%s)(DstIPEnd>=%s)(Proto=%d)(SrcPortStart<=%d)(SrcPortEnd>=%d)(DstPortStart<=%d)(DstPortEnd>=%d)",
-#endif
-				     "(&(objectClass=NuAccessControlList)(SrcIPStart<=%s)(SrcIPEnd>=%s)(DstIPStart<=%s)(DstIPEnd>=%s)(Proto=%d)(DstPortStart<=%d)(DstPortEnd>=%d)",
-				     ip_src,
-				     ip_src,
-				     ip_dst,
-				     ip_dst, (element->tracking).protocol,
-#if USE_SOURCE_PORT
-				     (element->tracking).source,
-				     (element->tracking).source,
-#endif
-				     (element->tracking).dest,
-				     (element->tracking).dest) >=
-			    (LDAP_QUERY_SIZE - 1)) {
+			ok = secure_snprintf(filter, sizeof(filter),
+				"(&(objectClass=NuAccessControlList)"
+				"(SrcIPStart<=%s)(SrcIPEnd>=%s)"
+				"(DstIPStart<=%s)(DstIPEnd>=%s)"
+				"(Proto=%d)"
+				"(DstPortStart<=%d)(DstPortEnd>=%d)",
+				ip_src, ip_src,
+				ip_dst,	ip_dst,
+				element->tracking.protocol,
+				element->tracking.dest, element->tracking.dest);
+			if (!ok) {
 				log_message(WARNING, DEBUG_AREA_MAIN,
 					    "LDAP query too big (more than %d bytes)\n",
 					    LDAP_QUERY_SIZE);
@@ -534,21 +497,26 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 
 		/* finish filter */
 		if (! element->os_sysname) {
-			g_strlcat(filter, "(!(OsName=*))",
-				  LDAP_QUERY_SIZE);
+			g_strlcat(filter, "(!(OsName=*))", LDAP_QUERY_SIZE);
 		}
 		if (! element->os_release) {
-			g_strlcat(filter, "(!(OsRelease=*))",
-				  LDAP_QUERY_SIZE);
+			g_strlcat(filter, "(!(OsRelease=*))", LDAP_QUERY_SIZE);
 		}
 		if (! element->os_version) {
-			g_strlcat(filter, "(!(OsVersion=*))",
-				  LDAP_QUERY_SIZE);
+			g_strlcat(filter, "(!(OsVersion=*))", LDAP_QUERY_SIZE);
 		}
 		if (! element->app_name) {
 			g_strlcat(filter, "(!(AppName=*))", LDAP_QUERY_SIZE);
 		}
 
+		if (! element->app_sig) {
+			g_strlcat(filter, "(!(AppSig=*))", LDAP_QUERY_SIZE);
+		} else {
+			gchar iffilter[256];
+			g_snprintf(iffilter, 256, "(|(AppSig=%s)(!(AppSig=*)))", element->app_sig);
+			g_strlcat(filter, iffilter, LDAP_QUERY_SIZE);
+		}
+
 		if (element->iface_nfo.indev[0] == '\0') {
 			g_strlcat(filter, "(!(InDev=*))", LDAP_QUERY_SIZE);
 		} else {
@@ -580,26 +548,25 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 		}
 
 		g_strlcat(filter, ")", LDAP_QUERY_SIZE);
-
 		debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
 				  "LDAP filter : \n%s\n", filter);
 
 	} else if ((element->tracking).protocol == IPPROTO_ICMP) {
-		if (snprintf(filter, LDAP_QUERY_SIZE - 1,
-			     "(&(objectClass=NuAccessControlList)"
-			     "(SrcIPStart<=%s)(SrcIPEnd>=%s)"
-			     "(DstIPStart<=%s)(DstIPEnd>=%s)"
-			     "(Proto=%d)"
-			     "(SrcPortStart<=%d)(SrcPortEnd>=%d)"
-			     "(DstPortStart<=%d)(DstPortEnd>=%d))",
-			     ip_src, ip_src,
-			     ip_dst, ip_dst,
-			     (element->tracking).protocol,
-			     (element->tracking).type,
-			     (element->tracking).type,
-			     (element->tracking).code,
-			     (element->tracking).code) >=
-		    (LDAP_QUERY_SIZE - 1)) {
+		ok = secure_snprintf(filter, sizeof(filter),
+			"(&(objectClass=NuAccessControlList)"
+			"(SrcIPStart<=%s)(SrcIPEnd>=%s)"
+			"(DstIPStart<=%s)(DstIPEnd>=%s)"
+			"(Proto=%d)"
+			"(SrcPortStart<=%d)(SrcPortEnd>=%d)"
+			"(DstPortStart<=%d)(DstPortEnd>=%d))",
+			ip_src, ip_src,
+			ip_dst, ip_dst,
+			element->tracking.protocol,
+			element->tracking.type,
+			element->tracking.type,
+			element->tracking.code,
+			element->tracking.code);
+		if (!ok) {
 			log_message(WARNING, DEBUG_AREA_MAIN,
 				    "LDAP query too big (more than %d bytes)\n",
 				    LDAP_QUERY_SIZE);
@@ -647,33 +614,18 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 		}
 #endif
 		if (err != LDAP_SUCCESS) {
-			switch (err) {
-				case LDAP_SERVER_DOWN:
-					/* we lost connection, so disable current one */
-					log_message(WARNING, DEBUG_AREA_MAIN,
-							"disabling current connection");
-					ldap_unbind_ext_s(ld, NULL, NULL);
-					ld = NULL;
-					g_private_set(params->ldap_priv, ld);
-					break;
-				case LDAP_NO_SUCH_OBJECT:
-					debug_log_message(DEBUG, DEBUG_AREA_MAIN, "no ldap object");
-					ldap_msgfree(res);
-					return NULL;
-				default:
-					log_message(INFO, DEBUG_AREA_MAIN,
-						    "ldap search failed: %s",
-						    ldap_err2string(err));
-					if (res) {
-						debug_log_message(DEBUG, DEBUG_AREA_MAIN,
-								  "freeing allocated ldap result");
-						ldap_msgfree(res);
-						res = NULL;
-					}
-					break;
+			if (err == LDAP_SERVER_DOWN) {
+				/* we lost connection, so disable current one */
+				log_message(WARNING, DEBUG_AREA_MAIN,
+						"disabling current connection");
+				ldap_unbind_ext_s(ld, NULL, NULL);
+				ld = NULL;
+				g_private_set(params->ldap_priv, ld);
+			} else {
+				break;
 			}
 		}
-	} while ((err != LDAP_SUCCESS) && (try < LDAP_MAX_TRY));
+	} while ((err != LDAP_SUCCESS) || (try < LDAP_MAX_TRY));
 
 	if ((try == LDAP_MAX_TRY) && (err != LDAP_SUCCESS)) {
 		log_message(WARNING, DEBUG_AREA_MAIN,
@@ -696,12 +648,10 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 				case NU_EXIT_ERROR: \
 					log_message(WARNING, DEBUG_AREA_MAIN, \
 						    "Invalid return from field_match_pattern"); \
-					ldap_msgfree(res); \
 					return NULL; \
 				default: \
 					log_message(WARNING, DEBUG_AREA_MAIN, \
 						    "Impossible return from field_match_pattern"); \
-					ldap_msgfree(res); \
 					return NULL; \
 					break; \
 			} \
@@ -718,29 +668,16 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 
 			/* allocate a new acl_group */
 			this_acl = g_new0(struct acl_group, 1);
-			if (this_acl == NULL) {
-				log_message(WARNING, DEBUG_AREA_MAIN,
-						"Unable to allocate new acl");
-				g_slist_free(g_list);
-				ldap_msgfree(res);
-				return NULL;
-			}
 			if (nuauthconf->prio_to_nok == 2) {
 				this = g_new0(struct weighted_acl, 1);
-				if (this == NULL) {
-					log_message(WARNING, DEBUG_AREA_MAIN,
-						    "Unable to allocate new weighted_acl");
-					g_free(this_acl);
-					g_slist_free(g_list);
-					ldap_msgfree(res);
-					return NULL;
-				}
 			}
+			g_assert(this_acl);
 			this_acl->users = NULL;
 			this_acl->groups = NULL;
 			this_acl->period = NULL;
 			this_acl->log_prefix = NULL;
 			this_acl->flags = ACL_FLAGS_NONE;
+			this_acl->auth_quality = 0;
 
 			/* get period */
 			attrs_array = ldap_get_values_len(ld, result, "TimeRange");
@@ -763,6 +700,14 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 			}
 			ldap_value_free_len(attrs_array);
 
+			/* get auth quality */
+			attrs_array = ldap_get_values_len(ld, result, "AuthQuality");
+			if (attrs_array && *attrs_array) {
+				sscanf((*attrs_array)->bv_val, "%d", (int *) &(this_acl->auth_quality));
+			}
+			ldap_value_free_len(attrs_array);
+
+
 			if (nuauthconf->prio_to_nok == 2) {
 				/* get weight */
 				attrs_array = ldap_get_values_len(ld, result, "AclWeight");
@@ -823,12 +768,6 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 					g_list = g_slist_prepend(g_list, this_acl);
 				}
 			} else {
-				g_free(this_acl->period);
-				g_free(this_acl->log_prefix);
-				if (this_acl->users)
-					g_slist_free(this_acl->users);
-				if (this_acl->groups)
-					g_slist_free(this_acl->groups);
 				g_free(this_acl);
 				if (nuauthconf->prio_to_nok == 2) {
 					g_free(this);
@@ -860,3 +799,5 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 
 /* @} */
 
+
+
