@@ -4,11 +4,11 @@
  **	       Vincent Deffontaines <vincent@gryzor.com>
  **	       Victor Stinner <haypo@inl.fr>
  **
- ** $Id: mysql.c 5230 2008-11-19 15:00:25Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,6 +26,10 @@
 #include <errno.h>
 #include <inttypes.h>
 
+#include "nuauthconf.h"
+
+#include "strings.h"
+
 /** Minimum buffer size to write an IPv6 in SQL syntax */
 #define IPV6_SQL_STRLEN (2+16*2+1)
 
@@ -193,114 +197,39 @@ static nu_error_t mysql_close_open_user_sessions(struct log_mysql_params
 /* Init mysql system */
 G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 {
-	confparams_t mysql_nuauth_vars[] = {
-		{"mysql_server_addr", G_TOKEN_STRING, 0,
-		 g_strdup(MYSQL_SERVER)}
-		,
-		{"mysql_server_port", G_TOKEN_INT, MYSQL_SERVER_PORT, NULL}
-		,
-		{"mysql_user", G_TOKEN_STRING, 0, g_strdup(MYSQL_USER)}
-		,
-		{"mysql_passwd", G_TOKEN_STRING, 0, g_strdup(MYSQL_PASSWD)}
-		,
-		{"mysql_db_name", G_TOKEN_STRING, 0,
-		 g_strdup(MYSQL_DB_NAME)}
-		,
-		{"mysql_table_name", G_TOKEN_STRING, 0,
-		 g_strdup(MYSQL_TABLE_NAME)}
-		,
-		{"mysql_users_table_name", G_TOKEN_STRING, 0,
-		 g_strdup(MYSQL_USERS_TABLE_NAME)}
-		,
-		{"mysql_request_timeout", G_TOKEN_INT,
-		 MYSQL_REQUEST_TIMEOUT, NULL}
-		,
-		{"mysql_use_ipv4_schema", G_TOKEN_INT,
-		 MYSQL_USE_IPV4_SCHEMA, NULL}
-		,
-		{"mysql_admin_bofh", G_TOKEN_INT,
-		 0, NULL}
-		,
-		{"mysql_bofh_victime_group", G_TOKEN_INT,
-		 0, NULL}
-		,
-		{"mysql_prefix_version", G_TOKEN_INT, PREFIX_VERSION_ORIG, NULL}
-		,
-		{"mysql_use_ssl", G_TOKEN_INT, MYSQL_USE_SSL, NULL}
-		,
-		{"mysql_ssl_keyfile", G_TOKEN_STRING, 0,
-		 g_strdup(MYSQL_SSL_KEYFILE)}
-		,
-		{"mysql_ssl_certfile", G_TOKEN_STRING, 0,
-		 g_strdup(MYSQL_SSL_CERTFILE)}
-		,
-		{"mysql_ssl_ca", G_TOKEN_STRING, 0, g_strdup(MYSQL_SSL_CA)}
-		,
-		{"mysql_ssl_capath", G_TOKEN_STRING, 0,
-		 g_strdup(MYSQL_SSL_CAPATH)}
-		,
-		{"mysql_ssl_cipher", G_TOKEN_STRING, 0,
-		 g_strdup(MYSQL_SSL_CIPHER)}
-	};
-	char *configfile = DEFAULT_CONF_FILE;
 	/* char *ldap_base_dn=LDAP_BASE; */
 	struct log_mysql_params *params =
 	    g_new0(struct log_mysql_params, 1);
 
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "Log_mysql module ($Revision: 5230 $)");
+		    "Log_mysql module ($Revision$)");
 	/* init global variables */
 	params->mysql_ssl_cipher = MYSQL_SSL_CIPHER;
 	params->hook = module->hook;
 
-	/* parse conf file */
-	if (module->configfile) {
-		parse_conffile(module->configfile,
-			       sizeof(mysql_nuauth_vars) /
-			       sizeof(confparams_t), mysql_nuauth_vars);
-	} else {
-		parse_conffile(configfile,
-			       sizeof(mysql_nuauth_vars) /
-			       sizeof(confparams_t), mysql_nuauth_vars);
-	}
 	/* set variables */
-
-#define READ_CONF(KEY) \
-	get_confvar_value(mysql_nuauth_vars, sizeof(mysql_nuauth_vars)/sizeof(confparams_t), KEY)
-#define READ_CONF_INT(VAR, KEY, DEFAULT) \
-	do { gpointer vpointer = READ_CONF(KEY); if (vpointer) VAR = *(int *)vpointer; else VAR = DEFAULT; } while (0)
-
-	params->mysql_server = (char *) READ_CONF("mysql_server_addr");
-	params->mysql_user = (char *) READ_CONF("mysql_user");
-	params->mysql_passwd = (char *) READ_CONF("mysql_passwd");
-	params->mysql_db_name = (char *) READ_CONF("mysql_db_name");
-	params->mysql_table_name = (char *) READ_CONF("mysql_table_name");
+	params->mysql_server = nuauth_config_table_get_or_default("mysql_server_addr", MYSQL_SERVER);
+	params->mysql_user = nuauth_config_table_get_or_default("mysql_user", MYSQL_USER);
+	params->mysql_passwd = nuauth_config_table_get_or_default("mysql_passwd", MYSQL_PASSWD);
+	params->mysql_db_name = nuauth_config_table_get_or_default("mysql_db_name", MYSQL_DB_NAME);
+	params->mysql_table_name = nuauth_config_table_get_or_default("mysql_table_name", MYSQL_TABLE_NAME);
 	params->mysql_users_table_name =
-	    (char *) READ_CONF("mysql_users_table_name");
+	    nuauth_config_table_get_or_default("mysql_users_table_name", MYSQL_USERS_TABLE_NAME);
 	params->mysql_ssl_keyfile =
-	    (char *) READ_CONF("mysql_ssl_keyfile");
+	    nuauth_config_table_get_or_default("mysql_ssl_keyfile", MYSQL_SSL_KEYFILE);
 	params->mysql_ssl_certfile =
-	    (char *) READ_CONF("mysql_ssl_certfile");
-	params->mysql_ssl_ca = (char *) READ_CONF("mysql_ssl_ca");
-	params->mysql_ssl_capath = (char *) READ_CONF("mysql_ssl_capath");
-	params->mysql_ssl_cipher = (char *) READ_CONF("mysql_ssl_cipher");
+	    nuauth_config_table_get_or_default("mysql_ssl_certfile", MYSQL_SSL_CERTFILE);
+	params->mysql_ssl_ca = nuauth_config_table_get_or_default("mysql_ssl_ca", MYSQL_SSL_CA);
+	params->mysql_ssl_capath = nuauth_config_table_get_or_default("mysql_ssl_capath", MYSQL_SSL_CAPATH);
+	params->mysql_ssl_cipher = nuauth_config_table_get_or_default("mysql_ssl_cipher", MYSQL_SSL_CIPHER);
 
-	READ_CONF_INT(params->mysql_server_port, "mysql_server_port",
-		      MYSQL_SERVER_PORT);
-	READ_CONF_INT(params->mysql_request_timeout,
-		      "mysql_request_timeout", MYSQL_REQUEST_TIMEOUT);
-	READ_CONF_INT(params->mysql_use_ssl, "mysql_use_ssl",
-		      MYSQL_USE_SSL);
-	READ_CONF_INT(params->mysql_use_ipv4_schema,
-		      "mysql_use_ipv4_schema", MYSQL_USE_IPV4_SCHEMA);
-	READ_CONF_INT(params->mysql_admin_bofh, "mysql_admin_bofh", 0);
-	READ_CONF_INT(params->mysql_prefix_version, "mysql_prefix_version", PREFIX_VERSION_NULOG2);
-	READ_CONF_INT(params->mysql_bofh_victim_group, "mysql_bofh_victim_group", 0);
-
-
-	/* free config struct */
-	free_confparams(mysql_nuauth_vars,
-			sizeof(mysql_nuauth_vars) / sizeof(confparams_t));
+	params->mysql_server_port = nuauth_config_table_get_or_default_int("mysql_server_port", MYSQL_SERVER_PORT);
+	params->mysql_request_timeout = nuauth_config_table_get_or_default_int("mysql_request_timeout", MYSQL_REQUEST_TIMEOUT);
+	params->mysql_use_ssl = nuauth_config_table_get_or_default_int("mysql_use_ssl", MYSQL_USE_SSL);
+	params->mysql_use_ipv4_schema = nuauth_config_table_get_or_default_int("mysql_use_ipv4_schema", MYSQL_USE_IPV4_SCHEMA);
+	params->mysql_admin_bofh = nuauth_config_table_get_or_default_int("mysql_admin_bofh", 0);
+	params->mysql_prefix_version = nuauth_config_table_get_or_default_int("mysql_prefix_version", PREFIX_VERSION_NULOG2); /* XXX: Was previously initialized as PREFIX_VERSION_ORIG*/
+	params->mysql_bofh_victim_group = nuauth_config_table_get_or_default_int("mysql_bofh_victim_group", 0);
 
 	if (params->mysql_admin_bofh) {
 		if (nuauthconf->single_user_client_limit !=  1 ) {
@@ -741,6 +670,12 @@ static inline int log_state_established(MYSQL * ld,
 		if (mysql_affected_rows(ld) >= 1) {
 			return 0;
 		} else {
+			/* If logging pool is full then waiting for entry will slow down
+			 * recovery by a huge time (and entry has probably to update has
+			 * probably not been updated. */
+			if (nuauthdatas->loggers_pool_full == TRUE) {
+				return 0;
+			}
 			if (update_status < 2) {
 				/* Sleep for 1/3 sec */
 				struct timespec sleep;
@@ -816,6 +751,12 @@ static inline int log_state_close(MYSQL * ld,
 	if (mysql_affected_rows(ld) >= 1) {
 		return 0;
 	} else {
+		/* If logging pool is full then waiting for entry will slow down
+		 * recovery by a huge time (and entry has probably to update has
+		 * probably not been updated. */
+		if (nuauthdatas->loggers_pool_full == TRUE) {
+			return 0;
+		}
 		if (update_status < 2) {
 			/* Sleep for 2/3 sec */
 			struct timespec sleep;
@@ -1055,8 +996,6 @@ nu_error_t destroy_user_connections(user_session_t * c_session,
 							  &msgdatas,
 							  params)
 					!= NU_EXIT_OK) {
-				log_message(SERIOUS_WARNING, DEBUG_AREA_MAIN,
-					    "Unable to build conntrack message");
 				mysql_free_result(result);
 				return NU_EXIT_ERROR;
 			}
@@ -1064,8 +1003,6 @@ nu_error_t destroy_user_connections(user_session_t * c_session,
 					(&msgdatas,
 					 AUTH_CONN_DESTROY)
 					!= NU_EXIT_OK) {
-				log_message(SERIOUS_WARNING, DEBUG_AREA_MAIN,
-					    "Unable to send conntrack message");
 				mysql_free_result(result);
 				return NU_EXIT_ERROR;
 			}
@@ -1119,7 +1056,7 @@ G_MODULE_EXPORT int user_session_logs(user_session_t * c_session,
 						"os_sysname, os_release, os_version, socket, start_time) "
 						"VALUES ('%lu', '%s', '%s', '%s', '%s', '%s', '%u', FROM_UNIXTIME(%lu))",
 						params->mysql_users_table_name,
-						c_session->user_id,
+						(unsigned long)c_session->user_id,
 						quoted_username,
 						ip_ascii,
 						quoted_osname,
