@@ -3,11 +3,11 @@
  **	written by Pierre Chifflier <chifflier AT inl DOT fr>
  **	written by Victor Stinner <victor.stinner AT haypocalc.com>
  **
- ** $Id: nuprelude.c 5220 2008-11-19 14:11:55Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,6 +20,7 @@
  */
 
 #include "log_prelude.h"
+#include "strings.h"
 #include <prelude.h>
 #include <prelude-log.h>
 #include <idmef-tree-wrap.h>
@@ -102,7 +103,7 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 
 
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "Log_nuprelude module ($Revision: 5220 $)");
+		    "Log_nuprelude module ($Revision$)");
 
 	params->packet_tpl = g_private_new((GDestroyNotify) destroy_idmef);
 	params->session_tpl = g_private_new((GDestroyNotify) destroy_idmef);
@@ -376,7 +377,7 @@ idmef_message_t* create_from_template(idmef_message_t *tpl, connection_t *conn)
 /**
  * Set operating system information on the client (OS type and version)
  */
-static void set_os_infos(idmef_message_t *idmef, const char* osname, const char *osrelease, const char *osversion)
+static void set_os_info(idmef_message_t *idmef, const char* osname, const char *osrelease, const char *osversion)
 {
 	char buffer[256];
 
@@ -389,7 +390,7 @@ static void set_os_infos(idmef_message_t *idmef, const char* osname, const char
 /**
  * Set information on NuFW (ip address, port)
  */
-static void set_nufw_infos(idmef_message_t *idmef, const char *nufw_address, const char *nufw_port)
+static void set_nufw_info(idmef_message_t *idmef, const char *nufw_address, const char *nufw_port)
 {
 	char buffer[256];
 
@@ -402,7 +403,7 @@ static void set_nufw_infos(idmef_message_t *idmef, const char *nufw_address, con
 void set_source0_address(idmef_message_t *idmef, struct in6_addr *addr)
 {
 	char ip_ascii[INET6_ADDRSTRLEN];
-	FORMAT_IPV6(addr, ip_ascii);
+	format_ipv6(addr, ip_ascii, INET6_ADDRSTRLEN, NULL);
 	add_idmef_object(idmef,
 			 "alert.source(0).node.address(0).address",
 			 ip_ascii);
@@ -444,7 +445,7 @@ static idmef_message_t *create_message_packet(idmef_message_t * tpl,
 
 	/* IP source/dest */
 	set_source0_address(idmef, &conn->tracking.saddr);
-	FORMAT_IPV6(&conn->tracking.daddr, ip_ascii);
+	format_ipv6(&conn->tracking.daddr, ip_ascii, INET6_ADDRSTRLEN, NULL);
 	add_idmef_object(idmef, "alert.target(0).node.address(0).address", ip_ascii);
 
 	/* IP protocol */
@@ -525,13 +526,13 @@ static idmef_message_t *create_message_packet(idmef_message_t * tpl,
 
 	/* os informations */
 	if (conn->os_sysname != NULL) {
-		set_os_infos(idmef, conn->os_sysname, conn->os_release, conn->os_version);
+		set_os_info(idmef, conn->os_sysname, conn->os_release, conn->os_version);
 	}
 
 	/* informations about nufw server */
 	if (conn->tls != NULL) {
-		FORMAT_IPV6(&conn->tls->peername, ip_ascii);
-		set_nufw_infos(idmef, ip_ascii, nuauthconf->authreq_port);
+		format_ipv6(&conn->tls->peername, ip_ascii, INET6_ADDRSTRLEN, NULL);
+		set_nufw_info(idmef, ip_ascii, nuauthconf->authreq_port);
 	}
 
 	return idmef;
@@ -599,11 +600,11 @@ static idmef_message_t *create_message_session(idmef_message_t * tpl,
 	/* set user informations */
 	add_user_information(idmef, session, 1);
 
-	FORMAT_IPV6(&session->server_addr, ip_ascii);
+	format_ipv6(&session->server_addr, ip_ascii, INET6_ADDRSTRLEN, NULL);
 	add_idmef_object(idmef, "alert.target(0).node.address(0).address", ip_ascii);
 
 	/* os informations */
-	set_os_infos(idmef, session->sysname, session->release, session->version);
+	set_os_info(idmef, session->sysname, session->release, session->version);
 	return idmef;
 }
 
@@ -633,7 +634,7 @@ static idmef_message_t *create_message_autherr(idmef_message_t * tpl,
 	secure_snprintf(buffer, sizeof(buffer), "%hu", session->sport);
 	add_idmef_object(idmef,	"alert.source(0).service.port", buffer);
 
-	FORMAT_IPV6(&session->server_addr, ip_ascii);
+	format_ipv6(&session->server_addr, ip_ascii, INET6_ADDRSTRLEN, NULL);
 	add_idmef_object(idmef, "alert.target(0).node.address(0).address", ip_ascii);
 
 	/* set user informations */
