@@ -1,12 +1,12 @@
 /*
- ** Copyright(C) 2003-2006 Eric Leblond <eric@regit.org>
- **		     Vincent Deffontaines <vincent@gryzor.com>
- **                   INL
- ** $Id: pgsql.c 4967 2008-10-14 14:09:06Z pollux $
+ ** Copyright(C) 2003-2010 INL
+ ** Written by Eric Leblond <eleblond@inl.fr>
+ **	       Vincent Deffontaines <vincent@gryzor.com>
+ **	       Pierre Chifflier <chifflier@inl.fr>
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,6 +32,10 @@
 #include <inttypes.h>
 #include "security.h"
 
+#include "nuauthconf.h"
+
+#include "nubase.h"
+
 static nu_error_t pgsql_close_open_user_sessions(struct log_pgsql_params
 						 *params);
 static PGconn *pgsql_conn_init(struct log_pgsql_params *params);
@@ -111,6 +115,7 @@ G_MODULE_EXPORT gboolean unload_module_with_params(gpointer params_p)
 		g_free(params->pgsql_db_name);
 		g_free(params->pgsql_table_name);
 		g_free(params->pgsql_users_table_name);
+		g_free(params->pgsql_auth_failure_table_name);
 	}
 	g_free(params);
 
@@ -166,58 +171,25 @@ static nu_error_t pgsql_close_open_user_sessions(struct log_pgsql_params
 /* Init pgsql system */
 G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 {
-	confparams_t pgsql_nuauth_vars[] = {
-		{"pgsql_server_addr", G_TOKEN_STRING, 0, g_strdup(PGSQL_SERVER)},
-		{"pgsql_server_port", G_TOKEN_INT, PGSQL_SERVER_PORT, NULL},
-		{"pgsql_user", G_TOKEN_STRING, 0, g_strdup(PGSQL_USER)},
-		{"pgsql_passwd", G_TOKEN_STRING, 0, g_strdup(PGSQL_PASSWD)},
-		{"pgsql_ssl", G_TOKEN_STRING, 0, g_strdup(PGSQL_SSL)},
-		{"pgsql_db_name", G_TOKEN_STRING, 0, g_strdup(PGSQL_DB_NAME)},
-		{"pgsql_table_name", G_TOKEN_STRING, 0, g_strdup(PGSQL_TABLE_NAME)},
-		{"pgsql_users_table_name", G_TOKEN_STRING, 0, g_strdup(PGSQL_USERS_TABLE_NAME)},
-		{"pgsql_request_timeout", G_TOKEN_INT, PGSQL_REQUEST_TIMEOUT, NULL},
-		{"pgsql_use_ipv4", G_TOKEN_INT, PGSQL_USE_IPV4, NULL}
-	};
-	unsigned int nb_params =
-	    sizeof(pgsql_nuauth_vars) / sizeof(confparams_t);
 	struct log_pgsql_params *params =
 	    g_new0(struct log_pgsql_params, 1);
 	module->params = params;
 
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "Log_pgsql module ($Revision: 4967 $)");
-	/* parse conf file */
-	if (module->configfile) {
-		parse_conffile(module->configfile, nb_params,
-			       pgsql_nuauth_vars);
-	} else {
-		parse_conffile(DEFAULT_CONF_FILE, nb_params,
-			       pgsql_nuauth_vars);
-	}
+		    "Log_pgsql module ($Revision$)");
 
 	/* set variables */
-#define READ_CONF(KEY) \
-	get_confvar_value(pgsql_nuauth_vars, nb_params, KEY)
-#define READ_CONF_INT(VAR, KEY, DEFAULT) \
-	do { gpointer vpointer = READ_CONF(KEY); if (vpointer) VAR = *(int *)vpointer; else VAR = DEFAULT; } while (0)
-
-	params->pgsql_server = (char *) READ_CONF("pgsql_server_addr");
-	READ_CONF_INT(params->pgsql_server_port, "pgsql_server_port",
-		      PGSQL_SERVER_PORT);
-	params->pgsql_user = (char *) READ_CONF("pgsql_user");
-	params->pgsql_passwd = (char *) READ_CONF("pgsql_passwd");
-	params->pgsql_ssl = (char *) READ_CONF("pgsql_ssl");
-	params->pgsql_db_name = (char *) READ_CONF("pgsql_db_name");
-	params->pgsql_table_name = (char *) READ_CONF("pgsql_table_name");
-	params->pgsql_users_table_name = (char *) READ_CONF("pgsql_users_table_name");
-	READ_CONF_INT(params->pgsql_request_timeout, "pgsql_request_timeout", PGSQL_REQUEST_TIMEOUT);
-	READ_CONF_INT(params->pgsql_use_ipv4, "pgsql_use_ipv4", PGSQL_USE_IPV4);
-
-	/* free config struct */
-	free_confparams(pgsql_nuauth_vars,
-			sizeof(pgsql_nuauth_vars) / sizeof(confparams_t));
-#undef READ_CONF
-#undef READ_CONF_INT
+	params->pgsql_server = nuauth_config_table_get_or_default("pgsql_server_addr", PGSQL_SERVER);
+	params->pgsql_server_port = nuauth_config_table_get_or_default_int("pgsql_server_port", PGSQL_SERVER_PORT);
+	params->pgsql_user = nuauth_config_table_get_or_default("pgsql_user", PGSQL_USER);
+	params->pgsql_passwd = nuauth_config_table_get_or_default("pgsql_passwd", PGSQL_PASSWD);
+	params->pgsql_ssl = nuauth_config_table_get_or_default("pgsql_ssl", PGSQL_SSL);
+	params->pgsql_db_name = nuauth_config_table_get_or_default("pgsql_db_name", PGSQL_DB_NAME);
+	params->pgsql_table_name = nuauth_config_table_get_or_default("pgsql_table_name", PGSQL_TABLE_NAME);
+	params->pgsql_users_table_name = nuauth_config_table_get_or_default("pgsql_users_table_name", PGSQL_USERS_TABLE_NAME);
+	params->pgsql_auth_failure_table_name = nuauth_config_table_get_or_default("pgsql_auth_failure_table_name", PGSQL_AUTH_FAILURE_TABLE_NAME);
+	params->pgsql_request_timeout = nuauth_config_table_get_or_default_int("pgsql_request_timeout", PGSQL_REQUEST_TIMEOUT);
+	params->pgsql_use_ipv4 = nuauth_config_table_get_or_default_int("pgsql_use_ipv4", PGSQL_USE_IPV4);
 
 	/* init thread private stuff */
 	params->pgsql_priv = g_private_new((GDestroyNotify) PQfinish);
@@ -270,18 +242,24 @@ static PGconn *pgsql_conn_init(struct log_pgsql_params *params)
 	return ld;
 }
 
-static char *quote_pgsql_string(char *text)
+static char *quote_pgsql_string(PGconn *ld, char *text)
 {
-	unsigned int length = strlen(text);
-	char *quoted = (char *) malloc(length * 2 + 1);
-	if (PQescapeString(quoted, text, length) == 0) {
+	unsigned int length = 0;
+	char *quoted = NULL;
+
+	if (text == NULL) {
+		return NULL;
+	}
+	length = strlen(text);
+	quoted = g_new0(char, length * 2 + 1);
+	if (PQescapeStringConn(ld, quoted, text, length, NULL) == 0) {
 		g_free(quoted);
 		return NULL;
 	}
 	return quoted;
 }
 
-static gchar *generate_osname(gchar * Name, gchar * Version,
+static gchar *generate_osname(PGconn *ld, gchar * Name, gchar * Version,
 			      gchar * Release)
 {
 	char *all, *quoted;
@@ -291,7 +269,7 @@ static gchar *generate_osname(gchar * Name, gchar * Version,
 		return g_strdup("");
 	}
 	all = g_strjoin("-", Name, Version, Release, NULL);
-	quoted = quote_pgsql_string(all);
+	quoted = quote_pgsql_string(ld,  all);
 	g_free(all);
 	return quoted;
 }
@@ -347,8 +325,8 @@ static int pgsql_insert(PGconn * ld, connection_t * element,
 	if (element->username) {
 		/* Get OS and application names */
 		char *quoted_username =
-		    quote_pgsql_string(element->username);
-		char *quoted_osname = generate_osname(element->os_sysname,
+		    quote_pgsql_string(ld, element->username);
+		char *quoted_osname = generate_osname(ld, element->os_sysname,
 						      element->os_version,
 						      element->os_release);
 		char *quoted_appname;
@@ -356,7 +334,7 @@ static int pgsql_insert(PGconn * ld, connection_t * element,
 		if (element->app_name != NULL
 		    && strlen(element->app_name) < APPNAME_MAX_SIZE)
 			quoted_appname =
-			    quote_pgsql_string(element->app_name);
+			    quote_pgsql_string(ld, element->app_name);
 		else
 			quoted_appname = g_strdup("");
 
@@ -365,7 +343,7 @@ static int pgsql_insert(PGconn * ld, connection_t * element,
 			  ", user_id, username, client_os, client_app",
 			  sizeof(request_fields));
 		ok = secure_snprintf(tmp_buffer, sizeof(tmp_buffer),
-				     ", '%u', '%s', '%s', '%s'",
+				     ", '%u', E'%s', E'%s', E'%s'",
 				     element->user_id,
 				     quoted_username,
 				     quoted_osname, quoted_appname);
@@ -423,6 +401,8 @@ static int pgsql_insert(PGconn * ld, connection_t * element,
 	/* do the query */
 	Result = PQexec(ld, sql_query);
 
+	g_free(sql_query);
+
 	/* check error */
 	if (!Result || PQresultStatus(Result) != PGRES_COMMAND_OK) {
 		log_message(SERIOUS_WARNING, DEBUG_AREA_MAIN,
@@ -489,7 +469,7 @@ static int pgsql_update_state(PGconn * ld,
 	char *ip_src, *ip_dst;
 	int nb_try = 0;
 	int nb_tuple;
-	gboolean ok;
+	gboolean ok = 0;
 
 	/* setup IP/TCP parameters */
 	if (!formatINET(params, tmp_inet1, sizeof(tmp_inet1),
@@ -666,11 +646,17 @@ G_MODULE_EXPORT int user_session_logs(user_session_t * c_session,
 {
 	char request[INSERT_REQUEST_VALUES_SIZE];
 	char addr_ascii[INET6_ADDRSTRLEN];
+	gchar *str_groups;
 	struct log_pgsql_params *params =
 	    (struct log_pgsql_params *) params_p;
 	gboolean ok;
 	PGresult *Result;
 	PGconn *ld = get_pgsql_handler(params);
+	gchar *q_user_name;
+	gchar *q_sysname;
+	gchar *q_release;
+	gchar *q_version;
+
 	if (ld == NULL)
 		return -1;
 
@@ -681,19 +667,32 @@ G_MODULE_EXPORT int user_session_logs(user_session_t * c_session,
 
 	switch (state) {
 	case SESSION_OPEN:
+		/* build list of user groups */
+		str_groups = str_print_group(c_session);
+		/* quote pgsql strings */
+		q_user_name = quote_pgsql_string(ld, c_session->user_name);
+		q_sysname = quote_pgsql_string(ld, c_session->sysname);
+		q_release = quote_pgsql_string(ld, c_session->release);
+		q_version = quote_pgsql_string(ld, c_session->version);
 		/* create new user session */
 		ok = secure_snprintf(request, sizeof(request),
-				     "INSERT INTO %s (user_id, username, ip_saddr, "
+				     "INSERT INTO %s (user_id, username, user_groups, ip_saddr, "
 				     "os_sysname, os_release, os_version, socket, start_time) "
-				     "VALUES ('%lu', '%s', '%s', '%s', '%s', '%s', '%u', ABSTIME(%lu))",
+				     "VALUES ('%lu', E'%s', '%s', '%s', E'%s', E'%s', E'%s', '%u', ABSTIME(%lu))",
 				     params->pgsql_users_table_name,
-				     c_session->user_id,
-				     c_session->user_name,
+				     (unsigned long)c_session->user_id,
+				     q_user_name,
+				     str_groups,
 				     addr_ascii,
-				     c_session->sysname,
-				     c_session->release,
-				     c_session->version,
+				     q_sysname,
+				     q_release,
+				     q_version,
 				     c_session->socket, time(NULL));
+		g_free(q_user_name);
+		g_free(q_sysname);
+		g_free(q_release);
+		g_free(q_version);
+		g_free(str_groups);
 		break;
 
 	case SESSION_CLOSE:
@@ -728,4 +727,126 @@ G_MODULE_EXPORT int user_session_logs(user_session_t * c_session,
 	return 0;
 }
 
+G_MODULE_EXPORT void auth_error_log(user_session_t * c_session,
+				    nuauth_auth_error_t error,
+				    const char *text, gpointer params_p)
+{
+	struct log_pgsql_params *params =
+	    (struct log_pgsql_params *) params_p;
+	char addr_ascii[INET6_ADDRSTRLEN];
+	char request_values[INSERT_REQUEST_VALUES_SIZE];
+	char request_fields[INSERT_REQUEST_FIELDS_SIZE];
+	char tmp_buffer[INSERT_REQUEST_VALUES_SIZE];
+	char * quoted_username = NULL;
+	gchar *str_groups;
+	gchar * sql_query;
+	gboolean ok;
+	PGresult *Result;
+	PGconn *ld = get_pgsql_handler(params);
+	if (ld == NULL)
+		return;
+
+
+	if (!formatINET(params, addr_ascii, sizeof(addr_ascii),
+				&c_session->addr, 0)) {
+		return;
+	}
+
+	quoted_username = quote_pgsql_string(ld, c_session->user_name);
+	/* create new user session */
+	ok = secure_snprintf(request_fields, sizeof(request_fields),
+			"INSERT INTO %s (username, ip_saddr, reason, time, "
+			"sport",
+			params->pgsql_auth_failure_table_name);
+
+	if (!ok) {
+		g_free(quoted_username);
+		return;
+	}
+
+	ok = secure_snprintf(request_values, sizeof(request_values),
+			"VALUES (E'%s', '%s', '%s', ABSTIME(%lu), '%d'",
+			quoted_username,
+			addr_ascii,
+			text,
+			time(NULL),
+			c_session->sport);
+
+	g_free(quoted_username);
+
+	if (!ok) {
+		return;
+	}
+
+	if (c_session->groups) {
+		g_strlcat(request_fields, ",user_id, user_groups",
+				sizeof(request_fields));
+		/* build list of user groups */
+		str_groups = str_print_group(c_session);
+
+		ok = secure_snprintf(tmp_buffer, sizeof(tmp_buffer),
+				     ", '%u', '%s'",
+				     c_session->user_id,
+				     str_groups);
+		g_free(str_groups);
+		if (!ok) {
+			return;
+		}
+		g_strlcat(request_values, tmp_buffer,
+			  sizeof(request_values));
+	}
+
+	if (c_session->sysname) {
+		char * q_sysname = quote_pgsql_string(ld, c_session->sysname);
+		char * q_release = quote_pgsql_string(ld, c_session->release);
+		char * q_version = quote_pgsql_string(ld, c_session->version);
+
+
+		g_strlcat(request_fields, "os_sysname, os_release, os_version)",
+				sizeof(request_fields));
+		ok = secure_snprintf(tmp_buffer, sizeof(tmp_buffer),
+				     ", E'%s', E'%s', E'%s')",
+				     q_sysname,
+				     q_release,
+				     q_version);
+		g_free(q_sysname);
+		g_free(q_release);
+		g_free(q_version);
+
+		if (!ok) {
+			return;
+		}
+		g_strlcat(request_values, tmp_buffer,
+			  sizeof(request_values));
+	} else {
+		g_strlcat(request_fields, ")", sizeof(request_fields));
+		g_strlcat(request_values, ");", sizeof(request_values));
+	}
+
+	/* create the sql query */
+	sql_query =
+	    g_strconcat(request_fields, "\n", request_values, NULL);
+	if (sql_query == NULL) {
+		log_message(SERIOUS_WARNING, DEBUG_AREA_MAIN,
+			    "Fail to build PostgreSQL query (maybe too long)!");
+		return;
+	}
+
+
+	/* do the query */
+	Result = PQexec(ld, sql_query);
+
+	g_free(sql_query);
+	/* check error */
+	if (!Result || PQresultStatus(Result) != PGRES_COMMAND_OK) {
+		log_message(SERIOUS_WARNING, DEBUG_AREA_MAIN,
+			    "[PostgreSQL] Cannot insert session: %s",
+			    PQerrorMessage(ld));
+	}
+	PQclear(Result);
+	return;
+
+
+}
+
 /** @} */
