@@ -5,7 +5,7 @@
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; version 2 of the License.
+** the Free Software Foundation; version 3 of the License.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,7 +30,6 @@ struct plaintext_user {
 
 struct plaintext_app {
 	gchar *appname;
-	gchar *appmd5;
 };
 
 struct plaintext_os {
@@ -73,6 +72,7 @@ struct plaintext_acl {
 	gchar *log_prefix;
 
 	gint flags;
+	gint auth_quality;
 };
 
 struct plaintext_params {
