@@ -3,11 +3,11 @@
  ** Copyright(C) 2004-2005 Mikael Berthe <mikael+nufw@lists.lilotux.net>
  ** Copyright(C) 2005-2008 INL
  **
- ** $Id: plaintext.c 5479 2009-01-10 13:02:31Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,6 +23,8 @@
 #include <string.h>
 #include "auth_plaintext.h"
 
+#include "nuauthconf.h"
+
 /**
  *
  * \ingroup AuthNuauthModules
@@ -272,8 +274,8 @@ static int parse_ips(char *ipsline, GSList ** ip_list, char *prefix)
 		if (compare_ipv6_with_mask
 		    (&this_ip.addr, &this_ip.addr,
 		     &this_ip.netmask) != 0) {
-			FORMAT_IPV6(&this_ip.addr, addr_ascii);
-			FORMAT_IPV6(&this_ip.netmask, mask_ascii);
+			format_ipv6(&this_ip.addr, addr_ascii, INET6_ADDRSTRLEN, NULL);
+			format_ipv6(&this_ip.netmask, mask_ascii, INET6_ADDRSTRLEN, NULL);
 			log_message(FATAL, DEBUG_AREA_MAIN,
 				    "%s parse_ips: Invalid network specification: IP=%s, netmask=%s!",
 				    prefix, addr_ascii, mask_ascii);
@@ -285,8 +287,8 @@ static int parse_ips(char *ipsline, GSList ** ip_list, char *prefix)
 		*ip_list = g_slist_prepend(*ip_list, this_ip_copy);
 
 #ifdef DEBUG_ENABLE
-		FORMAT_IPV6(&this_ip_copy->addr, addr_ascii);
-		FORMAT_IPV6(&this_ip_copy->netmask, mask_ascii);
+		format_ipv6(&this_ip_copy->addr, addr_ascii, INET6_ADDRSTRLEN, NULL);
+		format_ipv6(&this_ip_copy->netmask, mask_ascii, INET6_ADDRSTRLEN, NULL);
 		log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
 				"%s Adding IP = %s, netmask = %s",
 				prefix, addr_ascii,
@@ -321,7 +323,7 @@ static int read_user_list(struct plaintext_params *params)
 
 	if (!fd) {
 		log_message(WARNING, DEBUG_AREA_AUTH,
-			    "read_user_list: fopen error (%s)", params->plaintext_userfile);
+			    "read_user_list: fopen error");
 		return 1;
 	}
 
@@ -442,7 +444,7 @@ static int read_acl_list(struct plaintext_params *params)
 
 	if (!fd) {
 		log_message(FATAL, DEBUG_AREA_MAIN,
-			    "read_acl_list: fopen error (%s)", params->plaintext_aclfile);
+			    "read_acl_list: fopen error");
 		return 1;
 	}
 
@@ -508,6 +510,7 @@ static int read_acl_list(struct plaintext_params *params)
 			newacl->period = NULL;
 			newacl->log_prefix = NULL;
 			newacl->flags = ACL_FLAGS_NONE;
+			newacl->auth_quality = 0;
 			newacl->decision = DECISION_ACCEPT;
 			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
 					  "L.%d: ACL name found: [%s]", ln,
@@ -666,17 +669,7 @@ static int read_acl_list(struct plaintext_params *params)
 					  "L.%d: Read App name [%s]", ln,
 					  newapp->appname);
 
-			/*  MD5: */
-			if (sep) {
-				p_value = sep;
-				newapp->appmd5 =
-				    g_strdup(strip_line(p_value, 0));
-				debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-						  "L.%d: Read App MD5 [%s]",
-						  ln, newapp->appmd5);
-			}
-			newacl->apps =
-			    g_slist_prepend(newacl->apps, newapp);
+			newacl->apps = g_slist_prepend(newacl->apps, newapp);
 		} else if (!strcasecmp("os", p_key)) {	/*  OS */
 			char *sep;
 			struct plaintext_os *newos = g_new0(struct plaintext_os, 1);
@@ -752,6 +745,16 @@ static int read_acl_list(struct plaintext_params *params)
 			}
 			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
 					"L.%d: Read acl flags [%d]", ln, newacl->flags);
+		} else if (!strcasecmp("authquality", p_key)) {
+			if (sscanf(p_value, "%d", &newacl->auth_quality) != 1) {
+				log_message(FATAL, DEBUG_AREA_MAIN,
+						"L.%d: Malformed line (flags should be a number)",
+						ln);
+				fclose(fd);
+				return 2;
+			}
+			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
+					"L.%d: Read acl authquality [%d]", ln, newacl->auth_quality);
 		} else {
 			log_message(SERIOUS_WARNING, DEBUG_AREA_MAIN,
 				    "L.%d: Unknown key [%s] in ACL %s",
@@ -835,7 +838,6 @@ static int read_acl_list(struct plaintext_params *params)
 			     p_app = g_slist_next(p_app)) {
 				struct plaintext_app *app = p_app->data;
 				g_free(app->appname);
-				g_free(app->appmd5);
 				g_free(app);
 			}
 			/*  Free OS attributes */
@@ -884,50 +886,18 @@ static int read_acl_list(struct plaintext_params *params)
 
 G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 {
-	gpointer vpointer;
 	struct plaintext_params *params =
 	    g_new0(struct plaintext_params, 1);
-	confparams_t plaintext_nuauth_vars[] = {
-		{"plaintext_userfile", G_TOKEN_STRING, 0,
-		 g_strdup(TEXplaintext_USERFILE)},
-		{"plaintext_aclfile", G_TOKEN_STRING, 0,
-		 g_strdup(TEXplaintext_ACLFILE)}
-	};
-
 
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "Plaintext module ($Revision: 5479 $)");
-	/*  parse conf file */
-	if (module->configfile) {
-		parse_conffile(module->configfile,
-			       sizeof(plaintext_nuauth_vars) /
-			       sizeof(confparams_t), plaintext_nuauth_vars);
-	} else {
-		parse_conffile(DEFAULT_CONF_FILE,
-			       sizeof(plaintext_nuauth_vars) /
-			       sizeof(confparams_t), plaintext_nuauth_vars);
-	}
+		    "Plaintext module ($Revision$)");
+
 	/*  set variables */
-	vpointer = get_confvar_value(plaintext_nuauth_vars,
-				     sizeof(plaintext_nuauth_vars) /
-				     sizeof(confparams_t),
-				     "plaintext_userfile");
-	params->plaintext_userfile =
-	    (char *) (vpointer ? vpointer : params->plaintext_userfile);
-	vpointer =
-	    get_confvar_value(plaintext_nuauth_vars,
-			      sizeof(plaintext_nuauth_vars) /
-			      sizeof(confparams_t), "plaintext_aclfile");
-	params->plaintext_aclfile =
-	    (char *) (vpointer ? vpointer : params->plaintext_aclfile);
+	params->plaintext_userfile = nuauth_config_table_get_or_default("plaintext_userfile", TEXplaintext_USERFILE);
+	params->plaintext_aclfile = nuauth_config_table_get_or_default("plaintext_aclfile", TEXplaintext_ACLFILE);
 	params->plaintext_userlist = NULL;
 	params->plaintext_acllist = NULL;
 
-	/* free config struct */
-	free_confparams(plaintext_nuauth_vars,
-			sizeof(plaintext_nuauth_vars) /
-			sizeof(confparams_t));
-
 	module->params = (gpointer) params;
 
 	/*  Depending on the use of the module load user list or acl list */
@@ -991,12 +961,14 @@ static GSList *fill_user_by_username(const char *username, gpointer params)
  *  \param username user name string
  *  \param clientpass user provided password
  *  \param passlen password length
+ *  \param session pointer to the user_session_t:: that we working on
  *  \param params module related parameter
  *  \return SASL_OK if password is correct, other values are authentication
  *           failures
  */
 G_MODULE_EXPORT int user_check(const char *username,
 			       const char *clientpass, unsigned passlen,
+			       user_session_t *session,
 			       gpointer params)
 {
 	GSList *res;
@@ -1288,12 +1260,7 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element, gpointer params)
 			    "[plaintext] matching with ACL %s and decision %d",
 			    p_acl->aclname, p_acl->decision);
 		this_acl = g_new0(struct acl_group, 1);
-		if (this_acl == NULL) {
-			log_message(WARNING, DEBUG_AREA_MAIN,
-				    "Unable to allocate new acl");
-			g_slist_free(g_list);
-			return NULL;
-		}
+		g_assert(this_acl);
 		this_acl->answer = p_acl->decision;
 		this_acl->users = g_slist_copy(p_acl->users);
 		this_acl->groups = g_slist_copy(p_acl->groups);
@@ -1308,6 +1275,7 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element, gpointer params)
 			this_acl->log_prefix = NULL;
 		}
 		this_acl->flags = p_acl->flags;
+		this_acl->auth_quality = p_acl->auth_quality;
 
 		g_list = g_slist_append(g_list, this_acl);
 	}
