@@ -1,12 +1,13 @@
 /*
-** Copyright(C) 2006 INL
+** Copyright(C) 2006-2008 INL
 **          written by Eric Leblond <regit@inl.fr>
+**                     Pierre Chifflier <chifflier@inl.fr>
 **
-** $Id: x509_std.c 5409 2008-12-08 15:27:54Z pollux $
+** $Id$
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; version 2 of the License.
+** the Free Software Foundation; version 3 of the License.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -63,35 +64,13 @@ G_MODULE_EXPORT gboolean unload_module_with_params(gpointer params_p)
 
 G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 {
-	confparams_t x509_std_nuauth_vars[] = {
-		{"nuauth_tls_trusted_issuer_dn", G_TOKEN_STRING, 0, NULL}
-	};
-	gpointer vpointer;
 	struct x509_std_params *params = g_new0(struct x509_std_params, 1);
 
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "X509_std module ($Revision: 5409 $)");
+		    "X509_std module ($Revision$)");
 
-	/*  parse conf file */
-	if (module->configfile) {
-		parse_conffile(module->configfile,
-			       sizeof(x509_std_nuauth_vars) /
-			       sizeof(confparams_t), x509_std_nuauth_vars);
-	} else {
-		parse_conffile(DEFAULT_CONF_FILE,
-			       sizeof(x509_std_nuauth_vars) /
-			       sizeof(confparams_t), x509_std_nuauth_vars);
-	}
 	/*  set variables */
-	vpointer = get_confvar_value(x509_std_nuauth_vars,
-				     sizeof(x509_std_nuauth_vars) /
-				     sizeof(confparams_t),
-				     "nuauth_tls_trusted_issuer_dn");
-	params->trusted_issuer_dn = (gchar *) (vpointer);
-
-	/* free config struct */
-	free_confparams(x509_std_nuauth_vars,
-			sizeof(x509_std_nuauth_vars) / sizeof(confparams_t));
+	params->trusted_issuer_dn = nuauth_config_table_get("nauth_tls_trusted_issuer_dn");
 
 	module->params = (gpointer) params;
 
@@ -100,10 +79,10 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 }
 
 
-G_MODULE_EXPORT int certificate_check(gnutls_session session,
-				      gnutls_x509_crt cert,
+G_MODULE_EXPORT int certificate_check(nussl_session* session,
 				      gpointer params_p)
 {
+#if 0
 	struct x509_std_params *params =
 	    (struct x509_std_params *) params_p;
 	time_t expiration_time, activation_time;
@@ -154,12 +133,11 @@ G_MODULE_EXPORT int certificate_check(gnutls_session session,
 			return SASL_DISABLED;
 		}
 	}
-
+#endif
 	return SASL_OK;
 }
 
-G_MODULE_EXPORT gchar *certificate_to_uid(gnutls_session session,
-					  gnutls_x509_crt cert,
+G_MODULE_EXPORT gchar *certificate_to_uid(nussl_session* session,
 					  gpointer params)
 {
 	size_t size;
@@ -168,7 +146,7 @@ G_MODULE_EXPORT gchar *certificate_to_uid(gnutls_session session,
 	gchar *delim;
 
 	size = sizeof(dn);
-	gnutls_x509_crt_get_dn(cert, dn, &size);
+	nussl_get_peer_dn(session, dn, &size);
 
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_USER, "\tDN: %s", dn);
 
