@@ -2,11 +2,11 @@
  ** Copyright(C) 2005-2006 Eric Leblond <regit@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: modules_definition.h 3763 2007-09-25 22:03:22Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,7 +34,7 @@ GMutex *modules_mutex;
 typedef gboolean init_module_from_conf_t(module_t * module);
 
 typedef int user_check_callback(const char *user, const char *pass,
-				unsigned passlen, gpointer params);
+				unsigned passlen, user_session_t *session, gpointer params);
 
 typedef GSList *get_user_groups_callback(const char *user,
 					 gpointer params);
@@ -47,7 +47,7 @@ typedef GSList *acl_check_callback(connection_t * element,
 typedef void define_period_callback(GHashTable * periods, gpointer params);
 
 /* ip auth */
-typedef gchar *ip_auth_callback(tracking_t * header, gpointer params);
+typedef gchar *ip_auth_callback(auth_pckt_t * pckt, gpointer params);
 
 typedef int user_logs_callback(void *element, tcp_state_t state,
 			       gpointer params);
@@ -57,12 +57,10 @@ typedef int user_session_logs_callback(user_session_t * element,
 
 /* certificate stuff */
 
-typedef int certificate_check_callback(gnutls_session session,
-				       gnutls_x509_crt cert,
+typedef int certificate_check_callback(nussl_session *nussl,
 				       gpointer params);
 /* certificate to uid function */
-typedef gchar *certificate_to_uid_callback(gnutls_session session,
-					   gnutls_x509_crt cert,
+typedef gchar *certificate_to_uid_callback(nussl_session *nussl,
 					   gpointer params);
 
 typedef nu_error_t user_session_modify_callback(user_session_t * session,
@@ -75,6 +73,7 @@ typedef void auth_error_log_callback(user_session_t * session,
 				     nuauth_auth_error_t error,
 				     const char *message, gpointer params);
 
+typedef int postauth_proto_callback(user_session_t * session, gpointer params);
 /** @} */
 
 #endif
