@@ -1,12 +1,13 @@
 /*
- ** Copyright(C) 2005-2007 INL
+ ** Copyright(C) 2005-2008 INL
  ** Written by Eric Leblond <regit@inl.fr>
+ **            Pierre Chifflier <chifflier@inl.fr>
  **
- ** $Id: nuauthconf.h 3909 2007-11-07 22:09:19Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,8 +27,19 @@ int init_nuauthconf(struct nuauth_params **);
 int build_prenuauthconf(struct nuauth_params *prenuauthconf,
 			char *gwsrv_addr, policy_t connect_policy);
 
-void nuauth_reload(int signal);
+gboolean nuauth_reload(int signal);
 
 void free_nuauth_params(struct nuauth_params *data);
 
+int nuauth_parse_configuration(const char *filename);
+
+
+char *nuauth_config_table_get(const char *key);
+char *nuauth_config_table_get_alwaysstring(char *key);
+char *nuauth_config_table_get_or_default(char *key, char *replace);
+int nuauth_config_table_get_or_default_int(char *key, int defint);
+void nuauth_config_table_destroy(void);
+void nuauth_config_table_print(void *userdata, void (*func)(void *data, char *keyeqval));
+
+
 #endif
