@@ -33,6 +33,7 @@
 
 GHashTable *nufw_servers = NULL;
 GStaticMutex nufw_servers_mutex = G_STATIC_MUTEX_INIT;
+extern int nufw_servers_connected;
 
 void init_nufw_servers()
 {
@@ -48,6 +49,8 @@ nu_error_t add_nufw_server(int conn_fd, nufw_session_t * nu_session)
 	g_static_mutex_lock(&nufw_servers_mutex);
 	g_hash_table_insert(nufw_servers, GINT_TO_POINTER(conn_fd),
 			nu_session);
+
+	g_atomic_int_inc(&nufw_servers_connected);
 	g_static_mutex_unlock(&nufw_servers_mutex);
 
 	return NU_EXIT_OK;
@@ -87,7 +90,7 @@ void clean_nufw_session(nufw_session_t * c_session)
 	g_mutex_free(c_session->tls_lock);
 	g_free(c_session);
 
-	nufw_servers_connected--;
+	g_atomic_int_dec_and_test(&nufw_servers_connected);
 
 	debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_GW, "close nufw session: done");
 }
