@@ -1,12 +1,12 @@
 /*
- ** Copyright(C) 2007-2008 INL
+ ** Copyright(C) 2007 INL
  ** Written by Victor Stinner <victor.stinner@inl.fr>
  **
- ** $Id: nuthread.c 5470 2008-12-31 11:13:39Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -38,8 +38,9 @@ void thread_new(struct nuauth_thread_t *thread,
 	    g_thread_create((GThreadFunc) func, thread->mutex, TRUE, NULL);
 	if (thread->thread == NULL)
 	{
-		g_warning("FATAL ERROR: Unable to create thread %s!",
-			name);
+		log_message(FATAL, DEBUG_AREA_MAIN,
+			    "FATAL ERROR: Unable to create thread %s!",
+			    name);
 		exit(EXIT_FAILURE);
 	}
 	thread->valid = 1;
@@ -57,8 +58,9 @@ void thread_new_wdata(struct nuauth_thread_t *thread,
 	    g_thread_create((GThreadFunc) func, thread , TRUE, NULL);
 	if (thread->thread == NULL)
 	{
-		g_warning("FATAL ERROR: Unable to create thread %s!",
-			name);
+		log_message(FATAL, DEBUG_AREA_MAIN,
+			    "FATAL ERROR: Unable to create thread %s!",
+			    name);
 		exit(EXIT_FAILURE);
 	}
 	thread->valid = 1;
@@ -93,11 +95,11 @@ void thread_wait_end(struct nuauth_thread_t *thread)
 	GThread *self;
 	if (!thread->valid)
 		return;
-	log_message(DEBUG, DEBUG_AREA_MAIN, "Waiting for end of thread '%s'", thread->name);
+	log_message(DEBUG, DEBUG_AREA_MAIN, "Wait end of thread '%s'", thread->name);
 	self = g_thread_self();
 	if (self == thread->thread) {
 		log_message(INFO, DEBUG_AREA_MAIN,
-			    "Information : Avoiding thread deadlock : not waiting for the end of myself!");
+			    "Information: Avoid deadlock: don't wait end of active thread!");
 		return;
 	}
 	g_thread_join(thread->thread);
