@@ -1,12 +1,11 @@
 /*
- ** Copyright(C) 2003-2008 Eric Leblond <regit@inl.fr>
+ ** Copyright(C) 2003-2009 INL
+ ** Written by Eric Leblond <eleblond@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: pckt_authsrv.c 5223 2008-11-19 14:15:21Z regit $
- **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,6 +32,8 @@
 
 #include <auth_srv.h>
 #include <errno.h>
+#include <netinet/ip.h>
+#include <netinet/ip6.h>
 
 #include "pckt_authsrv_v3.h"
 
@@ -44,10 +45,11 @@ nu_error_t parse_dgram(connection_t * connection, unsigned char *dgram,
 		       unsigned int dgram_size, connection_t ** conn,
 		       nufw_message_t msg_type)
 {
+	unsigned char *orig_dgram = dgram;
 	unsigned int ip_hdr_size;
+	struct iphdr *ip = (struct iphdr *) dgram;
 	/* get ip headers till tracking is filled */
-	ip_hdr_size =
-	    get_ip_headers(&connection->tracking, dgram, dgram_size);
+	ip_hdr_size = get_ip_headers(&connection->tracking, dgram, dgram_size);
 
 	if (ip_hdr_size == 0) {
 		log_message(WARNING, DEBUG_AREA_PACKET | DEBUG_AREA_GW,
@@ -56,16 +58,6 @@ nu_error_t parse_dgram(connection_t * connection, unsigned char *dgram,
 		return NU_EXIT_ERROR;
 	}
 
-	if (( ip_hdr_size + sizeof(connection->tracking.payload)) <= dgram_size) {
-		memcpy(connection->tracking.payload,
-		       dgram + ip_hdr_size,
-		       sizeof(connection->tracking.payload));
-	} else {
-		memset(connection->tracking.payload,
-		       0,
-		       sizeof(connection->tracking.payload));
-	}
-
 	dgram += ip_hdr_size;
 	dgram_size -= ip_hdr_size;
 
@@ -73,9 +65,9 @@ nu_error_t parse_dgram(connection_t * connection, unsigned char *dgram,
 	/* get saddr and daddr */
 	/* check if proto is in Hello mode list (when hello authentication is used) */
 	if (nuauthconf->hello_authentication
-	    && localid_authenticated_protocol(connection->tracking.
-					      protocol)) {
+	    && localid_authenticated_protocol(connection)) {
 		connection->state = AUTH_STATE_HELLOMODE;
+		connection->auth_quality = AUTHQ_HELLO;
 		*conn = connection;
 	} else {
 		connection->state = AUTH_STATE_AUTHREQ;
@@ -83,8 +75,7 @@ nu_error_t parse_dgram(connection_t * connection, unsigned char *dgram,
 	switch (connection->tracking.protocol) {
 	case IPPROTO_TCP:
 		{
-			tcp_state_t tcp_state =
-			    get_tcp_headers(&connection->tracking, dgram,
+			tcp_state_t tcp_state = get_tcp_headers(&connection->tracking, dgram,
 					    dgram_size);
 			switch (tcp_state) {
 			case TCP_STATE_OPEN:
@@ -94,7 +85,7 @@ nu_error_t parse_dgram(connection_t * connection, unsigned char *dgram,
 					connection->state =
 					    AUTH_STATE_DONE;
 					log_message(WARNING, DEBUG_AREA_GW,
-						    "nufw sent a non SYN TCP packet, ignoring");
+						    "nufw sends non SYN TCP packet, ignoring");
 					free_connection(connection);
 					return NU_EXIT_NO_RETURN;
 				}
@@ -104,7 +95,7 @@ nu_error_t parse_dgram(connection_t * connection, unsigned char *dgram,
 					connection->state =
 					    AUTH_STATE_DONE;
 					log_message(WARNING, DEBUG_AREA_GW,
-						    "nufw sent a SYN ACK TCP packet, ignoring");
+						    "nufw sends SYN ACK TCP packet, ignoring");
 					free_connection(connection);
 					return NU_EXIT_NO_RETURN;
 				}
@@ -153,6 +144,13 @@ nu_error_t parse_dgram(connection_t * connection, unsigned char *dgram,
 			return NU_EXIT_ERROR;
 		}
 	}
+
+	if (ntohs(ip->tot_len) > STORED_PAYLOAD_SIZE)
+		connection->payload_len = STORED_PAYLOAD_SIZE;
+	else
+		connection->payload_len = ntohs(ip->tot_len);
+	memcpy(connection->payload, orig_dgram, connection->payload_len);
+
 	return NU_EXIT_CONTINUE;
 }
 
@@ -211,7 +209,7 @@ nu_error_t authpckt_new_connection(unsigned char *dgram,
 		log_message(WARNING, DEBUG_AREA_PACKET | DEBUG_AREA_GW,
 			    "NuFW packet too small: %d for a minimum of %lu",
 			    dgram_size,
-			    sizeof(nuv4_nufw_to_nuauth_auth_message_t));
+			    (unsigned long)sizeof(nuv4_nufw_to_nuauth_auth_message_t));
 		return NU_EXIT_ERROR;
 	}
 	dgram += sizeof(nuv4_nufw_to_nuauth_auth_message_t);
@@ -233,6 +231,7 @@ nu_error_t authpckt_new_connection(unsigned char *dgram,
 	connection->user_groups = NULL;
 	connection->decision = DECISION_NODECIDE;
 	connection->expire = -1;
+	connection->payload_len = 0;
 
 	connection->packet_id =
 	    g_slist_append(NULL, GUINT_TO_POINTER(ntohl(msg->packet_id)));
@@ -251,8 +250,7 @@ nu_error_t authpckt_new_connection(unsigned char *dgram,
 	/* connection is proto v4 because we are here */
 	connection->nufw_version = PROTO_NUFW_VERSION;
 
-	ret =
-	    parse_dgram(connection, dgram, dgram_size, conn,
+	ret = parse_dgram(connection, dgram, dgram_size, conn,
 			msg->msg_type);
 	if (ret != NU_EXIT_CONTINUE) {
 		return ret;
@@ -297,7 +295,7 @@ nu_error_t authpckt_conntrack(unsigned char *dgram, unsigned int dgram_size)
 		log_message(CRITICAL, DEBUG_AREA_PACKET | DEBUG_AREA_GW,
 				  "Auth conntrack: Improper length of packet (%d instead of %lu)",
 				  dgram_size,
-				  sizeof(struct nuv4_conntrack_message_t));
+				  (unsigned long)sizeof(struct nuv4_conntrack_message_t));
 		return NU_EXIT_ERROR;
 	}
 
@@ -339,6 +337,9 @@ nu_error_t authpckt_conntrack(unsigned char *dgram, unsigned int dgram_size)
 	datas->bytes_in = conntrack->bytes_in;
 	datas->packets_out = conntrack->packets_out;
 	datas->bytes_out = conntrack->bytes_out;
+	/* timestamp is not set in conntrack message, thus best guest */
+	datas->timestamp = time(NULL);
+
 
 	message->datas = datas;
 
@@ -484,6 +485,7 @@ nu_error_t authpckt_decode(unsigned char **pdgram,
 			log_message(CRITICAL, DEBUG_AREA_PACKET | DEBUG_AREA_GW,
 				    "NuFW protocol is unknown");
 		}
+		return NU_EXIT_ERROR;
 
 	}
 	return NU_EXIT_OK;
