@@ -1,12 +1,12 @@
 /*
- ** Copyright(C) 2005-2008 INL
+ ** Copyright(C) 2005-2009 INL
  ** Written by Eric Leblond <regit@inl.fr>
  **
- ** $Id: sasl.c 5028 2008-10-28 23:52:54Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -53,6 +53,8 @@
 #include <sasl/saslutil.h>
 #include "security.h"
 
+#include <nubase.h>
+
 gchar *mech_string_internal;
 gchar *mech_string_external;
 
@@ -113,6 +115,7 @@ static int userdb_checkpass(sasl_conn_t * conn,
 			    unsigned passlen, struct propctx *propctx)
 {
 	char *dec_user = NULL;
+	int ret;
 
 	/*
 	 * call module to get password
@@ -146,7 +149,8 @@ static int userdb_checkpass(sasl_conn_t * conn,
 	}
 
 
-	if (modules_user_check(dec_user, pass, passlen) == SASL_OK) {
+	ret = modules_user_check(dec_user, pass, passlen, (user_session_t *)context);
+	if (ret == SASL_OK) {
 		/* we're done */
 		if (nuauthconf->uses_utf8)
 			g_free(dec_user);
@@ -158,8 +162,8 @@ static int userdb_checkpass(sasl_conn_t * conn,
 	log_message(INFO, DEBUG_AREA_AUTH, "Bad auth from user at %s:%d",
 		    __FILE__, __LINE__);
 
-	sasl_seterror(conn, 0, "Bad auth from user" );
-	return SASL_NOUSER;
+	sasl_seterror(conn, 0, "Bad auth from user");
+	return ret;
 }
 
 
@@ -186,7 +190,7 @@ void my_sasl_init()
 
 }
 
-static int samp_send(gnutls_session session, const char *buffer,
+static int samp_send(nussl_session* nussl, const char *buffer,
 		     unsigned length)
 {
 	char *buf;
@@ -199,26 +203,30 @@ static int samp_send(gnutls_session session, const char *buffer,
 	result = sasl_encode64(buffer, length, buf + 3, alloclen - 3, &len);
 	if (result != SASL_OK) {
 		g_free(buf);
-		log_message(WARNING, DEBUG_AREA_AUTH,
-			"Encoding data in base64 failed");
+		log_message(WARNING, DEBUG_AREA_AUTH, "Encoding data in base64 failed");
 		return result;
 	}
 	memcpy(buf, "S: ", 3);
 
-	result = gnutls_record_send(session, buf, len + 3);
+	result = nussl_write(nussl, buf, len + 3);
+	if (result < 0)
+		log_message(WARNING, DEBUG_AREA_AUTH, "nussl_write() failed: %s", nussl_get_error(nussl));
+		
 	g_free(buf);
 	return result;
 }
 
-static unsigned samp_recv(gnutls_session session, char *buf, int bufsize)
+static unsigned samp_recv(nussl_session* nussl, char *buf, int bufsize)
 {
-	unsigned len;
+	unsigned int len;
 	int result;
 
-	len = gnutls_record_recv(session, buf, bufsize);
-	if (len <= 0) {
+	result = nussl_read(nussl, buf, bufsize);
+	if (result < 0) {
+		log_message(WARNING, DEBUG_AREA_AUTH, "nussl_read() failed: %s", nussl_get_error(nussl));
 		return 0;
 	}
+	len = (unsigned int)result;
 
 	result = sasl_decode64(buf + 3, (unsigned) strlen(buf + 3), buf,
 			       bufsize, &len);
@@ -230,6 +238,53 @@ static unsigned samp_recv(gnutls_session session, char *buf, int bufsize)
 	return len;
 }
 
+#define MAX_TRY 2
+nu_error_t negotiate_proto_version(user_session_t *c_session)
+{
+	int i = 0;
+	int n = 0;
+
+	while ((c_session->proto_version > PROTO_VERSION) && (i < MAX_TRY)) {
+		char data[10];
+
+		n = snprintf(data, 10, "%s %d", PROTO_STRING, PROTO_VERSION);
+		if (n <= 0) {
+			return NU_EXIT_ERROR;
+		}
+		n = nussl_write(c_session->nussl, data, strlen(data));
+		if (n < 0) {
+			return NU_EXIT_ERROR;
+		}
+		n = nussl_read(c_session->nussl, data, sizeof(data));
+		if (n<0) {
+			return NU_EXIT_ERROR;
+		}
+		if (((int) strlen(PROTO_STRING) + 2) <= n 
+				&& strncmp(data, PROTO_STRING,
+					strlen(PROTO_STRING)) ==
+				0) {
+			data[n] = 0;
+			c_session->proto_version =
+				atoi((char *) data +
+						strlen(PROTO_STRING));
+		} else {
+			return NU_EXIT_ERROR;
+		}
+		i++;
+	}
+	if (c_session->proto_version <= PROTO_VERSION) {
+		n = nussl_write(c_session->nussl, "OK", strlen("OK"));
+		if (n < 0) {
+			return NU_EXIT_ERROR;
+		}
+		return NU_EXIT_OK;
+	} else {
+		nussl_write(c_session->nussl, "NOK", strlen("NOK"));
+		return NU_EXIT_ERROR;
+	}
+	return NU_EXIT_ERROR;
+}
+
 /**
  *  fetch protocol version (or guess)
  *
@@ -267,25 +322,26 @@ nu_error_t get_proto_info(user_session_t * c_session)
 		}
 	case 0:
 		{
-			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
+			log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
 					  "Falling back to v3 protocol");
-			c_session->client_version = PROTO_VERSION_V20;
+			c_session->proto_version = PROTO_VERSION_V20;
 		}
 		break;
 	default:
 		{
-
 			if (FD_ISSET(c_session->socket, &wk_set)) {
 				char buffer[20];
 				memset(buffer, 0, sizeof(buffer));
 
-				debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
+				log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
 						  "Getting protocol information");
-				ret =
-				    gnutls_record_recv(*(c_session->tls),
+				ret = nussl_read(c_session->nussl,
 						       buffer,
 						       sizeof(buffer) - 1);
+
 				if (ret <= 0) {
+					log_message(INFO, DEBUG_AREA_AUTH,
+							  "nussl_read() failed: %s", nussl_get_error(c_session->nussl));
 					return NU_EXIT_ERROR;
 				}
 				if (((int) strlen(PROTO_STRING) + 2) <= ret
@@ -293,29 +349,42 @@ nu_error_t get_proto_info(user_session_t * c_session)
 					       strlen(PROTO_STRING)) ==
 				    0) {
 					buffer[ret] = 0;
-					c_session->client_version =
+					c_session->proto_version =
 					    atoi((char *) buffer +
 						 strlen(PROTO_STRING));
 
-					debug_log_message(VERBOSE_DEBUG,
+					log_message(VERBOSE_DEBUG,
 							  DEBUG_AREA_AUTH,
 							  "Protocol information: %d",
 							  c_session->
-							  client_version);
+							  proto_version);
+					if (c_session->proto_version >= PROTO_VERSION_V24) {
+						int ret = negotiate_proto_version(c_session);
+						if (ret != NU_EXIT_OK) {
+							log_message(INFO, DEBUG_AREA_AUTH,
+									"Unable to negotiate proto");
+							c_session->proto_version = PROTO_VERSION_NONE;
+						}
+						return;
+					}
 					/* sanity check on know protocol */
-					if (c_session->client_version !=
-					    PROTO_VERSION_V22) {
-						debug_log_message(INFO,
-								  DEBUG_AREA_AUTH,
-								  "Bad protocol, announced %d",
-								  c_session->
-								  client_version);
-						return NU_EXIT_ERROR;
+					switch (c_session->proto_version) {
+						case PROTO_VERSION_V22:
+						case PROTO_VERSION_V22_1:
+						case PROTO_VERSION_V24:
+							break;
+						default:
+							log_message(INFO,
+									  DEBUG_AREA_AUTH,
+									  "Bad protocol, announced %d",
+									  c_session->proto_version
+									  );
+							return NU_EXIT_ERROR;
 					}
 					return NU_EXIT_OK;
 				} else {
 					log_message(INFO, DEBUG_AREA_AUTH,
-						    "Error: bad proto string");
+						    "Error bad proto string");
 					return NU_EXIT_ERROR;
 				}
 			}
@@ -341,8 +410,9 @@ static int mysasl_negotiate(user_session_t * c_session, sasl_conn_t * conn)
 	int tls_len = 0;
 	unsigned sasl_len = 0;
 	int count;
+#if 0
 	gnutls_session session = *(c_session->tls);
-	gboolean external_auth = FALSE;
+#endif
 	ssize_t record_send;
 	unsigned len = tls_len;
 	int result, auth_result;
@@ -354,18 +424,20 @@ static int mysasl_negotiate(user_session_t * c_session, sasl_conn_t * conn)
 			    "generating mechanism list");
 		return result;
 	}
-	debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
+	log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
 			  "%d mechanisms : %s (length: %d)", count, data,
 			  sasl_len);
 	/* send capability list to client */
-	record_send = samp_send(session, data, sasl_len);
+	record_send = samp_send(c_session->nussl, data, sasl_len);
 	tls_len = sasl_len;
+#if 0
 	if ((record_send == GNUTLS_E_INTERRUPTED)
 	    || (record_send == GNUTLS_E_AGAIN)) {
 		debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
 				  "sasl nego: need to resend packet");
-		record_send = samp_send(session, data, tls_len);
+		record_send = samp_send(c_session->nussl, data, tls_len);
 	}
+#endif
 	if (record_send < 0) {
 		return SASL_FAIL;
 	}
@@ -373,12 +445,12 @@ static int mysasl_negotiate(user_session_t * c_session, sasl_conn_t * conn)
 			  "Now we know record_send >= 0");
 
 	memset(buf, 0, sizeof(buf));
-	tls_len = samp_recv(session, buf, sizeof(buf));
+	tls_len = samp_recv(c_session->nussl, buf, sizeof(buf));
 	if (tls_len <= 0) {
 		if (tls_len == 0) {
 			log_message(INFO, DEBUG_AREA_AUTH,
 				    "client didn't choose mechanism");
-			if (samp_send(session, "N", 1) <= 0)	/* send NO to client */
+			if (samp_send(c_session->nussl, "N", 1) <= 0)	/* send NO to client */
 				return SASL_FAIL;
 			return SASL_BADPARAM;
 		} else {
@@ -387,7 +459,7 @@ static int mysasl_negotiate(user_session_t * c_session, sasl_conn_t * conn)
 			return SASL_FAIL;
 		}
 	}
-	debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
+	log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
 			  "client chose mechanism %s", buf);
 
 	if (strlen(buf) < (size_t) tls_len) {
@@ -402,13 +474,30 @@ static int mysasl_negotiate(user_session_t * c_session, sasl_conn_t * conn)
 				   buf,
 				   data, len, &data, (unsigned *) &len);
 
-	if (auth_result != SASL_OK && auth_result != SASL_CONTINUE)
-		g_message("Starting SASL negotiation: %s",
-			  sasl_errstring(auth_result, NULL, NULL));
+	if (auth_result != SASL_OK && auth_result != SASL_CONTINUE) {
+		char *tempname = NULL;
+		log_message(INFO, DEBUG_AREA_AUTH, "Error starting SASL negotiation: %s (%d)",
+			  sasl_errstring(auth_result, NULL, NULL),
+			  auth_result);
+		result =
+		    sasl_getprop(conn, SASL_AUTHUSER,
+				 (const void **) &(tempname));
+		if (result != SASL_OK) {
+			g_warning("get user failed: %s", sasl_errstring(result, NULL, NULL));
+			return result;
+		}
+		if (tempname == NULL)
+		{
+			g_warning("sasl_getprop(SASL_AUTHUSER): username is NULL!");
+			return SASL_BADPARAM;
+		}
+		c_session->user_name = g_strdup(tempname);
+		return auth_result;
+	}
 
 	while (auth_result == SASL_CONTINUE) {
 		if (data) {
-			samp_send(session, data, len);
+			samp_send(c_session->nussl, data, len);
 		} else {
 			log_message(WARNING, DEBUG_AREA_AUTH,
 				    "No data to send--something's wrong");
@@ -417,38 +506,43 @@ static int mysasl_negotiate(user_session_t * c_session, sasl_conn_t * conn)
 				  "Waiting for client reply...");
 
 		memset(buf, 0, sizeof(buf));
-		len = samp_recv(session, buf, sizeof(buf));
+		len = samp_recv(c_session->nussl, buf, sizeof(buf));
 		data = NULL;
 		auth_result = sasl_server_step(conn, buf, len, &data, &len);
 	}
 
 	if (auth_result != SASL_OK) {
-		debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
+		log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
 				  "incorrect authentication");
+		if (c_session->proto_version >= PROTO_VERSION_V22_1) {
+			samp_send(c_session->nussl, "N", 1);
+		}
 	} else {
-		debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
+		log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
 				  "correct authentication");
+		if (c_session->proto_version >= PROTO_VERSION_V22_1) {
+			samp_send(c_session->nussl, "Y", 1);
+		}
 	}
 
 	if (c_session->user_name) {
-		external_auth = TRUE;
 		if (auth_result != SASL_OK) {
 			return auth_result;
 		}
 	}
 
-	if (external_auth == FALSE) {
+	if (c_session->auth_type != AUTH_TYPE_EXTERNAL) {
 		char *tempname = NULL;
 		result =
-		    sasl_getprop(conn, SASL_USERNAME,
+		    sasl_getprop(conn, SASL_AUTHUSER,
 				 (const void **) &(tempname));
 		if (result != SASL_OK) {
-			g_warning("get user failed");
+			g_warning("get user failed: %s", sasl_errstring(result, NULL, NULL));
 			return result;
 		}
 		if (tempname == NULL)
 		{
-			g_warning("sasl_getprop(SASL_USERNAME): username is NULL!");
+			g_warning("sasl_getprop(SASL_AUTHUSER): username is NULL!");
 			return SASL_BADPARAM;
 		}
 		c_session->user_name = g_strdup(tempname);
@@ -462,7 +556,7 @@ static int mysasl_negotiate(user_session_t * c_session, sasl_conn_t * conn)
 		    modules_get_user_groups(c_session->user_name);
 		if (c_session->groups == NULL) {
 			log_message(INFO, DEBUG_AREA_AUTH,
-					  "error when searching groups for user \"%s\"",
+					  "error when searching user groups for %s",
 					  c_session->user_name);
 			return SASL_BADAUTH;
 		}
@@ -470,13 +564,13 @@ static int mysasl_negotiate(user_session_t * c_session, sasl_conn_t * conn)
 		    modules_get_user_id(c_session->user_name);
 		if (c_session->user_id == 0) {
 			log_message(INFO, DEBUG_AREA_AUTH,
-				    "Couldn't get user ID for \"%s\"!",
-				    c_session->user_name);
+					"Couldn't get user ID for \"%s\"!",
+					c_session->user_name);
 			return SASL_BADAUTH;
 		}
 	}
 #if 0
-	if (gnutls_record_send(session, "O", 1) <= 0)	/* send YES to client */
+	if (nussl_write(c_session->nussl, "O", 1) < 0)	/* send YES to client */
 		return SASL_FAIL;
 #endif
 
@@ -484,141 +578,6 @@ static int mysasl_negotiate(user_session_t * c_session, sasl_conn_t * conn)
 	return SASL_OK;
 }
 
-int sasl_parse_user_os(user_session_t * c_session, char *buf, int buf_size)
-{
-	unsigned int len;
-	int decode;
-	struct nu_authfield *osfield;
-	gchar *dec_buf = NULL;
-	gchar **os_strings;
-	int dec_buf_size;
-	char address[INET6_ADDRSTRLEN];
-
-	osfield = (struct nu_authfield *) buf;
-
-	/* check buffer underflow */
-	if (buf_size < (int) sizeof(struct nu_authfield)) {
-		FORMAT_IPV6(&c_session->addr, address);
-		g_message("%s sent a too small osfield", address);
-		return SASL_FAIL;
-	}
-
-	if (osfield->type != OS_FIELD) {
-#ifdef DEBUG_ENABLE
-		log_message(DEBUG, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
-			    "osfield received %d,%d,%d from %s",
-			    osfield->type, osfield->option,
-			    ntohs(osfield->length), address);
-#endif
-		return SASL_FAIL;
-	}
-
-	dec_buf_size = ntohs(osfield->length);
-	if (dec_buf_size > 1024 || (ntohs(osfield->length) <= 4)) {
-		FORMAT_IPV6(&c_session->addr, address);
-		log_message(WARNING, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
-				"error osfield from %s is uncorrect, announced %d",
-				address, ntohs(osfield->length));
-		/* One more gryzor hack */
-		if (dec_buf_size > 4096)
-			log_message(WARNING, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
-				    "   Is %s running a 1.0 client?",
-				    address);
-#ifdef DEBUG_ENABLE
-		log_message(DEBUG, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
-			    "%s:%d osfield received %d,%d,%d ", __FILE__,
-			    __LINE__, osfield->type, osfield->option,
-			    ntohs(osfield->length));
-#endif
-		return SASL_BADAUTH;
-	}
-	dec_buf = g_new0(gchar, dec_buf_size);
-	decode =
-	    sasl_decode64(buf + 4, ntohs(osfield->length) - 4, dec_buf,
-			  dec_buf_size, &len);
-	if (decode != SASL_OK) {
-		g_free(dec_buf);
-		return SASL_BADAUTH;
-	}
-
-	/* should always be true for the moment */
-	if (osfield->option == OS_SRV) {
-		os_strings = g_strsplit(dec_buf, ";", 3);
-		if (os_strings[0] == NULL || os_strings[1] == NULL
-		    || os_strings[2] == NULL) {
-			g_strfreev(os_strings);
-			g_free(dec_buf);
-			return SASL_BADAUTH;
-		}
-		if (strlen(os_strings[0]) < 128) {
-			c_session->sysname = string_escape(os_strings[0]);
-			if (c_session->sysname == NULL) {
-				FORMAT_IPV6(&c_session->addr, address);
-				log_message(WARNING, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
-						"received sysname with invalid characters from %s",
-						address);
-				g_free(dec_buf);
-				return SASL_BADAUTH;
-			}
-		} else {
-			c_session->sysname = g_strdup(UNKNOWN_STRING);
-		}
-		if (strlen(os_strings[1]) < 128) {
-			c_session->release = string_escape(os_strings[1]);
-			if (c_session->release == NULL) {
-				FORMAT_IPV6(&c_session->addr, address);
-				log_message(WARNING, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
-						"received release with invalid characters from %s",
-						address);
-				g_free(dec_buf);
-				return SASL_BADAUTH;
-			}
-		} else {
-			c_session->release = g_strdup(UNKNOWN_STRING);
-		}
-		if (strlen(os_strings[2]) < 128) {
-			c_session->version = string_escape(os_strings[2]);
-			if (c_session->version == NULL) {
-				FORMAT_IPV6(&c_session->addr, address);
-				log_message(WARNING, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
-						"received version with invalid characters from %s",
-						address);
-				g_free(dec_buf);
-				return SASL_BADAUTH;
-			}
-		} else {
-			c_session->version = g_strdup(UNKNOWN_STRING);
-		}
-		/* print information */
-		if (c_session->sysname && c_session->release &&
-		    c_session->version) {
-
-#ifdef DEBUG_ENABLE
-			if (DEBUG_OR_NOT(DEBUG_LEVEL_DEBUG, DEBUG_AREA_USER)) {
-				FORMAT_IPV6(&c_session->addr, address);
-				g_message
-					("user %s at %s uses OS %s ,%s, %s",
-					 c_session->user_name, address,
-					 c_session->sysname,
-					 c_session->release,
-					 c_session->version);
-			}
-#endif
-		}
-		g_strfreev(os_strings);
-	} else {
-		FORMAT_IPV6(&c_session->addr, address);
-		log_message(DEBUG, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
-				"from %s : osfield->option is not OS_SRV ?!",
-				address);
-		g_free(dec_buf);
-		return SASL_FAIL;
-
-	}
-	g_free(dec_buf);
-	return SASL_OK;
-}
-
 /**
  * do the sasl negotiation, protocol v3
  *
@@ -635,63 +594,71 @@ static int mysasl_negotiate_v3(user_session_t * c_session,
 	int r = SASL_FAIL;
 	int count;
 	int ret = 0;
+#if 0
 	gnutls_session session = *(c_session->tls);
-	gboolean external_auth = FALSE;
+#endif
 	ssize_t record_send;
 
 	r = sasl_listmech(conn, NULL, "(", ",", ")", &data, &sasl_len,
 			  &count);
 	if (r != SASL_OK) {
 		log_message(WARNING, DEBUG_AREA_AUTH,
-			    "v3 proto: generating mechanism list");
+			    "proto v3: generating mechanism list");
 		return r;
 	}
-	debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH, "v3 proto: %d mechanisms : %s",
+	log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH, "proto v3: %d mechanisms : %s",
 			  count, data);
 	tls_len = sasl_len;
 	/* send capability list to client */
-	record_send = gnutls_record_send(session, data, tls_len);
+	record_send = nussl_write(c_session->nussl, (char*)data, tls_len);
+#if 0
 	if ((record_send == GNUTLS_E_INTERRUPTED)
 	    || (record_send == GNUTLS_E_AGAIN)) {
 		debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
-				  "v3 proto: sasl nego: need to resend packet");
-		record_send = gnutls_record_send(session, data, tls_len);
+				  "proto v3: sasl nego : need to resend packet");
+		record_send = nussl_write(c_session->nussl, data, tls_len);
 	}
+#endif
 	if (record_send < 0) {
 		return SASL_FAIL;
 	}
 	debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
-			  "v3 proto: Now we know record_send >= 0");
+			  "proto v3: Now we know record_send >= 0");
 
 	memset(chosenmech, 0, sizeof chosenmech);
-	tls_len =
-	    gnutls_record_recv(session, chosenmech, sizeof chosenmech);
+	tls_len = nussl_read(c_session->nussl, chosenmech, sizeof chosenmech);
 	if (tls_len <= 0) {
 		if (tls_len == 0) {
 			log_message(INFO, DEBUG_AREA_AUTH,
-				    "v3 proto: client didn't choose mechanism");
-			if (gnutls_record_send(session, "N", 1) <= 0)	/* send NO to client */
+				    "proto v3: client didn't choose mechanism");
+			if (nussl_write(c_session->nussl, "N", 1) < 0)	/* send NO to client */
 				return SASL_FAIL;
 			return SASL_BADPARAM;
 		} else {
 			log_message(INFO, DEBUG_AREA_AUTH,
-				    "v3 proto: sasl nego: tls crash");
+				    "proto v3: sasl nego : tls crash");
 			return SASL_FAIL;
 		}
 	}
-	debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
-			  "v3 proto: client chose mechanism %s", chosenmech);
+	log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
+			  "proto v3: client chose mechanism %s", chosenmech);
 
 	memset(buf, 0, sizeof buf);
-	tls_len = gnutls_record_recv(session, buf, sizeof(buf));
+	tls_len = nussl_read(c_session->nussl, buf, sizeof(buf));
 	if (tls_len != 1) {
 		if (tls_len <= 0) {
+			log_message(INFO, DEBUG_AREA_AUTH,
+					  "nussl_read() error: %s", nussl_get_error(c_session->nussl));
 			return SASL_FAIL;
 		}
-		debug_log_message(DEBUG, DEBUG_AREA_AUTH,
-				  "v3 proto: didn't receive first-sent parameter correctly");
-		if (gnutls_record_send(session, "N", 1) <= 0)	/* send NO to client */
+		log_message(DEBUG, DEBUG_AREA_AUTH,
+				  "proto v3: didn't receive first-sent parameter correctly");
+		if (nussl_write(c_session->nussl, "N", 1) < 0)	/* send NO to client */
+		{
+			log_message(INFO, DEBUG_AREA_AUTH,
+					  "nussl_write() error: %s", nussl_get_error(c_session->nussl));
 			return SASL_FAIL;
+		}
 		return SASL_BADPARAM;
 	}
 
@@ -700,15 +667,17 @@ static int mysasl_negotiate_v3(user_session_t * c_session,
 
 
 		memset(buf, 0, sizeof(buf));
-		tls_len = gnutls_record_recv(session, buf, sizeof(buf));
+		tls_len = nussl_read(c_session->nussl, buf, sizeof(buf));
 		if (tls_len <= 0) {
+			log_message(INFO, DEBUG_AREA_AUTH,
+					  "nussl_read() error: %s", nussl_get_error(c_session->nussl));
 			return SASL_FAIL;
 		}
 		/* start libsasl negotiation */
 		r = sasl_server_start(conn, chosenmech, buf, tls_len,
 				      &data, &sasl_len);
 	} else {
-		debug_log_message(DEBUG, DEBUG_AREA_AUTH, "v3 proto: start with no msg");
+		log_message(DEBUG, DEBUG_AREA_AUTH, "proto v3: start with no msg");
 		r = sasl_server_start(conn, chosenmech, NULL, 0, &data,
 				      &sasl_len);
 	}
@@ -716,10 +685,10 @@ static int mysasl_negotiate_v3(user_session_t * c_session,
 	if (r != SASL_OK && r != SASL_CONTINUE) {
 		gchar *user_name = NULL;
 
-		log_message(INFO, DEBUG_AREA_AUTH, "v3 proto: sasl negotiation error: %d",
+		log_message(INFO, DEBUG_AREA_AUTH, "proto v3: sasl negotiation error: %d",
 			    r);
 		ret =
-		    sasl_getprop(conn, SASL_USERNAME,
+		    sasl_getprop(conn, SASL_AUTHUSER,
 				 (const void **) &(user_name));
 		if (ret == SASL_OK) {
 			c_session->user_name = g_strdup(user_name);
@@ -727,36 +696,39 @@ static int mysasl_negotiate_v3(user_session_t * c_session,
 			c_session->user_name = NULL;
 		}
 
-		if (gnutls_record_send(session, "N", 1) <= 0)	/* send NO to client */
+		if (nussl_write(c_session->nussl, "N", 1) < 0)	/* send NO to client */
+		{
+			log_message(INFO, DEBUG_AREA_AUTH,
+					  "nussl_write() error: %s", nussl_get_error(c_session->nussl));
 			return SASL_FAIL;
+		}
 		return SASL_BADPARAM;
 	}
 
 	while (r == SASL_CONTINUE) {
 
+		if (nussl_write(c_session->nussl, "C", 1) < 0)	/* send CONTINUE to client */
+			return SASL_FAIL;
+
 		if (data) {
-			if (gnutls_record_send(session, "C", 1) <= 0)	/* send CONTINUE to client */
-				return SASL_FAIL;
-			if (gnutls_record_send(session, data, tls_len) < 0)
+			if (nussl_write(c_session->nussl, (char*)data, tls_len) < 0)
 				return SASL_FAIL;
 		} else {
-			if (gnutls_record_send(session, "C", 1) <= 0)	/* send CONTINUE to client */
-				return SASL_FAIL;
-			if (gnutls_record_send(session, "", 0) < 0)
+			if (nussl_write(c_session->nussl, "", 0) < 0)
 				return SASL_FAIL;
 		}
 
 
 		memset(buf, 0, sizeof buf);
-		tls_len = gnutls_record_recv(session, buf, sizeof buf);
-		if (tls_len <= 0) {
+		tls_len = nussl_read(c_session->nussl, buf, sizeof buf);
+		if (tls_len < 0) {
 #ifdef DEBUG_ENABLE
 			if (!tls_len) {
 				log_message(VERBOSE_DEBUG, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
-					    "v3 proto: Client disconnected during sasl negotiation");
+					    "proto v3: Client disconnected during sasl negotiation");
 			} else {
 				log_message(VERBOSE_DEBUG, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
-					    "v3 proto: TLS error during sasl negotiation");
+					    "proto v3: TLS error during sasl negotiation");
 			}
 #endif
 			return SASL_FAIL;
@@ -766,10 +738,10 @@ static int mysasl_negotiate_v3(user_session_t * c_session,
 		if (r != SASL_OK && r != SASL_CONTINUE) {
 #ifdef DEBUG_ENABLE
 			log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
-				    "v3 proto: error performing SASL negotiation: %s",
+				    "proto v3: error performing SASL negotiation: %s",
 				    sasl_errdetail(conn));
 #endif
-			if (gnutls_record_send(session, "N", 1) <= 0)	/* send NO to client */
+			if (nussl_write(c_session->nussl, "N", 1) < 0)	/* send NO to client */
 				return SASL_FAIL;
 			return SASL_BADPARAM;
 		}
@@ -777,24 +749,37 @@ static int mysasl_negotiate_v3(user_session_t * c_session,
 
 
 	if (r != SASL_OK) {
-		debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
-				  "v3 proto: incorrect authentication");
-		if (gnutls_record_send(session, "N", 1) <= 0)	/* send NO to client */
+		log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
+				  "proto v3: incorrect authentication");
+		/* try to get username */
+		if (c_session->user_name == NULL) {
+			char *tempname = NULL;
+			ret =
+				sasl_getprop(conn, SASL_AUTHUSER,
+						(const void **) &(tempname));
+			if (ret != SASL_OK) {
+				g_warning("proto v3: get user failed");
+				return ret;
+			} else {
+				c_session->user_name = g_strdup(tempname);
+			}
+		}
+		if (nussl_write(c_session->nussl, "N", 1) < 0)	/* send NO to client */
 			return SASL_FAIL;
 		return SASL_BADAUTH;
 	}
 
 
 	if (c_session->user_name)
-		external_auth = TRUE;
+		c_session->auth_type = AUTH_TYPE_EXTERNAL;
 
-	if (external_auth == FALSE) {
+	if (c_session->auth_type != AUTH_TYPE_EXTERNAL) {
 		char *tempname = NULL;
 		ret =
-		    sasl_getprop(conn, SASL_USERNAME,
+		    sasl_getprop(conn, SASL_AUTHUSER,
 				 (const void **) &(tempname));
 		if (ret != SASL_OK) {
-			g_warning("v3 proto: get user failed");
+			g_warning("proto v3: get user failed");
 			return ret;
 		} else {
 			c_session->user_name = g_strdup(tempname);
@@ -805,9 +790,9 @@ static int mysasl_negotiate_v3(user_session_t * c_session,
 		c_session->groups =
 		    modules_get_user_groups(c_session->user_name);
 		if (c_session->groups == NULL) {
-			debug_log_message(DEBUG, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
-					  "v3 proto: Couldn't get user groups");
-			if (gnutls_record_send(session, "N", 1) <= 0)	/* send NO to client */
+			log_message(DEBUG, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
+					  "proto v3: Couldn't get user groups");
+			if (nussl_write(c_session->nussl, "N", 1) < 0)	/* send NO to client */
 				return SASL_FAIL;
 			return SASL_BADAUTH;
 		}
@@ -815,11 +800,11 @@ static int mysasl_negotiate_v3(user_session_t * c_session,
 		    modules_get_user_id(c_session->user_name);
 		if (c_session->user_id == 0) {
 			log_message(INFO, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
-				    "v3 proto: Couldn't get user ID!");
+				    "proto v3: Couldn't get user ID!");
 		}
 	}
 
-	if (gnutls_record_send(session, "O", 1) <= 0)	/* send YES to client */
+	if (nussl_write(c_session->nussl, "O", 1) < 0)	/* send YES to client */
 		return SASL_FAIL;
 
 	/* negotiation complete */
@@ -836,42 +821,56 @@ int sasl_user_check(user_session_t * c_session)
 {
 	sasl_conn_t *conn = NULL;
 	sasl_security_properties_t secprops;
-	gboolean external_auth = FALSE;
-	char buf[1024];
-	char *iplocalport = NULL;
-	char ipremoteport[INET6_ADDRSTRLEN+20];
+	char iplocalport[INET6_ADDRSTRLEN +20];
+	char ipremoteport[INET6_ADDRSTRLEN +20];
 	int len;
-	int ret, buf_size;
+	int ret;
 	sasl_callback_t internal_callbacks[] = {
-		{SASL_CB_GETOPT, &internal_get_opt, NULL},
-		{SASL_CB_SERVER_USERDB_CHECKPASS, &userdb_checkpass, NULL},
+		{SASL_CB_GETOPT, &internal_get_opt, c_session},
+		{SASL_CB_SERVER_USERDB_CHECKPASS, &userdb_checkpass, c_session},
 		{SASL_CB_LIST_END, NULL, NULL}
 	};
 	sasl_callback_t external_callbacks[] = {
-		{SASL_CB_GETOPT, &external_get_opt, NULL},
-		{SASL_CB_SERVER_USERDB_CHECKPASS, &userdb_checkpass, NULL},
+		{SASL_CB_GETOPT, &external_get_opt, c_session},
+		{SASL_CB_SERVER_USERDB_CHECKPASS, &userdb_checkpass, c_session},
 		{SASL_CB_LIST_END, NULL, NULL}
 	};
 	sasl_callback_t *callbacks;
 
 	if (c_session->user_name) {
-		external_auth = TRUE;
 		c_session->auth_type = AUTH_TYPE_EXTERNAL;
+		c_session->auth_quality = AUTHQ_SSL;
 		callbacks = external_callbacks;
 	} else {
 		callbacks = internal_callbacks;
 		c_session->auth_type = AUTH_TYPE_INTERNAL;
+		c_session->auth_quality = AUTHQ_SASL;
 		if (!nuauthconf->nuauth_uses_fake_sasl) {
 			callbacks = NULL;
 		}
 	}
 
 	/* format "ip;port" */
-	FORMAT_IPV6(&c_session->addr, ipremoteport);
+	format_ipv6(&c_session->addr, ipremoteport, INET6_ADDRSTRLEN, NULL);
 	len = strlen(ipremoteport);
 	secure_snprintf(ipremoteport+len, sizeof(ipremoteport)-len,
 		";%hu", c_session->sport);
 
+	/* format "ip;port" */
+	format_ipv6(&c_session->server_addr, iplocalport, INET6_ADDRSTRLEN, NULL);
+	len = strlen(iplocalport);
+	secure_snprintf(iplocalport+len, sizeof(iplocalport)-len,
+		";%s", nuauthconf->userpckt_port);
+
+
+
+	debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_AUTH,
+			  "Starting SASL server: service=%s, hostname=%s, realm=%s, iplocal=%s, ipremote=%s",
+			  nuauthconf->krb5_service,
+			  nuauthconf->krb5_hostname,
+			  nuauthconf->krb5_realm,
+			  iplocalport, ipremoteport
+			 );
 	ret = sasl_server_new(nuauthconf->krb5_service,
 			nuauthconf->krb5_hostname,
 			nuauthconf->krb5_realm,
@@ -880,17 +879,18 @@ int sasl_user_check(user_session_t * c_session)
 	if (ret != SASL_OK) {
 		g_warning
 		    ("allocating connection state - failure at sasl_server_new()");
+		return ret;
 	}
 
 	secprops.min_ssf = 0;
-	secprops.max_ssf = 0;
+	secprops.max_ssf = UINT_MAX;
 	secprops.property_names = NULL;
 	secprops.property_values = NULL;
 	secprops.security_flags = SASL_SEC_NOANONYMOUS;	/* as appropriate */
 	secprops.maxbufsize = 65536;
 	sasl_setprop(conn, SASL_SEC_PROPS, &secprops);
 
-	if (external_auth) {
+	if (c_session->auth_type == AUTH_TYPE_EXTERNAL) {
 		sasl_ssf_t extssf = 0;
 
 #ifdef DEBUG_ENABLE
@@ -907,14 +907,14 @@ int sasl_user_check(user_session_t * c_session)
 			sasl_dispose(&conn);
 			log_message(INFO, DEBUG_AREA_AUTH,
 				    "Error setting external auth");
-			return SASL_FAIL;
+			return ret;
 		}
 		ret = sasl_setprop(conn, SASL_SSF_EXTERNAL, &extssf);
 		if (ret != SASL_OK) {
 			sasl_dispose(&conn);
 			log_message(INFO, DEBUG_AREA_AUTH,
 				    "Error setting external SSF");
-			return SASL_FAIL;
+			return ret;
 		}
 	}
 
@@ -925,8 +925,10 @@ int sasl_user_check(user_session_t * c_session)
 		return SASL_BADPARAM;
 	}
 
-	switch (c_session->client_version) {
+	switch (c_session->proto_version) {
 	case PROTO_VERSION_V22:
+	case PROTO_VERSION_V22_1:
+	case PROTO_VERSION_V24:
 		ret = mysasl_negotiate(c_session, conn);
 		break;
 	case PROTO_VERSION_V20:
@@ -945,36 +947,16 @@ int sasl_user_check(user_session_t * c_session)
 		if (ret == SASL_BADAUTH || ret == SASL_NOUSER) {
 			err = AUTH_ERROR_CREDENTIALS;
 			message =
-			    "SASL error: invalid credentials (username or password)";
+			    "Invalid credentials (username or password)";
 		} else {
 			err = AUTH_ERROR_INTERRUPTED;
 			message =
-			    "SASL error: authentication process interrupted";
+			    "Authentication process interrupted";
 		}
 		modules_auth_error_log(c_session, err, message);
 		return ret;
 	}
 
-
-	/* recv OS datas from client */
-	buf_size = gnutls_record_recv(*(c_session->tls), buf, sizeof buf);
-	if (buf_size <= 0) {
-		/* allo houston */
-		debug_log_message(DEBUG, DEBUG_AREA_USER,
-				  "error when receiving user OS");
-		return SASL_FAIL;
-	}
-
-	/* parse and validate OS */
-	ret = sasl_parse_user_os(c_session, buf, buf_size);
-	if (ret != SASL_OK)
-		return ret;
-
-	/* Tuning of user_session */
-	ret = modules_user_session_modify(c_session);
-	if (ret != SASL_OK)
-		return ret;
-
 	/* sasl connection is not used anymore */
 	return SASL_OK;
 }
