@@ -1,13 +1,13 @@
 /*
- ** Copyright(C) 2006-2008 INL
+ ** Copyright(C) 2006,2008,2008 INL
  ** Written by Eric Leblond <regit@inl.fr>
  ** INL : http://www.inl.fr/
  **
- ** $Id: take_decision.c 5345 2008-11-26 13:08:38Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -123,6 +123,10 @@ static void search_user_group_in_acl_groups(struct acl_group *datas,
 		/* find a group match, time to update decision */
 		*answer = datas->answer;
 		update_decision(datas, answer, test, element, expire);
+	} else {
+		if (*answer == DECISION_NODECIDE) {
+			update_connection_datas (element,datas);
+		}
 	}
 }
 
@@ -139,6 +143,10 @@ static void search_user_id_in_acl_groups(struct acl_group *datas,
 		/* find a group match, time to update decision */
 		*answer = datas->answer;
 		update_decision(datas, answer, test, element, expire);
+	} else {
+		if (*answer == DECISION_NODECIDE) {
+			update_connection_datas (element,datas);
+		}
 	}
 }
 
@@ -152,8 +160,8 @@ static void search_user_id_in_acl_groups(struct acl_group *datas,
  * It iters on each element of connection_t::acl_groups.
  * For each element, it test every groups to check
  * if the users belongs to one of them.
- * When a match is found, there is two cases:
- *  - if nuauth_params::prio_to_nok is 0 then we continue till we find a acl with
+ * When a match is found, there is three cases:
+ *  - if nuauth_params::prio_to_nok is 0 then we continue till we fing a acl with
  *  ACCEPT decision.
  *  - if nuauth_params::prio_to_nok is set to 1, we stop if the decision is to
  *  block the packet.
@@ -198,31 +206,42 @@ nu_error_t take_decision(connection_t * element, packet_place_t place)
 		     (parcours != NULL && test == TEST_NODECIDE);
 		     parcours = g_slist_next(parcours)) {
 			if (parcours->data != NULL) {
-				/* search for a userid-based rule */
-				search_user_id_in_acl_groups(((struct acl_group *)(parcours->data)),
-							     &answer,
-							     &test,
-							     element,
-							     &expire,
-							     element->user_id);
-				/* for each user group */
-				for (user_group = element->user_groups;
-				     user_group != NULL
-				     && test == TEST_NODECIDE;
-				     user_group =
-				     g_slist_next(user_group)) {
-					/* search user group in acl_groups */
-					if (((struct acl_group *)(parcours->data))->groups) {
-							search_user_group_in_acl_groups(
-								((struct acl_group *)(parcours->data)),
-								&answer,
-								&test,
-								element,
-								&expire,
-								user_group);
-
+				if (((struct acl_group *)(parcours->data))->auth_quality >
+						element->auth_quality) {
+					if (nuauthconf->reject_authenticated_drop) {
+						answer = DECISION_REJECT;
+					} else {
+						answer = DECISION_DROP;
 					}
-				}	/* end of user group loop */
+					update_decision((struct acl_group *)(parcours->data),
+							&answer, &test, element, &expire);
+				} else {
+					/* search for a userid-based rule */
+					search_user_id_in_acl_groups(((struct acl_group *)(parcours->data)),
+							&answer,
+							&test,
+							element,
+							&expire,
+							element->user_id);
+					/* for each user group */
+					for (user_group = element->user_groups;
+							user_group != NULL
+							&& test == TEST_NODECIDE;
+							user_group =
+							g_slist_next(user_group)) {
+						/* search user group in acl_groups */
+						if (((struct acl_group *)(parcours->data))->groups) {
+							search_user_group_in_acl_groups(
+									((struct acl_group *)(parcours->data)),
+									&answer,
+									&test,
+									element,
+									&expire,
+									user_group);
+
+						}
+					}	/* end of user group loop */
+				}
 			} else {
 				debug_log_message(DEBUG, DEBUG_AREA_MAIN,
 						  "Empty acl : bad things ...");
@@ -240,6 +259,7 @@ nu_error_t take_decision(connection_t * element, packet_place_t place)
 			answer = DECISION_DROP;
 		}
 	}
+	/** Drop per expiration (packet out of time window) */
 	if (expire == 0) {
 		if (nuauthconf->reject_authenticated_drop) {
 			answer = DECISION_REJECT;
@@ -249,18 +269,13 @@ nu_error_t take_decision(connection_t * element, packet_place_t place)
 	}
 	element->decision = answer;
 
+	element->expire = expire;
 
 	/* Call modules to do final tuning of packet (setting mark, expire modification ...) */
 	modules_finalize_packet(element);
 
-	if ((element->expire != -1) && (element->expire < expire)) {
-		debug_log_message(DEBUG, DEBUG_AREA_MAIN,
-				  " taken expire from element");
-		expire = element->expire;
-	}
-
 	/* we must put element in expire list if needed before decision is taken */
-	if (expire > 0) {
+	if (element->expire > 0) {
 		if (nuauthconf->nufw_has_conntrack) {
 			struct limited_connection *datas =
 			    g_new0(struct limited_connection, 1);
@@ -272,7 +287,9 @@ nu_error_t take_decision(connection_t * element, packet_place_t place)
 			memcpy(&(datas->tracking), &(element->tracking),
 			       sizeof(tracking_t));
 			datas->expire = expire;
+
 			datas->gwaddr = element->tls->peername;
+
 			message->datas = datas;
 			message->type = INSERT_MESSAGE;
 			g_async_queue_push(nuauthdatas->
@@ -295,7 +312,7 @@ nu_error_t take_decision(connection_t * element, packet_place_t place)
 		if (place == PACKET_IN_HASH) {
 			if (conn_cl_delete(element) == 0) {
 				log_message(CRITICAL, DEBUG_AREA_MAIN,
-					    "Unable to suppress packet from hash");
+						"Unable to suppress packet from hash");
 			}
 		} else {
 			free_connection(element);
@@ -321,7 +338,7 @@ nu_error_t apply_decision(connection_t * element)
 #endif
 
 	if (element->state == AUTH_STATE_USERPCKT) {
-		debug_log_message(WARNING, DEBUG_AREA_MAIN,
+		log_message(WARNING, DEBUG_AREA_MAIN,
 				  "BUG: Should not be in apply_decision for user only packet");
 		return NU_EXIT_ERROR;
 	}
@@ -390,29 +407,29 @@ void send_auth_response(gpointer packet_id_ptr, gpointer userdata)
 	uint32_t packet_id = GPOINTER_TO_UINT(packet_id_ptr);
 	int payload_size = 0;
 	int total_size = 0;
-	char *buffer = NULL;
 	nu_error_t ret = NU_EXIT_OK;
+	char buffer[sizeof(nuv3_nuauth_decision_response_t) + IPHDR_REJECT_LENGTH +
+		IP6HDR_REJECT_LENGTH + STORED_PAYLOAD_SIZE];
 
 	switch (element->nufw_version) {
 	case PROTO_VERSION_NUFW_V20:
 		{
-			nuv3_nuauth_decision_response_t *response = NULL;
+			nuv3_nuauth_decision_response_t *response = (nuv3_nuauth_decision_response_t *) buffer;
 			uint16_t mark16;
 			/* check if user id fit in 16 bits */
 			if (0xFFFF < element->mark) {
 				log_message(WARNING, DEBUG_AREA_MAIN,
-					    "Mark bigger than 16 bits, need to truncate the value.");
+					    "Mark don't fit in 16 bits, not to truncate the value.");
 			}
 			mark16 = (element->mark & 0xFFFF);
 			if (element->decision == DECISION_REJECT) {
 				payload_size =
-				    IPHDR_REJECT_LENGTH + PAYLOAD_SAMPLE;
+				    IPHDR_REJECT_LENGTH + element->payload_len;
 			}
 			/* allocate */
 			total_size =
 			    sizeof(nuv3_nuauth_decision_response_t) +
 			    payload_size;
-			response = (void*)g_new0(char, total_size);
 			response->protocol_version = PROTO_VERSION_NUFW_V20;
 			response->msg_type = AUTH_ANSWER;
 			response->mark = htons(mark16);
@@ -423,7 +440,7 @@ void send_auth_response(gpointer packet_id_ptr, gpointer userdata)
 			response->payload_len = htons(payload_size);
 			if (element->decision == DECISION_REJECT) {
 				char payload[IPHDR_REJECT_LENGTH +
-					     PAYLOAD_SAMPLE];
+					     STORED_PAYLOAD_SIZE];
 				struct iphdr *ip =
 				    (struct iphdr *) payload;
 
@@ -433,7 +450,7 @@ void send_auth_response(gpointer packet_id_ptr, gpointer userdata)
 				ip->ihl = IPHDR_REJECT_LENGTH_BWORD;
 				ip->tot_len =
 				    htons(IPHDR_REJECT_LENGTH +
-					  PAYLOAD_SAMPLE);
+					  element->payload_len);
 				ip->ttl = 64;	/* write dummy ttl */
 				ip->protocol = element->tracking.protocol;
 				/* dummy convert to IPv4 as nufw on the other side does not support IPv6 at all */
@@ -444,8 +461,8 @@ void send_auth_response(gpointer packet_id_ptr, gpointer userdata)
 
 				/* write transport layer */
 				memcpy(payload + IPHDR_REJECT_LENGTH,
-				       element->tracking.payload,
-				       PAYLOAD_SAMPLE);
+				       element->payload,
+				       element->payload_len);
 
 				/* write icmp reject packet */
 				memcpy((char *) response +
@@ -454,12 +471,11 @@ void send_auth_response(gpointer packet_id_ptr, gpointer userdata)
 				       payload, payload_size);
 			}
 
-			buffer = (void *) response;
 		}
 		break;
 	case PROTO_VERSION_NUFW_V22_2:
 		{
-			nuv4_nuauth_decision_response_t *response = NULL;
+			nuv4_nuauth_decision_response_t *response = (nuv4_nuauth_decision_response_t *) buffer;
 			int use_icmp6 = 0;
 			uint32_t mark = element->mark;
 
@@ -471,17 +487,16 @@ void send_auth_response(gpointer packet_id_ptr, gpointer userdata)
 				if (use_icmp6)
 					payload_size =
 					    IP6HDR_REJECT_LENGTH +
-					    PAYLOAD6_SAMPLE;
+					    element->payload_len;
 				else
 					payload_size =
 					    IPHDR_REJECT_LENGTH +
-					    PAYLOAD_SAMPLE;
+					    element->payload_len;
 			}
 			/* allocate */
 			total_size =
 			    sizeof(nuv4_nuauth_decision_response_t) +
 			    payload_size;
-			response = (void*)g_new0(char, total_size);
 			response->protocol_version = PROTO_VERSION_NUFW_V22_2;
 			response->msg_type = AUTH_ANSWER;
 			response->tcmark = htonl(mark);
@@ -493,7 +508,7 @@ void send_auth_response(gpointer packet_id_ptr, gpointer userdata)
 			if (element->decision == DECISION_REJECT) {
 				if (use_icmp6) {
 					char payload[IP6HDR_REJECT_LENGTH +
-						     PAYLOAD6_SAMPLE];
+						     STORED_PAYLOAD_SIZE];
 					struct ip6_hdr *ip =
 					    (struct ip6_hdr *) payload;
 
@@ -513,8 +528,8 @@ void send_auth_response(gpointer packet_id_ptr, gpointer userdata)
 					/* write transport layer */
 					memcpy(payload +
 					       IP6HDR_REJECT_LENGTH,
-					       element->tracking.payload,
-					       PAYLOAD6_SAMPLE);
+					       element->payload,
+					       element->payload_len);
 
 					/* write icmp reject packet */
 					memcpy((char *) response +
@@ -523,7 +538,7 @@ void send_auth_response(gpointer packet_id_ptr, gpointer userdata)
 					       payload, payload_size);
 				} else {
 					char payload[IPHDR_REJECT_LENGTH +
-						     PAYLOAD_SAMPLE];
+						     STORED_PAYLOAD_SIZE];
 					struct iphdr *ip =
 					    (struct iphdr *) payload;
 
@@ -535,7 +550,7 @@ void send_auth_response(gpointer packet_id_ptr, gpointer userdata)
 					    IPHDR_REJECT_LENGTH_BWORD;
 					ip->tot_len =
 					    htons(IPHDR_REJECT_LENGTH +
-						  PAYLOAD_SAMPLE);
+						  element->payload_len);
 					ip->ttl = 64;	/* write dummy ttl */
 					ip->protocol =
 					    element->tracking.protocol;
@@ -547,8 +562,8 @@ void send_auth_response(gpointer packet_id_ptr, gpointer userdata)
 					/* write transport layer */
 					memcpy(payload +
 					       IPHDR_REJECT_LENGTH,
-					       element->tracking.payload,
-					       PAYLOAD_SAMPLE);
+					       element->payload,
+					       element->payload_len);
 
 					/* write icmp reject packet */
 					memcpy((char *) response +
@@ -558,7 +573,6 @@ void send_auth_response(gpointer packet_id_ptr, gpointer userdata)
 				}
 			}
 
-			buffer = (void *) response;
 		}
 		break;
 	default:
@@ -568,10 +582,9 @@ void send_auth_response(gpointer packet_id_ptr, gpointer userdata)
 	}
 
 	if (DEBUG_OR_NOT (DEBUG_LEVEL_DEBUG, DEBUG_AREA_PACKET)) {
-		print_connection(element, "Answ Packet");
+		print_connection_wid(element, "Answ Packet", TRUE, packet_id);
 	}
 	ret = nufw_session_send(element->tls, buffer, total_size);
-	g_free(buffer);
 	if (ret != NU_EXIT_OK) {
 		declare_dead_nufw_session(element->tls);
 	} else {
