@@ -1,13 +1,12 @@
 /*
- ** Copyright(C) 2004-2008 INL
+ ** Copyright(C) 2004-2009 INL
  ** Written by  Eric Leblond <regit@inl.fr>
  **             Vincent Deffontaines <gryzor@inl.fr>
- **
- ** $Id: tls_user.c 5487 2009-02-03 14:26:42Z regit $
+ **             Pierre Chifflier <chifflier@inl.fr>
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,6 +20,10 @@
  */
 
 #include "auth_srv.h"
+#include "tls.h"
+#include <fcntl.h>
+
+#include "nuauthconf.h"
 
 /**
  * \ingroup TLS
@@ -28,13 +31,13 @@
  * @{
  */
 
-/** \file tls_user.c
- *  \brief Manage clients connections and messages.
+/**
+ * \brief Handle phase after authentication and till client is active. Defined in tls_sasl.c
  *
- * The thread tls_user_authsrv() wait for clients in tls_user_main_loop().
+ * It also handle preclient list to be able to disconnect user if authentication take too long.
  */
 
-extern int nuauth_tls_auth_by_cert;
+extern struct nuauth_tls_t nuauth_tls;
 
 /**
  * List of new clients which are in authentication state. This list is
@@ -140,50 +143,52 @@ void* pre_client_check(GMutex *mutex)
  * authentication threads.
  *
  * \param c_session SSL RX packet
+ * \param c_data pointer that will point to the parsed data
  * \return a nu_error_t::, NU_EXIT_CONTINUE if read done, NU_EXIT_OK if read complete, NU_EXIT_ERROR on error
  */
-static nu_error_t treat_user_request(user_session_t * c_session)
+nu_error_t treat_user_request(user_session_t * c_session,
+				     struct tls_buffer_read **c_data)
 {
-	struct tls_buffer_read *datas;
 	int header_length;
 	struct nu_header *header;
+	struct tls_buffer_read *data;
 
 	if (c_session == NULL)
 		return NU_EXIT_ERROR;
 
-	datas = g_new0(struct tls_buffer_read, 1);
-	if (datas == NULL)
+	data = g_new0(struct tls_buffer_read, 1);
+	if (data == NULL)
 		return NU_EXIT_ERROR;
-	datas->socket = 0;
-	datas->tls = c_session->tls;
-	datas->ip_addr = c_session->addr;
-	datas->client_version = c_session->client_version;
+	data->socket = 0;
+	data->ip_addr = c_session->addr;
+	data->proto_version = c_session->proto_version;
+	data->auth_quality = c_session->auth_quality;
 
-	/* copy packet datas */
-	datas->buffer = g_new0(char, CLASSIC_NUFW_PACKET_SIZE);
-	if (datas->buffer == NULL) {
-		g_free(datas);
+	/* copy packet data */
+	data->buffer = g_new0(char, CLASSIC_NUFW_PACKET_SIZE);
+	if (data->buffer == NULL) {
+		g_free(data);
 		return NU_EXIT_ERROR;
 	}
 	g_mutex_lock(c_session->tls_lock);
-	datas->buffer_len =
-	    gnutls_record_recv(*(c_session->tls), datas->buffer,
+	data->buffer_len = nussl_read(c_session->nussl, data->buffer,
 			       CLASSIC_NUFW_PACKET_SIZE);
+
 	g_mutex_unlock(c_session->tls_lock);
-	if (datas->buffer_len < (int) sizeof(struct nu_header)) {
+	if (data->buffer_len < (int) sizeof(struct nu_header)) {
 #ifdef DEBUG_ENABLE
-		if (datas->buffer_len <= 0)
+		if (data->buffer_len <= 0)
 			log_message(DEBUG, DEBUG_AREA_USER,
-				    "Received error from user \"%s\": %s",
-				    c_session->user_name,
-				    gnutls_strerror(datas->buffer_len));
+				    "Received error from user %s (%s)",
+				    c_session->user_name, nussl_get_error(c_session->nussl));
 #endif
-		free_buffer_read(datas);
+		free_buffer_read(data);
 		return NU_EXIT_OK;
 	}
 
-	/* get header to check if we need to get more datas */
-	header = (struct nu_header *) datas->buffer;
+
+	/* get header to check if we need to get more data */
+	header = (struct nu_header *) data->buffer;
 	header_length = ntohs(header->length);
 
 	/* is it an "USER HELLO" message ? */
@@ -192,83 +197,129 @@ static nu_error_t treat_user_request(user_session_t * c_session)
 		debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_USER,
 				  "tls user: HELLO from user \"%s\"",
 				  c_session->user_name);
-		free_buffer_read(datas);
+		free_buffer_read(data);
 		return NU_EXIT_CONTINUE;
 	}
 
 	/* if message content is bigger than CLASSIC_NUFW_PACKET_SIZE, */
 	/* continue to read the content */
 	if (header->proto == PROTO_VERSION
-	    && header_length > datas->buffer_len
+	    && header_length > data->buffer_len
 	    && header_length < MAX_NUFW_PACKET_SIZE) {
 		int tmp_len;
 
 		/* we realloc and get what we miss */
-		datas->buffer = g_realloc(datas->buffer, header_length);
-		header = (struct nu_header *) datas->buffer;
+		data->buffer = g_realloc(data->buffer, header_length);
+		header = (struct nu_header *) data->buffer;
 
 		g_mutex_lock(c_session->tls_lock);
-		tmp_len =
-		    gnutls_record_recv(*(c_session->tls),
-				       datas->buffer +
+		tmp_len = nussl_read(c_session->nussl,
+				       data->buffer +
 				       CLASSIC_NUFW_PACKET_SIZE,
-				       header_length - datas->buffer_len);
+				       header_length - data->buffer_len);
 		g_mutex_unlock(c_session->tls_lock);
 		if (tmp_len <= 0) {
-			free_buffer_read(datas);
+			free_buffer_read(data);
 			return NU_EXIT_ERROR;
 		}
-		datas->buffer_len += tmp_len;
+		data->buffer_len += tmp_len;
 	}
 
 	/* check message type because USER_HELLO has to be ignored */
 	if (header->msg_type == USER_HELLO) {
-		free_buffer_read(datas);
+		free_buffer_read(data);
 		return NU_EXIT_CONTINUE;
 	}
 
+	/* looks like a regular auth attempt, update last_request */
+	c_session->last_request = time(NULL);
+
 	/* check authorization if we're facing a multi user packet */
 	if (header->option == 0x0) {
 		/* this is an authorized packet we fill the buffer_read structure */
-		datas->user_name = g_strdup(c_session->user_name);
-		datas->user_id = c_session->user_id;
-		datas->groups = g_slist_copy(c_session->groups);
+		data->user_name = g_strdup(c_session->user_name);
+		data->user_id = c_session->user_id;
+		data->groups = g_slist_copy(c_session->groups);
 		if (c_session->sysname) {
-			datas->os_sysname = g_strdup(c_session->sysname);
-			if (datas->os_sysname == NULL) {
-				free_buffer_read(datas);
+			data->os_sysname = g_strdup(c_session->sysname);
+			if (data->os_sysname == NULL) {
+				free_buffer_read(data);
 				return NU_EXIT_ERROR;
 			}
 		}
 		if (c_session->release) {
-			datas->os_release = g_strdup(c_session->release);
-			if (datas->os_release == NULL) {
-				free_buffer_read(datas);
+			data->os_release = g_strdup(c_session->release);
+			if (data->os_release == NULL) {
+				free_buffer_read(data);
 				return NU_EXIT_ERROR;
 			}
 		}
 		if (c_session->version) {
-			datas->os_version = g_strdup(c_session->version);
-			if (datas->os_version == NULL) {
-				free_buffer_read(datas);
+			data->os_version = g_strdup(c_session->version);
+			if (data->os_version == NULL) {
+				free_buffer_read(data);
 				return NU_EXIT_ERROR;
 			}
 		}
-
-		debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN | DEBUG_AREA_USER,
-				  "Pushing packet to user_checker");
-		thread_pool_push(nuauthdatas->user_checkers, datas,
-				   NULL);
 	} else {
 		log_message(INFO, DEBUG_AREA_USER,
 			    "Bad packet, option of header is not set or unauthorized option from user \"%s\".",
 			    c_session->user_name);
-		free_buffer_read(datas);
+		free_buffer_read(data);
 		return NU_EXIT_OK;
 	}
+
+	*c_data = data;
 	return NU_EXIT_CONTINUE;
 }
 
+/**
+ * Function called by client sasl thread, to complete TLS handshake
+ *    - Call nussl_session_handshake()
+ *    - Check client certificate
+ *
+ * \return If an error occurs returns 1, else returns 0.
+ */
+int tls_user_do_handshake(struct client_connection *current_client_conn, struct tls_user_context_t *context)
+{
+	int ret;
+	char cipher[256];
+
+	/* do not verify FQDN field from client */
+	nussl_set_session_flag(current_client_conn->nussl,
+		NUSSL_SESSFLAG_IGNORE_ID_MISMATCH,
+		1
+		);
+
+	// XXX default value is 30s, should be a configuration value
+	nussl_set_connect_timeout(current_client_conn->nussl, 30);
+
+	ret = nussl_session_handshake(current_client_conn->nussl,context->nussl);
+	if ( ret ) {
+		log_message(WARNING, DEBUG_AREA_MAIN | DEBUG_AREA_USER,
+			    "New client connection from %s failed during nussl_session_handshake(): %s",
+			    current_client_conn->str_addr,
+			    nussl_get_error(context->nussl));
+		return 1;
+	}
+
+	nussl_session_get_cipher(current_client_conn->nussl, cipher, sizeof(cipher));
+	log_message(INFO, DEBUG_AREA_MAIN | DEBUG_AREA_USER,
+		    "TLS handshake with client %s succeeded, cipher is %s",
+		    current_client_conn->str_addr, cipher);
+
+	/* Check certificate hook */
+	ret = modules_check_certificate(current_client_conn->nussl);
+	if ( ret ) {
+		log_message(WARNING, DEBUG_AREA_MAIN | DEBUG_AREA_USER,
+			    "New client connection from %s failed during modules_check_certificate()",
+			    current_client_conn->str_addr);
+		return 1;
+	}
+
+	return 0;
+}
+
 /**
  * Function called on new client connection:
  *    - Call accept()
@@ -291,14 +342,28 @@ int tls_user_accept(struct tls_user_context_t *context)
 	int socket;
 	gint option_value;
 	unsigned short sport;
+	char address[INET6_ADDRSTRLEN];
 
-	/* Wait for a connect */
-	socket = accept(context->sck_inet,
-			(struct sockaddr *) &sockaddr, &len_inet);
-	if (socket == -1) {
-		log_message(WARNING, DEBUG_AREA_USER, "accept");
+	current_client_conn = g_new0(struct client_connection, 1);
+
+	current_client_conn->nussl = nussl_session_accept(context->nussl);
+	if ( ! current_client_conn->nussl ) {
+		log_message(WARNING, DEBUG_AREA_MAIN | DEBUG_AREA_USER,
+			    "New client connection failed during nussl_session_accept(): %s", nussl_get_error(context->nussl));
+		g_free(current_client_conn);
+		return 1;
 	}
 
+	if (nussl_session_getpeer(current_client_conn->nussl, (struct sockaddr *) &sockaddr, &len_inet) != NUSSL_OK)
+	{
+		log_message(WARNING, DEBUG_AREA_MAIN | DEBUG_AREA_USER,
+			    "New client connection failed during nussl_session_getpeer(): %s", nussl_get_error(context->nussl));
+		g_free(current_client_conn);
+		return 1;
+	}
+
+	socket = nussl_session_get_fd(current_client_conn->nussl);
+
 	/* if system is in reload: drop new client */
 	if (nuauthdatas->need_reload) {
 		shutdown(socket, SHUT_RDWR);
@@ -306,6 +371,21 @@ int tls_user_accept(struct tls_user_context_t *context)
 		return 0;
 	}
 
+	/* Extract client address (convert it to IPv6 if it's IPv4) */
+	/* if (sockaddr.ss_family == AF_INET) { -> same as tls_nufw.c */
+	if (sockaddr6->sin6_family == AF_INET) {
+		ipv4_to_ipv6(sockaddr4->sin_addr, &addr);
+		sport = ntohs(sockaddr4->sin_port);
+	} else {
+		addr = sockaddr6->sin6_addr;
+		sport = ntohs(sockaddr6->sin6_port);
+	}
+
+	format_ipv6(&addr, address, sizeof(address), NULL);
+	log_message(DEBUG, DEBUG_AREA_MAIN | DEBUG_AREA_USER,
+			"nuauth: user connection attempt from %s\n",
+			address);
+
 	if (get_number_of_clients() >= context->nuauth_tls_max_clients) {
 		log_message(WARNING, DEBUG_AREA_MAIN | DEBUG_AREA_USER,
 			    "too many clients (%d configured)",
@@ -315,19 +395,11 @@ int tls_user_accept(struct tls_user_context_t *context)
 		return 1;
 	}
 
-	/* Extract client address (convert it to IPv6 if it's IPv4) */
-	if (sockaddr.ss_family == AF_INET) {
-		ipv4_to_ipv6(sockaddr4->sin_addr, &addr);
-		sport = ntohs(sockaddr4->sin_port);
-	} else {
-		addr = sockaddr6->sin6_addr;
-		sport = ntohs(sockaddr6->sin6_port);
-	}
-
-	current_client_conn = g_new0(struct client_connection, 1);
 	current_client_conn->socket = socket;
 	current_client_conn->addr = addr;
 	current_client_conn->sport = sport;
+	current_client_conn->str_addr = g_strdup(address);
+	current_client_conn->srv_context = context;
 
 	/* Set KEEP ALIVE on connection */
 	option_value = 1;
@@ -363,7 +435,6 @@ void tls_user_check_activity(struct tls_user_context_t *context,
 			     int socket)
 {
 	user_session_t *c_session;
-	int u_request;
 	debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_USER,
 			  "user activity on socket %d", socket);
 
@@ -371,26 +442,20 @@ void tls_user_check_activity(struct tls_user_context_t *context,
 	 * in push mode modification of hash are done in push_worker */
 	c_session = get_client_datas_by_socket(socket);
 
+	if (c_session == NULL) {
+		log_message(INFO, DEBUG_AREA_MAIN | DEBUG_AREA_USER,
+			  "User session can not be found");
+		return;
+	}
+
 	if (nuauthconf->session_duration && c_session->expire < time(NULL)) {
 		delete_client_by_socket(socket);
 		return;
 	}
 
-	u_request = treat_user_request(c_session);
-	if (u_request == NU_EXIT_OK) {
-		debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_USER,
-				  "client disconnect on socket %d",
-				  socket);
-		/* clean client structure */
-		delete_client_by_socket(socket);
-	} else if (u_request != NU_EXIT_CONTINUE) {
-#ifdef DEBUG_ENABLE
-		log_message(VERBOSE_DEBUG, DEBUG_AREA_USER,
-			    "treat_user_request() failure");
-#endif
-		/* better to disconnect: cleaning client structure */
-		delete_client_by_socket(socket);
-	}
+	debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN | DEBUG_AREA_USER,
+			  "Pushing packet to user_checker");
+	thread_pool_push(nuauthdatas->user_checkers, c_session, NULL);
 }
 
 /**
@@ -443,6 +508,21 @@ void tls_user_main_loop(struct tls_user_context_t *context, GMutex * mutex)
 	struct timeval tv;
 	disconnect_user_msg_t *disconnect_msg;
 
+	/* create unix pipe */
+	if (pipe(user_pipefd) == -1) {
+		log_message(CRITICAL, DEBUG_AREA_MAIN,
+		    "[+] Unable to open user pipe.");
+		nuauth_ask_exit();
+	}
+	if (fcntl(user_pipefd[0], F_SETFL, (fcntl(user_pipefd[0], F_GETFL)|O_NONBLOCK))) {
+		log_message(CRITICAL, DEBUG_AREA_MAIN,
+		    "[+] Unable to set pipe to non-blocking.");
+		nuauth_ask_exit();
+	}
+
+	FD_SET(user_pipefd[0], &context->tls_rx_set);
+
+
 	log_message(INFO, DEBUG_AREA_USER,
 		    "[+] NuAuth is waiting for client connections.");
 	while (g_mutex_trylock(mutex)) {
@@ -453,7 +533,7 @@ void tls_user_main_loop(struct tls_user_context_t *context, GMutex * mutex)
 		 * tls_sasl_connect_ok() and are send when a new user is connected.
 		 */
 		c_pop = g_async_queue_try_pop(mx_queue);
-		while (c_pop != NULL) {
+		while (c_pop) {
 			int socket = GPOINTER_TO_INT(c_pop);
 
 			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_USER,
@@ -474,28 +554,11 @@ void tls_user_main_loop(struct tls_user_context_t *context, GMutex * mutex)
 		 */
 		while ((disconnect_msg = g_async_queue_try_pop(context->cmd_queue)) != NULL){
 			if (disconnect_msg->socket == -1) {
-				nu_error_t ret;
-				for (i = 0; i < context->mx; ++i) {
-					if (FD_ISSET(i, &context->tls_rx_set) &&
-					    context->sck_inet != i) {
-						ret = delete_client_by_socket(i);
-						if (ret != NU_EXIT_OK) {
-							break;
-						}
-					}
-				}
-				disconnect_msg->result = ret;
+				disconnect_msg->result = kill_all_clients();
 			} else {
-				if (FD_ISSET(disconnect_msg->socket,
-					     &context->tls_rx_set) &&
-				    context->sck_inet != disconnect_msg->socket) {
-					disconnect_msg->result =
-						delete_client_by_socket(disconnect_msg->socket);
-				} else {
-					disconnect_msg->result = NU_EXIT_CONTINUE;
-				}
+				disconnect_msg->result = delete_client_by_socket(disconnect_msg->socket);
 			}
-			g_cond_signal(disconnect_msg->cond);
+			g_mutex_unlock(disconnect_msg->mutex);
 		}
 
 		/* wait new events during 1 second */
@@ -503,6 +566,9 @@ void tls_user_main_loop(struct tls_user_context_t *context, GMutex * mutex)
 		for (i = 0; i < context->mx; ++i) {
 			if (FD_ISSET(i, &context->tls_rx_set))
 				FD_SET(i, &wk_set);
+			if (i == user_pipefd[0]) {
+				FD_SET(i, &wk_set);
+			}
 		}
 		tv.tv_sec = 0;
 		tv.tv_usec = 250000;
@@ -548,14 +614,29 @@ void tls_user_main_loop(struct tls_user_context_t *context, GMutex * mutex)
 					continue;
 			}
 
+			if (FD_ISSET(user_pipefd[0], &wk_set)) {
+				int32_t gb_socket;
+				while (read(user_pipefd[0], &gb_socket, sizeof(gb_socket)) >0) {
+					debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_USER,
+							  "FD %d return to working set", gb_socket);
+					FD_SET(gb_socket, &context->tls_rx_set);
+				}
+				if (gb_socket + 1 > context->mx)
+					context->mx = gb_socket + 1;
+				continue;
+			}
 			/*
 			 * check for client activity
 			 */
 			for (i = 0; i < context->mx; ++i) {
 				if (i == context->sck_inet)
 					continue;
-				if (FD_ISSET(i, &wk_set))
+				if (FD_ISSET(i, &wk_set)) {
 					tls_user_check_activity(context, i);
+					/* remove socket from wk_set it will be added
+					 * back by user checker. */
+					FD_CLR(i, &context->tls_rx_set);
+				}
 			}
 		}
 		tls_user_update_mx(context);
@@ -580,64 +661,91 @@ void tls_user_servers_init()
 	/* create tls sasl worker thread pool */
 	nuauthdatas->tls_sasl_worker =
 	    g_thread_pool_new((GFunc) tls_sasl_connect, NULL,
-			      nuauthconf->nb_auth_checkers, TRUE,
+			      nuauthconf->nb_auth_checkers, FALSE,
 			      NULL);
 }
 
+/**
+ * Set request_cert and auth_by_cert params depending on the configuration
+ */
+int tls_user_setcert_auth_params(int requestcert, int authcert)
+{
+	int disable_request_warning;
+
+	disable_request_warning = nuauth_config_table_get_or_default_int("nuauth_tls_disable_request_warning", FALSE);
+
+	nuauth_tls.auth_by_cert = authcert;
+
+	if (NUSSL_VALID_REQ_TYPE(requestcert)) {
+		nuauth_tls.request_cert = requestcert;
+	} else {
+		log_message(INFO, DEBUG_AREA_AUTH | DEBUG_AREA_USER,
+				"[%i] config: Invalid nuauth_tls_auth_by_cert value: %d",
+				getpid(), authcert);
+		return 0;
+	}
+
+	if ((nuauth_tls.auth_by_cert == MANDATORY_AUTH_BY_CERT)
+	&& (nuauth_tls.request_cert != NUSSL_CERT_REQUIRE)) {
+		log_message(INFO, DEBUG_AREA_AUTH | DEBUG_AREA_USER,
+			    "Mandatory certificate authentication asked, asking certificate");
+		nuauth_tls.request_cert = NUSSL_CERT_REQUIRE;
+	}
+
+	/* always ask for certificates - but don't error if none were sent */
+	if (nuauth_tls.request_cert == 0)
+		nuauth_tls.request_cert = 1;
+
+	log_message(INFO, DEBUG_AREA_AUTH | DEBUG_AREA_USER,"request_cert = %i", nuauth_tls.request_cert);
+	log_message(INFO, DEBUG_AREA_AUTH | DEBUG_AREA_USER,"auth_by_cert = %i", nuauth_tls.auth_by_cert);
+
+	if (!disable_request_warning) {
+		if (nuauth_tls.request_cert != 2) {
+			g_warning ("[%i] nuauth: client certificates are not required\n"
+				"nuauth will *NOT* check client certificates.\n"
+				"Set nuauth_tls_request_cert=2 to request certificates.\n",
+				getpid());
+		} else {
+			log_message(INFO, DEBUG_AREA_AUTH | DEBUG_AREA_USER,
+				    "Client certificates are required.");
+		}
+	}
+
+	return 1;
+}
+
 /**
  * Create TLS user context.
  */
 int tls_user_init(struct tls_user_context_t *context)
 {
-	confparams_t nuauth_tls_vars[] = {
-		{"nuauth_tls_max_clients", G_TOKEN_INT,
-		 NUAUTH_TLS_MAX_CLIENTS, NULL},
-		{"nuauth_auth_nego_timeout", G_TOKEN_INT,
-		 AUTH_NEGO_TIMEOUT, NULL}
-	};
 	char *errmsg;
 	int result;
 
+	int ret;
+
+	/*const unsigned int nb_params = sizeof(nuauth_tls_vars) / sizeof(confparams_t);*/
+	int int_authcert;
+	int int_requestcert;
+	char *dh_params_file;
+
 	context->sck_inet = nuauth_bind(&errmsg, context->addr, context->port, "user");
 	if (context->sck_inet < 0) {
 		log_message(FATAL, DEBUG_AREA_MAIN | DEBUG_AREA_USER,
 			    "FATAL ERROR: User bind error: %s", errmsg);
 		log_message(FATAL, DEBUG_AREA_MAIN | DEBUG_AREA_USER,
 			    "Check that nuauth is not running twice. nuauth exiting!");
-		return 0;
+		exit(EXIT_FAILURE);
 	}
 
-	/* get config file setup */
-	/* parse conf file */
-	if (!parse_conffile(DEFAULT_CONF_FILE,
-		       sizeof(nuauth_tls_vars) / sizeof(confparams_t),
-		       nuauth_tls_vars)) {
-	        log_message(FATAL, DEBUG_AREA_MAIN, "Failed to load config file %s", DEFAULT_CONF_FILE);
-		return 0;
-	}
-
-
-#define READ_CONF(KEY) \
-	get_confvar_value(nuauth_tls_vars, sizeof(nuauth_tls_vars)/sizeof(confparams_t), KEY)
-
-	context->nuauth_tls_max_clients =
-	    *(unsigned int *) READ_CONF("nuauth_tls_max_clients");
-	context->nuauth_auth_nego_timeout =
-	    *(int *) READ_CONF("nuauth_auth_nego_timeout");
-#undef READ_CONF
-
-	/* free config struct */
-	free_confparams(nuauth_tls_vars,
-			sizeof(nuauth_tls_vars) / sizeof(confparams_t));
-
 	context->cmd_queue = g_async_queue_new();
 
 	/* listen */
 	result = listen(context->sck_inet, 20);
 	if (result == -1) {
-		g_warning("user listen() failed, exiting");
+		log_message(FATAL, DEBUG_AREA_MAIN,
+			    "user listen() failed, exiting");
 		exit(EXIT_FAILURE);
-		return 0;
 	}
 
 	/* init fd_set */
@@ -645,6 +753,86 @@ int tls_user_init(struct tls_user_context_t *context)
 	FD_SET(context->sck_inet, &context->tls_rx_set);
 	context->mx = context->sck_inet + 1;
 	mx_queue = g_async_queue_new();
+
+	/* Init ssl session */
+	/* TODO: make sure request_cert | auth_by_cert is for user and change to nufw if required */
+	context->nuauth_tls_max_clients = nuauth_config_table_get_or_default_int("nuauth_tls_max_clients", NUAUTH_TLS_MAX_CLIENTS);
+	context->nuauth_auth_nego_timeout = nuauth_config_table_get_or_default_int("nuauth_auth_nego_timeout", NUAUTH_TLS_MAX_CLIENTS);
+	/* ssl related conf */
+	int_requestcert = nuauth_config_table_get_or_default_int("nuauth_tls_request_cert", 2);
+	int_authcert = nuauth_config_table_get_or_default_int("nuauth_tls_auth_by_cert", FALSE);
+	dh_params_file = nuauth_config_table_get("nuauth_tls_dh_params");
+
+	if (!tls_user_setcert_auth_params(int_requestcert, int_authcert)) {
+		log_message(FATAL, DEBUG_AREA_MAIN,
+			    "Invalid request_cert or auth_by_cert option");
+		exit(EXIT_FAILURE);
+	}
+
+	/* We add the crl file function check every second only if we have a crl */
+	if ( nuauth_tls.crl_file ) {
+		cleanup_func_push(refresh_crl_file);
+	}
+
+	context->nussl = nussl_session_create_with_fd(context->sck_inet,
+						      nuauth_tls.request_cert);
+	if (!context->nussl ) {
+		log_message(FATAL, DEBUG_AREA_MAIN,
+			    "Cannot create session from fd!");
+		exit(EXIT_FAILURE);
+	}
+
+	ret = nussl_ssl_set_keypair(context->nussl, nuauth_tls.cert, nuauth_tls.key);
+	if ( ret != NUSSL_OK ) {
+		log_message(FATAL, DEBUG_AREA_MAIN,
+			    "Failed to load user key/certificate: %s",
+			    nussl_get_error(context->nussl));
+		exit(EXIT_FAILURE);
+	}
+
+	ret = nussl_ssl_trust_cert_file(context->nussl, nuauth_tls.ca);
+	if ( ret != NUSSL_OK ) {
+		log_message(FATAL, DEBUG_AREA_MAIN,
+			    "Failed to load user certificate authority: %s",
+			    nussl_get_error(context->nussl));
+		exit(EXIT_FAILURE);
+	}
+
+	if (nuauth_tls.capath) {
+		ret = nussl_ssl_trust_dir(context->nussl, nuauth_tls.capath);
+		if ( ret != NUSSL_OK ) {
+			log_message(FATAL, DEBUG_AREA_MAIN,
+					"Failed to load user certificate authority directory: %s",
+					nussl_get_error(context->nussl));
+			exit(EXIT_FAILURE);
+		}
+	}
+
+	if (nuauth_tls.crl_file) {
+		ret = nussl_ssl_set_crl_file(context->nussl, nuauth_tls.crl_file, nuauth_tls.ca);
+		if ( ret != NUSSL_OK ) {
+			log_message(FATAL, DEBUG_AREA_MAIN,
+					"Failed to load certificate revocation list (CRL): %s",
+					nussl_get_error(context->nussl));
+			exit(EXIT_FAILURE);
+		}
+	}
+
+	ret = NUSSL_ERROR;
+	if (dh_params_file) {
+		ret = nussl_session_set_dh_file(context->nussl, dh_params_file);
+	}
+	if (ret != NUSSL_OK &&
+	    nussl_session_set_dh_bits(context->nussl, DH_BITS) != NUSSL_OK) {
+		log_message(FATAL, DEBUG_AREA_MAIN,
+			    "Unable to initialize Diffie Hellman params.");
+		exit(EXIT_FAILURE);
+	}
+
+	if (nuauth_tls.ciphers) {
+		nussl_session_set_ciphers(context->nussl, nuauth_tls.ciphers);
+	}
+
 	return 1;
 }
 
@@ -689,10 +877,10 @@ void *push_worker(GMutex * mutex)
 		switch (message->type) {
 		case WARN_MESSAGE:
 			global_msg->addr =
-			    ((tracking_t *) message->datas)->saddr;
+			    (((auth_pckt_t *) message->datas)->header).saddr;
 			global_msg->found = FALSE;
 			/* search in client array */
-			warn_clients(global_msg);
+			warn_clients(global_msg, NULL, NULL);
 			/* do we have found something */
 			if (!ipv6_equal(&global_msg->addr, &in6addr_any)) {
 				if (global_msg->found == FALSE) {
@@ -715,13 +903,12 @@ void *push_worker(GMutex * mutex)
 
 		case INSERT_MESSAGE:
 			{
-				struct tls_insert_data *datas =
-				    message->datas;
-				if (datas->data) {
-					add_client(datas->socket,
-						   datas->data);
+				struct tls_insert_data *data = message->datas;
+				if (data->data) {
+					add_client(data->socket,
+						   data->data);
 				}
-				g_free(datas);
+				g_free(data);
 			}
 			break;
 		default:
@@ -747,7 +934,10 @@ void *push_worker(GMutex * mutex)
 void *tls_user_authsrv(struct nuauth_thread_t *thread)
 {
 	struct tls_user_context_t *context = thread->data;
-	int ok = tls_user_init(context);
+	int ok = 0;
+
+	ok = tls_user_init(context);
+
 	if (ok) {
 		tls_user_main_loop(context, thread->mutex);
 	} else {
@@ -792,6 +982,43 @@ void tls_user_start_servers(GSList *servers)
 	g_strfreev(user_servers);
 }
 
+/**
+ * Refresh crl in the user contexts
+ *
+ */
+
+void tls_crl_update_user_session(GSList *session)
+{
+
+	GSList *listrunner = session;
+	int ret;
+
+	while ( listrunner ) {
+		struct nuauth_thread_t *nuauth_thread = listrunner->data;
+		struct tls_user_context_t *context = nuauth_thread->data;
+
+		// Don't update the CRL when nufw is not yet connected
+		if (context->nussl == NULL) {
+			listrunner = g_slist_next(listrunner);
+			continue;
+		}
+
+		ret = nussl_ssl_set_crl_file(context->nussl, nuauth_tls.crl_file, nuauth_tls.ca);
+
+		if (ret != NUSSL_OK) {
+			log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_CRITICAL,
+					"[%i] User TLS: CRL file reloading failed (%s)",
+					getpid(), nussl_get_error(context->nussl));
+		}
+
+		listrunner = g_slist_next(listrunner);
+
+	}
+	g_slist_free(listrunner);
+
+}
+
+
 /**
  * @}
  */
