@@ -1,13 +1,13 @@
 /*
- ** Copyright(C) 2004-2008 INL http://www.inl.fr/
+ ** Copyright(C) 2004-2009 INL http://www.inl.fr/
  ** Written by  Eric Leblond <regit@inl.fr>
  **             Vincent Deffontaines <vincent@inl.fr>
  **
- ** $Id: user_authsrv.c 5500 2009-02-03 21:44:56Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,32 +24,65 @@
 #include <sys/time.h>
 #include <sasl/saslutil.h>
 
-static GSList *userpckt_decode(struct tls_buffer_read *datas);
+#define RETURN_NO_LOG return
+
+static GSList *userpckt_decode(struct tls_buffer_read *data);
 
 /**
- * Get user datas (containing datagram) and goes till inclusion
+ * Get user data (containing datagram) and goes till inclusion
  * (or decision) on packet.
  *
  * Call userpckt_decode()
  *
- * \param userdata Pointer to a struct tls_buffer_read: containing the datagram
+ * \param user_session Pointer to a struct user_session_t: containing the data to treat
  * \param data NULL (unused)
  */
-void user_check_and_decide(gpointer userdata, gpointer data)
+void user_check_and_decide(gpointer user_session, gpointer data)
 {
 	GSList *conn_elts = NULL;
 	GSList *conn_elt_l;
 	connection_t *conn_elt;
+	struct tls_buffer_read *userdata = NULL;
+	nu_error_t u_request;
+	user_session_t *usersession = user_session;
 
 	debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_USER, "entering user_check");
 
+	/* Call treat user request */
+	u_request = treat_user_request(usersession, &userdata);
+	if (u_request == NU_EXIT_OK) {
+		debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_USER,
+				  "client disconnect");
+		/* clean client structure */
+		delete_client_by_socket(usersession->socket);
+		RETURN_NO_LOG;
+	} else if (u_request != NU_EXIT_CONTINUE) {
+#ifdef DEBUG_ENABLE
+		log_message(VERBOSE_DEBUG, DEBUG_AREA_USER,
+			    "treat_user_request() failure");
+#endif
+		/* better to disconnect: cleaning client structure */
+		delete_client_by_socket(usersession->socket);
+		RETURN_NO_LOG;
+	}
+
+
+	/* send socket back to user select */
+	write(user_pipefd[1], &usersession->socket, sizeof(usersession->socket));
+
+	if (userdata == NULL) {
+		RETURN_NO_LOG;
+	}
+
 	/* reload condition */
 	conn_elts = userpckt_decode(userdata);
 
 	if (conn_elts == NULL) {
+		if (((struct nu_header *) userdata->buffer)->msg_type != EXTENDED_PROTO) {
+			log_message(INFO, DEBUG_AREA_USER,
+					"User packet decoding failed");
+		}
 		free_buffer_read(userdata);
-		log_message(INFO, DEBUG_AREA_USER,
-			    "User packet decoding failed");
 		return;
 	}
 
@@ -78,7 +111,7 @@ void user_check_and_decide(gpointer userdata, gpointer data)
 				if (DEBUG_OR_NOT
 						(DEBUG_LEVEL_INFO, DEBUG_AREA_USER)) {
 					char ip_ascii[INET6_ADDRSTRLEN];
-					FORMAT_IPV6(&tlsdata->ip_addr, ip_ascii);
+					format_ipv6(&tlsdata->ip_addr, ip_ascii, INET6_ADDRSTRLEN, NULL);
 					g_message
 						("User \"%s\" on %s tried to authenticate packet from other ip",
 						 conn_elt->username, ip_ascii);
@@ -97,7 +130,6 @@ void user_check_and_decide(gpointer userdata, gpointer data)
 		}
 		g_async_queue_push(nuauthdatas->connections_queue,
 				conn_elt);
-
 	}
 	g_slist_free(conn_elts);
 	free_buffer_read(userdata);
@@ -204,9 +236,9 @@ int user_process_field_app(struct nu_authreq *authreq,
 	if ((ret = sasl_decode64((char *) appfield + 4, len, dec_appname, len, &reallen)) != SASL_OK) {
 		log_message(INFO, DEBUG_AREA_USER,
 			    "user packet announced a badly encoded app name, sasl_error %d", ret);
-		if (ret == SASL_BADPROT) {
-			log_message(WARNING, DEBUG_AREA_USER, "User \"%s\" is sending buggy packets, try upgrading its client.", connection->username);
-		}
+		if (ret == SASL_BADPROT)
+			log_message(INFO, DEBUG_AREA_USER, "Try upgrading your client");
+
 		g_free(dec_appname);
 		return -1;
 	}
@@ -225,10 +257,43 @@ int user_process_field_app(struct nu_authreq *authreq,
 		connection->app_name = NULL;
 	}
 	g_free(dec_appname);
-	connection->app_md5 = NULL;
 	return 1;
 }
 
+int user_process_field_hash(struct nu_authreq *authreq,
+			   connection_t * connection,
+			   int field_buffer_len,
+			   struct nu_authfield_app *appfield)
+{
+	gchar *appsig = NULL;
+	unsigned int len = appfield->length -  sizeof(struct nu_authfield_app);
+
+	debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_USER, "\tgot HASH field");
+
+	/* this has to be smaller than field size */
+	if (field_buffer_len < (int) appfield->length) {
+		log_message(WARNING, DEBUG_AREA_USER,
+			    "Improper application field length signaled in authreq header %d < %d",
+			    field_buffer_len, appfield->length);
+		return -1;
+	}
+
+	if (len > 256 || (len <= 0)) {
+		/* it is reaaally long (or too short), we ignore packet (too lasy to kill client) */
+		log_message(INFO, DEBUG_AREA_USER,
+			    "user packet announced a bad length app name : %d",
+			    len);
+		return -1;
+	}
+
+	appsig = g_new0(gchar, len + 1);
+
+	memcpy(appsig, (char *)appfield + sizeof(struct nu_authfield_app), len);
+	appsig[len] = 0;
+
+	connection->app_sig = appsig;
+	return 1;
+}
 
 int user_process_field(struct nu_authreq *authreq,
 		       uint8_t header_option,
@@ -247,6 +312,8 @@ int user_process_field(struct nu_authreq *authreq,
 
 	switch (field->type) {
 	case IPV6_FIELD:
+		debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_USER,
+				  "IPV6_FIELD");
 		if (auth_buffer_len <
 				(int) sizeof(struct nu_authfield_ipv6)) {
 			log_message(WARNING, DEBUG_AREA_USER,
@@ -265,7 +332,7 @@ int user_process_field(struct nu_authreq *authreq,
 					"Auth buffer too small for field type");
 			return -1;
 		}
-		switch (connection->client_version) {
+		switch (connection->proto_version) {
 		case PROTO_VERSION_V20:
 			if (user_process_field_ipv4
 			    (connection,
@@ -273,17 +340,20 @@ int user_process_field(struct nu_authreq *authreq,
 				return -1;
 			break;
 		case PROTO_VERSION_V22:
+		case PROTO_VERSION_V24:
 			log_message(WARNING, DEBUG_AREA_USER,
 				    "Proto V4 user sends an IPV4_FIELD");
 			return -1;
 		default:
 			log_message(WARNING, DEBUG_AREA_USER,
 				    "Unknown protocol %d client has sent an IPV4_FIELD",
-				    connection->client_version);
+				    connection->proto_version);
 		}
 		break;
 
 	case APP_FIELD:
+		debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_USER,
+				  "APP_FIELD");
 		if (auth_buffer_len <
 		    (int) sizeof(struct nu_authfield_app)) {
 			return -1;
@@ -294,12 +364,23 @@ int user_process_field(struct nu_authreq *authreq,
 			return -1;
 		break;
 
-	case USERNAME_FIELD:
-		log_message(WARNING, DEBUG_AREA_USER,
-			    "Received USERNAME_FIELD, this is BAD! multiuser client are born-dead");
-		return -1;
+	case HASH_FIELD:
+		debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_USER,
+				  "HASH_FIELD");
+		if (auth_buffer_len <
+		    (int) sizeof(struct nu_authfield_app)) {
+			return -1;
+		}
+		if (user_process_field_hash
+		    (authreq, connection, field->length,
+		     (struct nu_authfield_app *) field) < 0)
+			return -1;
+		break;
+
 
 	case HELLO_FIELD:
+		debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_USER,
+				  "HELLO_FIELD");
 		if (auth_buffer_len <
 		    (int) sizeof(struct nu_authfield_hello)) {
 			return -1;
@@ -320,17 +401,17 @@ int user_process_field(struct nu_authreq *authreq,
 /**
  * \brief Parse user request
  *
- * \param datas Buffer read on a TLS socket
+ * \param data Buffer read on a TLS socket
  * \return Single linked list of connections (of type ::connection_t).
  */
-GSList *user_request(struct tls_buffer_read * datas)
+GSList *user_request(struct tls_buffer_read * data)
 {
-	char *dgram = datas->buffer;
+	char *dgram = data->buffer;
 	struct nu_header *header = (struct nu_header *) dgram;
 	GSList *conn_elts = NULL;
 	connection_t *connection = NULL;
 	char *start;
-	int buffer_len = datas->buffer_len;
+	int buffer_len = data->buffer_len;
 	int auth_buffer_len;
 	int field_length;
 	struct nu_authreq *authreq;
@@ -363,15 +444,15 @@ GSList *user_request(struct tls_buffer_read * datas)
 		connection->acl_groups = NULL;
 		connection->user_groups = NULL;
 		connection->app_name = NULL;
-		connection->app_md5 = NULL;
-		connection->username = g_strdup(datas->user_name);
+		connection->app_sig = NULL;
+		connection->username = NULL;
 		connection->cacheduserdatas = NULL;
 		connection->packet_id = NULL;
 		connection->expire = -1;
 		connection->flags = ACL_FLAGS_NONE;
-		connection->client_version = datas->client_version;
+		connection->proto_version = data->proto_version;
+		connection->auth_quality = data->auth_quality;
 		connection->decision = DECISION_NODECIDE;
-
 #ifdef PERF_DISPLAY_ENABLE
 		if (nuauthconf->debug_areas & DEBUG_AREA_PERF) {
 			gettimeofday(&(connection->arrival_time), NULL);
@@ -430,13 +511,16 @@ GSList *user_request(struct tls_buffer_read * datas)
 		}
 
 		/* here all packet related information are filled-in */
-		connection->user_id = datas->user_id;
-		connection->user_groups = g_slist_copy(datas->groups);
-		connection->os_sysname = g_strdup(datas->os_sysname);
-		connection->os_release = g_strdup(datas->os_release);
-		connection->os_version = g_strdup(datas->os_version);
+		if (connection->username == NULL) {
+			connection->username = g_strdup(data->user_name);
+		}
+		connection->user_id = data->user_id;
+		connection->user_groups = g_slist_copy(data->groups);
+		connection->os_sysname = g_strdup(data->os_sysname);
+		connection->os_release = g_strdup(data->os_release);
+		connection->os_version = g_strdup(data->os_version);
 		/* copy client version information */
-		connection->client_version = datas->client_version;
+		connection->proto_version = data->proto_version;
 
 		if (connection->user_groups == NULL) {
 			log_message(INFO, DEBUG_AREA_USER,
@@ -457,20 +541,55 @@ GSList *user_request(struct tls_buffer_read * datas)
 	return conn_elts;
 }
 
+static GSList *treat_extended_proto(struct tls_buffer_read *data)
+{
+	int ret;
+	struct nu_header *header = (struct nu_header *) data->buffer;
+
+	if (sizeof(*header) > (size_t)data->buffer_len) {
+		log_message(WARNING, DEBUG_AREA_USER,
+				"Error: too small message");
+		return NULL;
+	}
+
+	if (header->length > data->buffer_len) {
+		log_message(WARNING, DEBUG_AREA_USER,
+				"Error: message bigger than buffer (%d vs %d)",
+				header->length,
+				data->buffer_len);
+		return NULL;
+	}
+
+	if (! llist_empty(&(nuauthdatas->ext_proto_l))) {
+		ret = process_ext_message(data->buffer + sizeof(struct nu_header),
+				data->buffer_len - sizeof(struct nu_header),
+				&(nuauthdatas->ext_proto_l),
+				data);
+		if (ret != SASL_OK) {
+			log_message(WARNING, DEBUG_AREA_USER,
+					"Error when processing extended proto message");
+		}
+	} else {
+		log_message(WARNING, DEBUG_AREA_USER,
+				"No protocol extension supported but extended proto message");
+	}
+	return NULL;
+}
+
 /**
- * Decode user datagram packet and fill a connection with datas
+ * Decode user datagram packet and fill a connection with data
  * (called by user_check_and_decide()).
  *
- * \param datas Pointer to a struct tls_buffer_read:
+ * \param data Pointer to a struct tls_buffer_read:
  * \return Single linked list of connections (of type connection_t).
  */
-static GSList *userpckt_decode(struct tls_buffer_read *datas)
+static GSList *userpckt_decode(struct tls_buffer_read *data)
 {
-	char *dgram = datas->buffer;
+	char *dgram = data->buffer;
 	struct nu_header *header = (struct nu_header *) dgram;
 
 	/* check buffer underflow */
-	if (datas->buffer_len < (int) sizeof(struct nu_header)) {
+	if (data->buffer_len < (int) sizeof(struct nu_header)) {
 		log_message(WARNING, DEBUG_AREA_USER,
 			    "Received buffer too small to read header");
 		return NULL;
@@ -493,9 +612,13 @@ static GSList *userpckt_decode(struct tls_buffer_read *datas)
 		return NULL;
 	}
 
-	if (header->msg_type != USER_REQUEST) {
-		log_message(WARNING, DEBUG_AREA_USER, "Unsupported message type");
-		return NULL;
+	switch (header->msg_type) {
+		case USER_REQUEST:
+			return user_request(data);
+		case EXTENDED_PROTO:
+			return treat_extended_proto(data);
+		default:
+			log_message(WARNING, DEBUG_AREA_USER, "Unsupported message type");
+			return NULL;
 	}
-	return user_request(datas);
 }
