@@ -4,11 +4,11 @@
  **            Vincent Deffontaines <vincent@gryzor.com>
  ** INL http://www.inl.fr/
  **
- ** $Id: audit.c 5119 2008-11-10 17:24:25Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,6 +30,7 @@
  */
 
 #include <nufw.h>
+#include <nubase.h>
 
 /**
  * Output traffic statistics (packets received/accepted).
@@ -37,16 +38,18 @@
  */
 void process_poll(int signum)
 {
+/*
 	log_area_printf(DEBUG_AREA_MAIN,
 #ifdef DEBUG_ENABLE
 		DEBUG_LEVEL_SERIOUS_WARNING,
 #else
 		DEBUG_LEVEL_FATAL,
 #endif
-			"AUDIT: rx=%d tx=%d track_size=%d list=%s",
-			pckt_rx, pckt_tx, packets_list.length,
-			(packets_list.start ==
-			 NULL) ? "empty" : "one packet or more");
+		"AUDIT: rx=%d tx=%d track_size=%d list=%s",
+		pckt_rx, pckt_tx, packets_list.length,
+		(packets_list.start ==
+		 NULL) ? "empty" : "one packet or more");
+*/
 }
 
 /**
@@ -85,5 +88,3 @@ void process_hup(int signum)
 	shutdown_tls();
 	pthread_mutex_unlock(&tls.mutex);
 }
-
-
