@@ -1,14 +1,12 @@
 /*
- ** Copyright (C) 2002-2008 INL
+ ** Copyright (C) 2002-2009 INL
  ** Written by ric Leblond <eric@regit.org>
  **            Vincent Deffontaines <vincent@gryzor.com>
  ** INL http://www.inl.fr/
  **
- ** $Id: authsrv.c 5119 2008-11-10 17:24:25Z regit $
- **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,7 +19,11 @@
  */
 
 #include "nufw.h"
-#include "ipv6.h"
+
+#include <nubase.h>
+
+/* Mutex used to lock acces to nfqueue or ipqueue (see strcture.h) */
+pthread_mutex_t ipq_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 /** \file nufw/authsrv.c
  *  \brief Process NuAuth packets
@@ -52,9 +54,13 @@ int auth_process_answer(char *dgram, int dgram_size)
 	if (dgram_size <
 	    (int) (sizeof(nuv4_nuauth_decision_response_t) + payload_len)
 	    || ((payload_len != 0) && (payload_len != (20 + 8))
-		&& (payload_len != (40 + 8)))) {
+		&& (payload_len != (40 + 8)) &&
+		(dgram_size != (int) (sizeof(nuv4_nuauth_decision_response_t) + payload_len)))) {
 		log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_WARNING,
-				"[!] Packet with improper size");
+				"[!] Packet with improper size: payload of %d, received %d (vs %d)",
+				payload_len,
+				dgram_size,
+				(int) (sizeof(nuv4_nuauth_decision_response_t) + payload_len));
 		return -1;
 	}
 
@@ -103,7 +109,8 @@ int auth_process_answer(char *dgram, int dgram_size)
 				"(*) Rejecting %" PRIu32, packet_id);
 		IPQ_SET_VERDICT(packet_id, NF_DROP);
 		if (send_icmp_unreach(dgram +
-				  sizeof(nuv4_nuauth_decision_response_t)) == -1) {
+				  sizeof(nuv4_nuauth_decision_response_t),
+				  payload_len) == -1) {
 			log_area_printf(DEBUG_AREA_PACKET, DEBUG_LEVEL_WARNING,
 					"(*) Could not sent ICMP reject for %" PRIu32, packet_id);
 		}
@@ -120,7 +127,164 @@ int auth_process_answer(char *dgram, int dgram_size)
 }
 
 #ifdef HAVE_LIBCONNTRACK
+#ifdef HAVE_NEW_NFCT_API
 
+static int conn_update_cb(enum nf_conntrack_msg_type type,
+	      struct nf_conntrack *ct,
+	      void *data)
+{
+	uint32_t *status = (uint32_t *) data;
+
+	*status = nfct_get_attr_u32(ct, ATTR_STATUS);
+	return NFCT_CB_CONTINUE;
+}
+
+static int build_nfct_from_message(struct nf_conntrack *ct, struct nuv4_conntrack_message_t *packet_hdr)
+{
+	/* use setters to build entry */
+	nfct_set_attr_u8(ct, ATTR_L4PROTO, packet_hdr->ip_protocol);
+
+	if (is_ipv4(&packet_hdr->ip_src) && is_ipv4(&packet_hdr->ip_dst)) {
+		nfct_set_attr_u8(ct, ATTR_L3PROTO, AF_INET);
+		nfct_set_attr_u32(ct, ATTR_IPV4_SRC,
+				  packet_hdr->ip_src.s6_addr32[3]);
+		nfct_set_attr_u32(ct, ATTR_IPV4_DST,
+				  packet_hdr->ip_dst.s6_addr32[3]);
+	} else {
+		nfct_set_attr_u8(ct, ATTR_L3PROTO, AF_INET6);
+#if 0
+		memcpy(&orig->src.v6, &packet_hdr->ip_src,
+				sizeof(orig->src.v6));
+		memcpy(&orig->dst.v6, &packet_hdr->ip_dst,
+				sizeof(orig->dst.v6));
+#endif
+	}
+
+	switch (packet_hdr->ip_protocol) {
+		case IPPROTO_TCP:
+		case IPPROTO_UDP:
+			nfct_set_attr_u16(ct, ATTR_PORT_SRC,
+					  packet_hdr->src_port);
+			nfct_set_attr_u16(ct, ATTR_PORT_DST,
+					  packet_hdr->dest_port);
+			break;
+		default:
+			return 0;
+	}
+	nfct_setobjopt(ct, NFCT_SOPT_SETUP_REPLY);
+	return 1;
+}
+
+/**
+ * Process NuAuth message of type #AUTH_CONN_DESTROY
+ */
+
+int auth_process_conn_destroy(char *dgram, int dgram_size)
+{
+	struct nuv4_conntrack_message_t *packet_hdr;
+	struct nf_conntrack *ct;
+
+	/* check packet size */
+	if (dgram_size < (int) sizeof(struct nuv4_conntrack_message_t)) {
+		return -1;
+	}
+
+	packet_hdr = (struct nuv4_conntrack_message_t *) dgram;
+
+	if (ntohs(packet_hdr->msg_length) != sizeof(struct nuv4_conntrack_message_t)) {
+		return -1;
+	}
+
+	ct = nfct_new();
+
+	if (build_nfct_from_message(ct, packet_hdr)) {
+		debug_log_printf(DEBUG_AREA_GW | DEBUG_AREA_PACKET,
+				 DEBUG_LEVEL_VERBOSE_DEBUG,
+				 "Deleting entry from conntrack after NuAuth request");
+		(void) nfct_query(cth, NFCT_Q_DESTROY, ct);
+	}
+
+	nfct_destroy(ct);
+	return ntohs(packet_hdr->msg_length);
+}
+
+/**
+ * Process NuAuth message of type #AUTH_CONN_UPDATE
+ */
+int auth_process_conn_update(char *dgram, int dgram_size)
+{
+	struct nuv4_conntrack_message_t *packet_hdr;
+	uint32_t status;
+
+	struct nf_conntrack *ct;
+	/* check packet size */
+	if (dgram_size < (int) sizeof(struct nuv4_conntrack_message_t)) {
+		debug_log_printf(DEBUG_AREA_GW, DEBUG_LEVEL_DEBUG,
+				 "NuAuth sent too small message");
+		return -1;
+	}
+	packet_hdr = (struct nuv4_conntrack_message_t *) dgram;
+
+	if (ntohs(packet_hdr->msg_length) != sizeof(struct nuv4_conntrack_message_t)) {
+		return -1;
+	}
+
+	if (packet_hdr->timeout) {
+		ct = nfct_new();
+		if (build_nfct_from_message(ct, packet_hdr)) {
+			int ret;
+			/* getting conn to be able to update status without
+			 * breaking anything */
+			nfct_callback_register(cth, NFCT_T_ALL, conn_update_cb, &status);
+			ret = nfct_query(cth, NFCT_Q_GET, ct);
+			if (ret != 0) {
+				log_area_printf(DEBUG_AREA_MAIN,
+						DEBUG_LEVEL_WARNING,
+						"Conntrack fetching was impossible: %s",
+						strerror(errno));
+				nfct_callback_unregister(cth);
+				nfct_destroy(ct);
+				return ret;
+			}
+			debug_log_printf(DEBUG_AREA_GW,
+					DEBUG_LEVEL_VERBOSE_DEBUG,
+					"conn status is %d",
+					status);
+
+			debug_log_printf(DEBUG_AREA_GW,
+					DEBUG_LEVEL_VERBOSE_DEBUG,
+					"Will set timeout to %d after NuAuth request (was %d)",
+					ntohl(packet_hdr->timeout),
+					nfct_get_attr_u32(ct, ATTR_TIMEOUT)
+					);
+			nfct_set_attr_u32(ct, ATTR_TIMEOUT,
+					ntohl(packet_hdr->timeout));
+
+			status |= IPS_FIXED_TIMEOUT;
+			nfct_set_attr_u32(ct, ATTR_STATUS, status);
+			debug_log_printf(DEBUG_AREA_GW,
+					DEBUG_LEVEL_VERBOSE_DEBUG,
+					"setting conn status to %d",
+					status);
+			ret = nfct_query(cth, NFCT_Q_UPDATE, ct);
+
+			if (ret != 0) {
+				log_area_printf(DEBUG_AREA_MAIN,
+						DEBUG_LEVEL_WARNING,
+						"Conntrack update was impossible: %s",
+						strerror(errno));
+				nfct_callback_unregister(cth);
+				nfct_destroy(ct);
+				return ret;
+			}
+			nfct_callback_unregister(cth);
+		}
+		nfct_destroy(ct);
+	}
+	return ntohs(packet_hdr->msg_length);
+}
+
+#else
 int build_nfct_tuple_from_message(struct nfct_tuple *orig,
 				  struct nuv4_conntrack_message_t
 				  *packet_hdr)
@@ -255,6 +419,8 @@ int auth_process_conn_update(char *dgram, int dgram_size)
 	}
 	return ntohs(packet_hdr->msg_length);
 }
+
+#endif				/* HAVE_NEW_NFCT_API */
 #endif				/* HAVE_LIBCONNTRACK */
 
 /**
@@ -267,7 +433,7 @@ int auth_process_conn_update(char *dgram, int dgram_size)
  *
  *  \return -1 in case of error
  */
-static inline int auth_packet_to_decision(char *dgram, int dgram_size)
+static int auth_packet_to_decision(char *dgram, int dgram_size)
 {
 	if (dgram_size < 2) {
 		debug_log_printf(DEBUG_AREA_GW, DEBUG_LEVEL_DEBUG,
@@ -320,10 +486,6 @@ void *authsrv(void *data)
 	int read_size;
 	char cdgram[512];
 	char *dgram = cdgram;
-	int socket = (int) gnutls_transport_get_ptr(*tls.session);
-	fd_set wk_set;
-	int select_result;
-	struct timeval tv;
 
 	log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_VERBOSE_DEBUG,
 			"[+] Start auth server thread");
@@ -331,52 +493,22 @@ void *authsrv(void *data)
 	while (pthread_mutex_trylock(&tls.auth_server_mutex) == 0) {
 		pthread_mutex_unlock(&tls.auth_server_mutex);
 
-		/* Set timeout: one second */
-		tv.tv_sec = 1;
-		tv.tv_usec = 0;
-
-		read_size = 0;
-
-		/* wait new event on socket */
-		FD_ZERO(&wk_set);
-		FD_SET(socket, &wk_set);
-		select_result =
-		    select(socket + 1, &wk_set, NULL, NULL, &tv);
-		if (select_result == -1) {
-			int err = errno;
-			if (err == EINTR) {
-				continue;
-			}
-			log_area_printf(DEBUG_AREA_GW,
-					DEBUG_LEVEL_WARNING,
-					"[+] select() in authsrv.c failure: code %u",
-					errno);
-			break;
-		}
-
-		/* catch timeout */
-		if (select_result == 0) {
-			/* timeout! */
-			continue;
-		}
-
 		/* memset(dgram, 0, sizeof dgram); */
 		pthread_mutex_lock(&tls.mutex);
 		if (tls.session)
-			ret = gnutls_record_recv(*tls.session, dgram, sizeof cdgram);
+			ret = nussl_read(tls.session, dgram, sizeof cdgram);
 		else
 			ret = 0;
 		pthread_mutex_unlock(&tls.mutex);
-		if (ret < 0) {
-			if (gnutls_error_is_fatal(ret)) {
-				log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_CRITICAL,
-					       "[+] Fatal error on TLS session: %s",
-					       gnutls_strerror(ret));
-				break;
-			}
-		} else if (ret == 0) {
-			log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_CRITICAL,
-					"[+] Peer has closed TLS session (certificate issue or nuauth disconnect ?)");
+		if (ret == NUSSL_SOCK_TIMEOUT)
+		{
+			usleep(10000); /* Without that, the other thread can't get the lock to tls.mutex */
+			continue;
+		}
+		if (ret <= 0) {
+			log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_VERBOSE_DEBUG,
+					"Error during nussl_read: %s", nussl_get_error(tls.session));
+			break;
 		} else {
 			do {
 				read_size = auth_packet_to_decision(dgram, ret);
