@@ -5,11 +5,11 @@
 **            Vincent Deffontaines <vincent@gryzor.com>
 ** INL http://www.inl.fr/
 **
-** $Id: common.c 5465 2008-12-26 23:40:38Z regit $
+** $Id$
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation, version 2 of the License.
+** the Free Software Foundation, version 3 of the License.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,6 +29,7 @@
  */
 
 #include "nufw.h"
+
 #include <stdlib.h>
 #include <time.h>
 #include <linux/icmp.h>		/* icmphdr */
@@ -36,7 +37,10 @@
 #include <netinet/ip.h>		/* iphdr */
 #include <netinet/ip6.h>	/* ip6_hdr */
 
-/* datas stuffs */
+#include <nubase.h>
+
+
+/* data stuff */
 
 #ifdef PERF_DISPLAY_ENABLE
 /**
@@ -74,20 +78,11 @@ int timeval_substract(struct timeval *result, struct timeval *x,
  */
 void close_tls_session()
 {
-	int socket;
-
 	if (tls.session == NULL)
 		return;
 
 	pthread_mutex_destroy(&tls.auth_server_mutex);
-
-	socket = (int) gnutls_transport_get_ptr(*tls.session);
-	gnutls_certificate_free_credentials(tls.xcred);
-	gnutls_bye(*tls.session, GNUTLS_SHUT_WR);
-	gnutls_deinit(*tls.session);
-	shutdown(socket, SHUT_RDWR);
-	close(socket);
-	free(tls.session);
+	nussl_session_destroy(tls.session);
 	tls.session = NULL;
 }
 
@@ -269,22 +264,27 @@ __u16 icmp_cksum(__u16 * buf, int nbytes)
 	return (__u16) ~ sum;
 }
 
-int send_icmp_ipv4_unreach(char *payload)
+int send_icmp_ipv4_unreach(char *payload, int payload_len)
 {
 	struct sockaddr_in to;
-	char buffer[sizeof(struct icmphdr) + 20 + 8];
+	char buffer[256];
 	struct iphdr *ip = (struct iphdr *) payload;
 	struct icmphdr *icmp = (struct icmphdr *) buffer;
 
+	if (payload_len + sizeof(struct icmphdr) > 256) {
+		return -1;
+	}
 	/* write ICMP header */
 	icmp->type = 3;
-	icmp->code = 0;
+	icmp->code = 3;
 	icmp->checksum = 0x0000;
 	icmp->un.frag.__unused = 0;
 	icmp->un.frag.mtu = 0;
 
 	/* copy old packet header */
-	memcpy(buffer + sizeof(struct icmphdr), payload, 20 + 8);
+	memcpy(buffer + sizeof(struct icmphdr),
+			(char *)payload + sizeof(*ip),
+			payload_len - sizeof(*ip));
 
 	/* get destination IPv4 address */
 	memset(&to, 0, sizeof(to));
@@ -292,20 +292,25 @@ int send_icmp_ipv4_unreach(char *payload)
 	to.sin_addr.s_addr = ip->saddr;
 
 	/* compute icmp checksum */
-	icmp->checksum = icmp_cksum((__u16 *) buffer, sizeof(buffer));
+	icmp->checksum = icmp_cksum((__u16 *) buffer,
+				    sizeof(struct icmphdr) + payload_len - sizeof(*ip));
 
 	/* send packet */
-	return sendto(raw_sock4, buffer, sizeof(buffer), 0,
+	return sendto(raw_sock4, buffer,
+		      sizeof(struct icmphdr) + payload_len - sizeof(*ip), 0,
 		      (struct sockaddr *) &to, sizeof(to));
 }
 
-int send_icmp_ipv6_unreach(char *payload)
+int send_icmp_ipv6_unreach(char *payload, int payload_len)
 {
 	struct sockaddr_in6 to;
-	char buffer[sizeof(struct icmp6_hdr) + 40 + 8];
+	char buffer[256];
 	struct ip6_hdr *ip = (struct ip6_hdr *) payload;
 	struct icmp6_hdr *icmp = (struct icmp6_hdr *) buffer;
 
+	if (payload_len + sizeof(struct icmp6_hdr) > 256) {
+		return -1;
+	}
 	/* write ICMP header */
 	memset(icmp, 0, sizeof(*icmp));
 	icmp->icmp6_type = 1;
@@ -313,7 +318,7 @@ int send_icmp_ipv6_unreach(char *payload)
 	/* checksum and data are nul */
 
 	/* copy old packet header */
-	memcpy(buffer + sizeof(*icmp), payload, 40 + 8);
+	memcpy(buffer + sizeof(*icmp), payload, payload_len);
 
 	/* get destination IPv6 address */
 	memset(&to, 0, sizeof(to));
@@ -328,19 +333,19 @@ int send_icmp_ipv6_unreach(char *payload)
 
 	if (raw_sock6 > 0) {
 		/* send packet */
-		return sendto(raw_sock6, buffer, sizeof(buffer), 0,
-			      (struct sockaddr *) &to, sizeof(to));
+		return sendto(raw_sock6, buffer, payload_len + sizeof(struct icmp6_hdr),
+				0, (struct sockaddr *) &to, sizeof(to));
 	} else {
 		return 0;
 	}
 }
 
-int send_icmp_unreach(char *payload)
+int send_icmp_unreach(char *payload, int payload_len)
 {
 	struct iphdr *ip4 = (struct iphdr *) payload;
 	if (ip4->version == AF_INET) {
-		return send_icmp_ipv4_unreach(payload);
+		return send_icmp_ipv4_unreach(payload, payload_len);
 	} else {
-		return send_icmp_ipv6_unreach(payload);
+		return send_icmp_ipv6_unreach(payload, payload_len);
 	}
 }
