@@ -3,11 +3,11 @@
  ** Written by Eric Leblond <regit@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: conntrack.c 5067 2008-11-04 21:13:48Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,12 +25,12 @@
  * Connection tracking function if NuFW is compiled with \#HAVE_LIBCONNTRACK.
  */
 
-#undef DEBUG_CONNTRACK
+#define DEBUG_CONNTRACK
 
 #include "nufw.h"
-#include "ipv6.h"
 #ifdef HAVE_LIBCONNTRACK
 
+#include <nubase.h>
 #include <libnetfilter_conntrack/libnetfilter_conntrack_tcp.h>
 
 #ifdef HAVE_NEW_NFCT_API
@@ -136,9 +136,9 @@ void fill_message(struct nuv4_conntrack_message_t *message,
 
 #ifdef DEBUG_CONNTRACK
 	printf("(*) New conntrack event: ");
-	FORMAT_IPV6(&message->ip_src, ascii);
-		printf(" src=%s", ascii);
-	FORMAT_IPV6(&message->ip_dst, ascii);
+	format_ipv6(&message->ip_src, ascii, INET6_ADDRSTRLEN, NULL);
+	printf(" src=%s", ascii);
+	format_ipv6(&message->ip_dst, ascii, INET6_ADDRSTRLEN, NULL);
 	printf(" dst=%s\n", ascii);
 #endif
 }
@@ -146,8 +146,8 @@ void fill_message(struct nuv4_conntrack_message_t *message,
 /**
  * Send message to TLS tunnel on new netfilter conntrack event.
  *
- * \param arg Pointer to a connection of type ::nfct_conntrack
  * \param type Event type (IPCTNL_MSG_CT_DELETE, IPCTNL_MSG_CT_NEW, ...)
+ * \param conn Pointer to a connection of type ::nfct_conntrack
  * \param flags Event flags (no used)
  * \param data (no data, NULL pointer)
  * \return If an error occurs returns -1, else returns 0
@@ -233,13 +233,15 @@ int update_handler(struct nfct_conntrack *conn, unsigned int flags, int type,
 	if (tls.session) {
 		debug_log_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_DEBUG,
 				 "Sending conntrack event to nuauth.");
-		ret = gnutls_record_send(*(tls.session),
-					 &message,
+		ret = nussl_write(tls.session,
+					 (char*)&message,
 					 sizeof(struct
 						nuv4_conntrack_message_t)
 		    );
-		if (ret <= 0) {
+		if (ret < 0) {
 			/* warn sender thread that it will need to reconnect at next access */
+			debug_log_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_DEBUG,
+					 "Error during nussl_write.");
 			shutdown_tls();
 			pthread_mutex_unlock(&tls.mutex);
 			return callback_ret;
