@@ -1,13 +1,13 @@
 /*
- ** Copyright (C) 2007 INL
+ ** Copyright (C) 2007,2009 INL
  ** Written by Eric Leblond <regit@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: iface.c 4312 2008-01-15 08:51:45Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,15 +18,22 @@
  ** along with this program; if not, write to the Free Software
  ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
-
-
 #include "nufw.h"
 
+#include <nubase.h>
+
+
 #ifdef HAVE_NFQ_INDEV_NAME
+
+/* mutex used to get around non thread-safeness of iface resolution
+ * in libnfnetlink */
+pthread_mutex_t iface_mutex;
+
 int get_interface_information(struct nlif_handle *inst,
 			      struct queued_pckt *q_pckt,
 			      struct nfq_data *nfad)
 {
+	pthread_mutex_lock(&iface_mutex);
 	nfq_get_indev_name(inst, nfad, q_pckt->indev);
 	if (q_pckt->indev[0] == '*') {
 		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_DEBUG,
@@ -58,12 +65,15 @@ int get_interface_information(struct nlif_handle *inst,
 				"Get physoutdev information: %s",
 				q_pckt->physoutdev);
 	}
+	pthread_mutex_unlock(&iface_mutex);
 	return 1;
 }
 
 struct nlif_handle *iface_table_open()
 {
 	struct nlif_handle *inst;
+
+	pthread_mutex_init(&iface_mutex, NULL);
 	/* opening ifname resolution handle */
 	inst = nlif_open();
 	if (inst == NULL) {
@@ -78,16 +88,24 @@ struct nlif_handle *iface_table_open()
 
 int iface_treat_message(struct nlif_handle *inst)
 {
+	int ret;
 	debug_log_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_DEBUG,
 			 "Network interface event");
-	return nlif_catch(inst);
+
+	pthread_mutex_lock(&iface_mutex);
+	ret = nlif_catch(inst);
+	pthread_mutex_unlock(&iface_mutex);
+	return ret;
 }
 
 void iface_table_close(struct nlif_handle *inst)
 {
 	debug_log_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_DEBUG,
 			 "Free iface resolution instance");
+	pthread_mutex_lock(&iface_mutex);
 	nlif_close(inst);
+	pthread_mutex_unlock(&iface_mutex);
+	pthread_mutex_destroy(&iface_mutex);
 }
 
 #endif				/* #ifdef HAVE_NFQ_INDEV_NAME */
