@@ -1,15 +1,13 @@
 /*
- ** Copyright (C) 2002-2008 INL
+ ** Copyright (C) 2002-2009 INL
  ** Written by Eric Leblond <regit@inl.fr>
  **            Vincent Deffontaines <vincent@gryzor.com>
  **            Pierre Chifflier <chifflier@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: main.c 5477 2009-01-09 10:51:49Z regit $
- **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,6 +28,11 @@
  */
 
 #include "nufw.h"
+
+#ifdef HAVE_GETOPT_H
+# include <getopt.h>
+#endif
+
 #include <linux/netfilter.h>
 #include <unistd.h>
 #include <pthread.h>
@@ -38,16 +41,17 @@
 #include <syslog.h>
 #include <errno.h>
 #include <sys/types.h>
+#include <fcntl.h>	/* O_RDWR */
 
-GCRY_THREAD_OPTION_PTHREAD_IMPL;
+#include <nubase.h>
+
+#include "nufwconf.h"
 
 char *key_file = NULL;
 char *cert_file = NULL;
 
 /* packet server thread */
-struct nufw_threadtype packet_thread;
-/* maintenance thread */
-struct nufw_threadtype maint_thread;
+struct nufw_threadtype thread;
 
 /* packet server thread */
 struct nufw_signals signals;
@@ -56,6 +60,8 @@ struct nufw_signals signals;
  * during compilation/installation) */
 #define NUFW_PID_FILE  LOCAL_STATE_DIR "/run/nufw.pid"
 
+char * nufw_config_file = DEFAULT_NUFW_CONF_FILE;
+
 /**
  * Stop threads and then wait until threads exit.
  */
@@ -63,10 +69,7 @@ void nufw_stop_thread()
 {
 	/* ask threads to stop */
 	pthread_mutex_lock(&tls.auth_server_mutex);
-	pthread_mutex_lock(&packet_thread.mutex);
-	if (crl_file) {
-		pthread_mutex_lock(&maint_thread.mutex);
-	}
+	pthread_mutex_lock(&thread.mutex);
 
 	/* wait for thread end */
 	log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_MESSAGE,
@@ -75,12 +78,8 @@ void nufw_stop_thread()
 		pthread_join(tls.auth_server, NULL);
 	}
 	pthread_mutex_unlock(&tls.auth_server_mutex);
-	pthread_join(packet_thread.thread, NULL);
-	pthread_mutex_unlock(&packet_thread.mutex);
-	if (crl_file) {
-		pthread_join(maint_thread.thread, NULL);
-		pthread_mutex_unlock(&maint_thread.mutex);
-	}
+	pthread_join(thread.thread, NULL);
+	pthread_mutex_unlock(&thread.mutex);
 }
 
 /**
@@ -102,9 +101,6 @@ void nufw_prepare_quit()
 	nfct_close(cth);
 #endif
 
-	/* quit gnutls */
-	gnutls_global_deinit();
-
 	/* free memory */
 	free(key_file);
 	free(cert_file);
@@ -168,19 +164,6 @@ void nufw_cleanup(int signal)
 	exit(EXIT_SUCCESS);
 }
 
-void *maintenance_thread_fn(void *data)
-{
-	struct nufw_threadargument *thread_arg = data;
-	struct nufw_threadtype *this = thread_arg->thread;
-	while (pthread_mutex_trylock(&this->mutex) == 0) {
-		pthread_mutex_unlock(&this->mutex);
-
-		sleep(NUFW_MAINT_DELAY);
-		tls_maintenance();
-	}
-	return NULL;
-}
-
 /**
  * Create packet server thread: init mutex and create thread
  * with packetsrv() function. Send pointer to ::thread to
@@ -190,6 +173,8 @@ void create_thread()
 {
 	/* should be static because thread may read data after this function exits */
 	static struct nufw_threadargument arg;
+	arg.thread = &thread;
+	arg.parent_pid = getpid();
 
 	/* set attribute to "joinable thread" */
 	pthread_attr_t attr;
@@ -197,13 +182,11 @@ void create_thread()
 	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
 
 	/* create mutex */
-	pthread_mutex_init(&packet_thread.mutex, NULL);
+	pthread_mutex_init(&thread.mutex, NULL);
 
 	/* try to create the thread */
-	arg.thread = &packet_thread;
-	arg.parent_pid = getpid();
-	if (pthread_create(&packet_thread.thread, &attr, packetsrv, &arg) != 0) {
-		pthread_mutex_destroy(&packet_thread.mutex);
+	if (pthread_create(&thread.thread, &attr, packetsrv, &arg) != 0) {
+		pthread_mutex_destroy(&thread.mutex);
 		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_FATAL,
 				"Fail to create thread!");
 		exit(EXIT_FAILURE);
@@ -218,20 +201,8 @@ void create_thread()
 	}
 #endif
 
-	if (crl_file) {
-		pthread_mutex_init(&maint_thread.mutex, NULL);
-		arg.thread = &maint_thread;
-		arg.parent_pid = getpid();
-		if (pthread_create
-				(&maint_thread.thread, &attr,
-				 maintenance_thread_fn, &arg) == EAGAIN) {
-			exit(EXIT_FAILURE);
-		}
-	}
-
 }
 
-
 /**
  * Install signals:
  *   - Set SIGTERM handler to nufw_cleanup()
@@ -307,7 +278,6 @@ void install_signals()
 		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_WARNING,
 				"Warning: Could not set signal HUP");
 	}
-
 }
 
 /**
@@ -361,34 +331,53 @@ void nufw_daemonize()
 	/* set log engine */
 	log_engine = LOG_TO_SYSLOG;
 
-	/* Close stdin, stdout, stderr. */
-	(void) close(0);
-	(void) close(1);
-	(void) close(2);
-}
-
-
-/**
- * Create nuauth address: ::adr_srv
- */
-void create_nuauth_address()
-{
-	int ecode;
-	struct addrinfo hints;
-	memset(&hints, 0, sizeof hints);
-	hints.ai_flags = AI_PASSIVE;
-	hints.ai_socktype = SOCK_STREAM;
-	hints.ai_family = PF_UNSPEC;
-
-	ecode = getaddrinfo(authreq_addr, authreq_port, &hints, &adr_srv);
-	if (ecode != 0) {
-		log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_CRITICAL,
-				"Can not resolve NuAuth hostname \"%s\"!",
-				authreq_addr);
-		exit(EXIT_FAILURE);
+	/* warning: do not close fd (0 1 2), or this will create problems when trying
+	 * to create child process with a pipe (dup2 fails with error EBADF)
+	 */
+	{
+		int fd = open("/dev/null",O_RDWR);
+		dup2(fd, STDIN_FILENO);
+		dup2(fd, STDOUT_FILENO);
+		dup2(fd, STDERR_FILENO);
+		close(fd);
 	}
 }
 
+/**
+ * Parse configuration values and set variables
+ */
+int nufw_use_config()
+{
+	char * value;
+
+	value = nufw_config_table_get("nufw_tls_cacert");
+	if (value != NULL && ca_file == NULL) {
+		ca_file = strdup(value);
+	}
+
+	value = nufw_config_table_get_or_default("nufw_tls_cert",DEFAULT_NUFW_CERT);
+	if (value != NULL && cert_file == NULL) {
+		cert_file = strdup(value);
+	}
+
+	value = nufw_config_table_get_or_default("nufw_tls_key",DEFAULT_NUFW_KEY);
+	if (value != NULL && key_file == NULL) {
+		key_file = strdup(value);
+	}
+
+	value = nufw_config_table_get("nufw_tls_crl");
+	if (value != NULL && crl_file == NULL) {
+		crl_file = strdup(value);
+	}
+
+	value = nufw_config_table_get_or_default("nufw_destination", AUTHREQ_ADDR);
+	if (value != NULL && strlen(authreq_addr) == 0) {
+		SECURE_STRNCPY(authreq_addr, value, sizeof authreq_addr);
+		printf("Sending Auth request to %s\n", authreq_addr);
+	}
+
+	return 0;
+}
 
 /**
  * Initialization checks
@@ -414,6 +403,83 @@ int init_checks()
 	return 1;
 }
 
+static struct option long_options[] = {
+	{"help", 0, NULL, 'h'},
+	{"version", 0, NULL, 'V'},
+	{"daemon", 0, NULL, 'D'},
+	{"no-strict", 0, NULL, 's'},
+	{"strict", 0, NULL, 'S'},
+	{"no-fqdn", 0, NULL, 'N'},
+	{"key", 1, NULL, 'k'},
+	{"cert", 1, NULL, 'c'},
+	{"ca", 1, NULL, 'a'},
+	{"crl", 1, NULL, 'r'},
+	{"check-dn", 1, NULL, 'n'},
+	{"verbose", 0, NULL, 'v'},
+	{"debug-area", 1, NULL, 'A'},
+	{"ipv4", 0, NULL, '4'},
+	{"mark", 0, NULL, 'm'},
+	{"conntrack", 0, NULL, 'C'},
+	{"marked-only", 0, NULL, 'M'},
+	{"destination", 1, NULL, 'd'},
+	{"port", 1, NULL, 'p'},
+	{"queue", 1, NULL, 'q'},
+	{"queue-len", 1, NULL, 'L'},
+	{"timeout", 1, NULL, 't'},
+	{"track-size", 1, NULL, 'T'},
+
+	{0, 0, 0, 0}
+};
+
+
+void display_usage(void)
+{
+	fprintf(stdout, "%s [-hVc"
+#ifdef HAVE_LIBCONNTRACK
+		"CM"
+#endif
+		"v[v[v[v[v[v[v[v[v[v]]]]]]]]]] [-d remote_addr] [-p remote_port]  [-t packet_timeout] [-T track_size]"
+#ifdef USE_NFQUEUE
+				" [-q queue_num]"
+#ifdef HAVE_NFQ_SET_QUEUE_MAXLEN
+				" [-L queue_maxlen]"
+#endif
+#endif
+				"\n\
+\t-h (--help       ): display this help and exit\n\
+\t-V (--version    ): display version and exit\n\
+\t-D (--daemon     ): daemonize\n\
+\t-f (--config     ): use specific config file\n\
+\t-s (--no-strict  ): do not enforce strict checking of TLS certificates\n\
+\t-S (--strict     ): this option does nothing, it is here for backward compatibility\n\
+\t-N (--no-fqdn    ): do not check nuauth fqdn (-d params) against provided certificate\n\
+\t-k (--key        ): certificate key file\n\
+\t-c (--cert       ): certificate file\n\
+\t-a (--ca         ): certificate authority file (strict checking is done if selected) (default: none)\n\
+\t-r (--crl        ): use specified file as crl file (default: none)\n\
+\t-n (--check-dn   ): use specified string as the needed DN of nuauth (enforce certificate checking) (default is to)\n\
+\t\tcheck the DN against nuauth FQDN specified using the -d option)\n\
+\t-v (--verbose    ): increase debug level (+1 for each 'v') (max useful number: 10)\n\
+\t-A (--debug-area ): debug areas (see man page for details)\n\
+\t-4 (--ipv4       ): use this flag if your system does not have IPv6 support for nfnetlink\n\
+\t-m (--mark       ): mark packet with nuauth provided mark\n"
+#ifdef HAVE_LIBCONNTRACK
+"\t-C (--conntrack  ): listen to conntrack events (needed for connection expiration)\n\
+\t-M (--marked-only): only report event on marked connections to nuauth (implies -C and -m)\n"
+#endif
+"\t-d (--destination): remote address we send auth requests to (address of the nuauth server) (default: 127.0.0.1)\n\
+\t-p (--port       ): remote port we send auth requests to (TCP port nuauth server listens on) (default: 4128)\n"
+#if USE_NFQUEUE
+"\t-q (--queue      ): use nfqueue number (default: 0)\n"
+#ifdef HAVE_NFQ_SET_QUEUE_MAXLEN
+"\t-L (--queue-len  ): set queue max len (default: 1024)\n"
+#endif
+#endif
+"\t-t (--timeout    ): timeout to forget about packets when they don't match (default: 15 s)\n\
+\t-T (--track-size ): track size (default : 1000)\n",
+				PACKAGE_TARNAME);
+}
+
 
 /**
  * Main function of NuFW:
@@ -444,7 +510,7 @@ int main(int argc, char *argv[])
 {
 	/* option */
 #if USE_NFQUEUE
-	char *options_list = "4sSNDhVvmq:"
+	char *options_list = "4sSNDf:hVvmq:"
 #ifdef HAVE_NFQ_SET_QUEUE_MAXLEN
 	    "L:"
 #endif
@@ -454,15 +520,16 @@ int main(int argc, char *argv[])
 #endif
 	    ;
 #else
-	char *options_list = "4sSNDhVvmc:k:a:n:r:d:p:t:T:A:";
+	char *options_list = "4sSNDf:hVvmc:k:a:n:r:d:p:t:T:A:";
 #endif
 	int option, daemonize = 0;
 	char *version = PACKAGE_VERSION;
+	nufw_no_ipv6 = 0;
 
 	/* initialize variables */
 
 	log_engine = LOG_TO_STD;	/* default is to send debug messages to stdout + stderr */
-	snprintf(authreq_port, sizeof(authreq_port), "%hu", AUTHREQ_PORT);
+	authreq_port = AUTHREQ_PORT;
 	packet_timeout = PACKET_TIMEOUT;
 	track_size = TRACK_SIZE;
 	cert_file = NULL;
@@ -470,7 +537,7 @@ int main(int argc, char *argv[])
 	ca_file = NULL;
 	crl_file = NULL;
 	nuauth_cert_dn = NULL;
-	SECURE_STRNCPY(authreq_addr, AUTHREQ_ADDR, sizeof authreq_addr);
+	authreq_addr[0] = '\0';
 	debug_level = DEFAULT_DEBUG_LEVEL;
 	debug_areas = DEFAULT_DEBUG_AREAS;
 #if USE_NFQUEUE
@@ -486,7 +553,6 @@ int main(int argc, char *argv[])
 #endif
 	nufw_set_mark = 0;
 	nufw_strict_tls = 1;
-	nufw_no_ipv6 = 0;
 	nufw_fqdn_check = 1;
 
 
@@ -494,8 +560,16 @@ int main(int argc, char *argv[])
 			"[+] Start NuFW");
 
 	/*parse options */
-	while ((option = getopt(argc, argv, options_list)) != -1) {
+	while ((option = getopt_long(argc, argv, options_list, long_options, NULL)) != -1) {
 		switch (option) {
+		case 'f':
+			nufw_config_file = strdup(optarg);
+			if (nufw_config_file == NULL) {
+				fprintf(stderr,
+					"Couldn't malloc! Exiting");
+				exit(EXIT_FAILURE);
+			}
+			break;
 		case 'k':
 			key_file = strdup(optarg);
 			if (key_file == NULL) {
@@ -528,7 +602,6 @@ int main(int argc, char *argv[])
 				exit(EXIT_FAILURE);
 			}
 			break;
-
 		case 'n':
 			nuauth_cert_dn = strdup(optarg);
 			if (nuauth_cert_dn == NULL) {
@@ -549,8 +622,7 @@ int main(int argc, char *argv[])
 			debug_level += 1;
 			break;
 		case 'p':
-			SECURE_STRNCPY(authreq_port, optarg,
-				       sizeof(authreq_port));
+			authreq_port = atoi(optarg);
 			break;
 			/* destination IP */
 		case 'd':
@@ -578,7 +650,6 @@ int main(int argc, char *argv[])
 			nufw_fqdn_check = 0;
 			break;
 		case 'S':
-			nufw_strict_tls = 1;
 			break;
 		case 'N':
 			nufw_fqdn_check = 0;
@@ -610,54 +681,21 @@ int main(int argc, char *argv[])
 #endif				/* USE_NFQUEUE */
 
 		case 'h':
-			fprintf(stdout, "%s [-hVcsS"
-#ifdef HAVE_LIBCONNTRACK
-				"CM"
-#endif
-				"v[v[v[v[v[v[v[v[v[v]]]]]]]]]] [-d remote_addr] [-p remote_port]  [-t packet_timeout] [-T track_size]"
-#ifdef USE_NFQUEUE
-				" [-q queue_num]"
-#ifdef HAVE_NFQ_SET_QUEUE_MAXLEN
-				" [-L queue_maxlen]"
-#endif
-#endif
-				"\n\
-\t-h: display this help and exit\n\
-\t-V: display version and exit\n\
-\t-D: daemonize\n\
-\t-S: do strict checking of TLS negociation [default]\n\
-\t-s: do not enforce strict checking of TLS certificates\n\
-\t-N: do not check nuauth fqdn (-d params) against provided certificate\n\
-\t-k: use specified file as key file\n\
-\t-c: use specified file as cert file\n\
-\t-a: use specified file as ca file (strict checking is done if selected) (default: none)\n\
-\t-r: use specified file as crl file (default: none)\n\
-\t-n: use specified string as the needed DN of nuauth (enforce certificate checking) (default is to\n\
-\t\tcheck the DN against nuauth FQDN specified using the -d option)\n\
-\t-v: increase debug level (+1 for each 'v') (max useful number: 10)\n\
-\t-A: debug areas (see man page for details)\n\
-\t-4: use this flag if your system does not have IPv6 support for nfnetlink\n\
-\t-m: mark packet with nuauth provided mark\n"
-#ifdef HAVE_LIBCONNTRACK
-				"\t-C: listen to conntrack events (needed for connection expiration)\n\
-\t-M: only report event on marked connections to nuauth (implies -C and -m)\n"
-#endif
-				"\t-d: Address of the nuauth server (default: 127.0.0.1)\n\
-\t-p: TCP port of the nuauth server (default: 4128)\n"
-#if USE_NFQUEUE
-				"\t-q: use nfqueue number (default: 0)\n"
-#ifdef HAVE_NFQ_SET_QUEUE_MAXLEN
-				"\t-L: set queue max len (default: 1024)\n"
-#endif
-#endif
-				"\t-t: timeout to forget about packets when they don't match (default: 15 s)\n\
-\t-T: track size (default : 1000)\n",
-				PACKAGE_TARNAME);
-
+			display_usage();
 			exit(EXIT_SUCCESS);
 		}
 	}
 
+	if (nufw_parse_configuration(nufw_config_file) != 0) {
+		printf("Error while parsing configuration file\n");
+		exit(EXIT_FAILURE);
+	}
+
+	if (nufw_use_config() != 0) {
+		printf("Error while setting configuration values from file\n");
+		exit(EXIT_FAILURE);
+	}
+
 	if (getuid()) {
 		printf("nufw must be run as root! Sorry\n");
 		exit(EXIT_FAILURE);
@@ -677,7 +715,7 @@ int main(int argc, char *argv[])
 
 	install_signals();
 
-	init_log_engine();
+	init_log_engine("nufw");
 
 	/* open ICMP (IPv4) socket */
 	raw_sock4 = socket(PF_INET, SOCK_RAW, 1);	/* 1: ICMP protocol */
@@ -696,9 +734,6 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	/* Create address adr_srv */
-	create_nuauth_address();
-
 	/* create packet list */
 	packets_list.start = NULL;
 	packets_list.end = NULL;
@@ -711,29 +746,38 @@ int main(int argc, char *argv[])
 	pthread_mutex_init(&tls.mutex, NULL);
 
 	/* start GNU TLS library */
-	gcry_control(GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
-	gnutls_global_init();
+	if (nussl_init() != NUSSL_OK) {
+		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_FATAL,
+				"Unable to initialize NuSSL library.");
 
+	}
 
 #ifdef HAVE_LIBCONNTRACK
 	cth = nfct_open(CONNTRACK, 0);
 #endif
 
-	invalid_tls_params = 0;
+	log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_INFO,
+			"[+] NuFW server starting");
+
+
 	/* do initial connect */
 	tls_connect();
-
 	if (tls.session) {
+		char buf[256];
+		buf[0] = '\0';
+		nussl_session_get_cipher(tls.session, buf, sizeof(buf));
 		log_area_printf(DEBUG_AREA_GW,
 				DEBUG_LEVEL_WARNING,
-				"[+] TLS connection to nuauth established");
+				"[+] TLS connection to nuauth established (%s:%d), cipher is %s",
+				authreq_addr, authreq_port,
+				(buf[0] != '\0') ? buf : "none" );
 	} else {
 		log_area_printf(DEBUG_AREA_GW,
 				DEBUG_LEVEL_CRITICAL,
-				"[!] TLS connection to nuauth can NOT be established");
+				"[!] TLS connection to nuauth can NOT be established (%s:%d)",
+				authreq_addr, authreq_port);
 	}
 
-
 	/* create packet server thread */
 	create_thread();
 
@@ -742,14 +786,8 @@ int main(int argc, char *argv[])
 
 	if (daemonize == 0) {
 		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_CRITICAL,
-				"NuFW launched in foreground (without -D),"
-				"logging to stdout and stderr only (no "
-				"syslog).");
-	}
-
-	if ( ! nufw_strict_tls ) {
-		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_WARNING,
-				"NuFW did not start under the TLS strict mode");
+				"NuFW launched in foreground (without -D option), "
+				"logging to stdout and stderr only (no syslog).");
 	}
 
 	/* control stuff */
