@@ -4,11 +4,11 @@
  **            Vincent Deffontaines <vincent@gryzor.com>
  ** INL http://www.inl.fr/
  **
- ** $Id: nufw.h 5458 2008-12-22 13:35:37Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -52,11 +52,10 @@
 #include <unistd.h>
 #include <assert.h>
 #include <strings.h>
-#include <gnutls/gnutls.h>
 #include <gcrypt.h>
 #include <errno.h>
 
-#include "log.h"
+#include <nussl.h>
 #include "security.h"
 #include "structure.h"
 
@@ -64,6 +63,9 @@
 #include <linux/netfilter.h>	/* for NF_ACCEPT */
 #include <libnetfilter_queue/libnetfilter_queue.h>
 
+/** Default value of config file */
+#define DEFAULT_NUFW_CONF_FILE CONFIG_DIR "/nufw.conf"
+
 /** Default value of ::nfqueue_num */
 #define DEFAULT_NFQUEUE 0
 
@@ -109,17 +111,16 @@ void *conntrack_event_handler(void *data);
 /** If equals to 1, compile with x509 certificate support */
 #define USE_X509 1
 
-/* delay between maintenance task (mainly CRL refresh) */
-#define NUFW_MAINT_DELAY 30
-
 /** Default value, prefixed with CONFIG_DIR, of ::key_file */
 #define KEYFILE "/nufw-key.pem"
+#define DEFAULT_NUFW_KEY  CONFIG_DIR KEYFILE
 
 /** Default value, prefixed with CONFIG_DIR, of ::cert_file */
 #define CERTFILE "/nufw-cert.pem"
+#define DEFAULT_NUFW_CERT  CONFIG_DIR CERTFILE
 
 struct nuauth_conn {
-	gnutls_session *session;
+	nussl_session *session;
 	pthread_mutex_t mutex;
 	unsigned char auth_server_running;
 	pthread_t auth_server;
@@ -127,7 +128,6 @@ struct nuauth_conn {
 #ifdef HAVE_LIBCONNTRACK
 	pthread_t conntrack_event_handler;
 #endif
-	gnutls_certificate_credentials xcred;
 };
 
 struct queued_pckt {
@@ -154,8 +154,6 @@ pthread_cond_t *session_active_cond;
 pthread_mutex_t *session_destroyed_mutex;
 pthread_mutex_t *session_active_mutex;
 
-int invalid_tls_params;
-
 /**
  * Address informations of NuAuth server: hostname ::authreq_addr,
  * port ::authreq_port. Used in tls_connect().
@@ -183,7 +181,6 @@ int auth_request_send(uint8_t type, struct queued_pckt *pckt);
 
 void close_tls_session();
 void shutdown_tls();
-void tls_maintenance();
 
 int padd(packet_idl * packet);
 int psearch_and_destroy(uint32_t packet_id, uint32_t * mark);
@@ -195,6 +192,6 @@ void process_usr2(int signum);
 void process_poll(int signum);
 void process_hup(int signum);
 
-int send_icmp_unreach(char *payload);
+int send_icmp_unreach(char *payload, int payload_len);
 
 #endif				/* _NUFW_HEADER_H */
