@@ -1,14 +1,14 @@
 /*
- ** Copyright (C) 2002-2007 INL
+ ** Copyright (C) 2002-2008 INL
  ** Written by Eric Leblond <eric@regit.org>
  **            Vincent Deffontaines <vincent@gryzor.com>
  ** INL http://www.inl.fr/
  **
- ** $Id: packetsrv.c 5475 2009-01-06 16:59:25Z regit $
+ ** $Id$
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
- ** the Free Software Foundation, version 2 of the License.
+ ** the Free Software Foundation, version 3 of the License.
  **
  ** This program is distributed in the hope that it will be useful,
  ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,6 +22,8 @@
 
 #include "nufw.h"
 
+#include <nubase.h>
+
 #ifdef HAVE_NFQ_INDEV_NAME
 #  include "iface.h"
 #endif
@@ -49,8 +51,10 @@
 int look_for_tcp_flags(unsigned char *dgram, unsigned int datalen)
 {
 	struct iphdr *iphdrs = (struct iphdr *) dgram;
-	/* check need some datas */
+	/* check need some data */
 	if (datalen < sizeof(struct iphdr) + sizeof(struct tcphdr)) {
+		log_area_printf(DEBUG_AREA_PACKET, DEBUG_LEVEL_VERBOSE_DEBUG,
+				"Incorrect packet data length");
 		return 0;
 	}
 	/* check IP version */
@@ -59,7 +63,7 @@ int look_for_tcp_flags(unsigned char *dgram, unsigned int datalen)
 			struct tcphdr *tcphdrs =
 			    (struct tcphdr *) (dgram + 4 * iphdrs->ihl);
 			if (tcphdrs->fin || tcphdrs->ack || tcphdrs->rst) {
-				return 1;
+				RETURN_NO_LOG 1;
 			}
 		}
 	}
@@ -97,6 +101,8 @@ static int treat_packet(struct nfq_handle *qh, struct nfgenmsg *nfmsg,
 
 	q_pckt.payload_len = nfq_get_payload(nfa, &(q_pckt.payload));
 	if (q_pckt.payload_len == -1) {
+		log_area_printf(DEBUG_AREA_PACKET, DEBUG_LEVEL_INFO,
+				"Unable to get payload");
 		return 0;
 	}
 
@@ -129,21 +135,22 @@ static int treat_packet(struct nfq_handle *qh, struct nfgenmsg *nfmsg,
 			q_pckt.packet_id = ntohl(ph->packet_id);
 			auth_request_send(AUTH_CONTROL, &q_pckt);
 			IPQ_SET_VERDICT(q_pckt.packet_id, NF_ACCEPT);
-			return 1;
+			RETURN_NO_LOG 1;
 		} else {
+			log_area_printf(DEBUG_AREA_PACKET, DEBUG_LEVEL_VERBOSE_DEBUG,
+					"Can not get the packet headers");
 			return 0;
 		}
 	}
 	current = calloc(1, sizeof(packet_idl));
+	current->nfmark = q_pckt.mark;
+	current->timestamp = q_pckt.timestamp ;
+	current->id = 0;
 	if (current == NULL) {
 		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_MESSAGE,
 				"Can not allocate packet_id");
 		return 0;
 	}
-	current->nfmark = q_pckt.mark;
-	current->timestamp = q_pckt.timestamp ;
-	current->id = 0;
-	current->next = NULL;
 #ifdef PERF_DISPLAY_ENABLE
 	gettimeofday(&(current->arrival_time), NULL);
 #endif
@@ -152,9 +159,9 @@ static int treat_packet(struct nfq_handle *qh, struct nfgenmsg *nfmsg,
 	if (ph) {
 		current->id = ntohl(ph->packet_id);
 	} else {
+		free(current);
 		log_area_printf(DEBUG_AREA_PACKET, DEBUG_LEVEL_INFO,
 				"Can not get id for message");
-		free(current);
 		return 0;
 	}
 
@@ -173,8 +180,7 @@ static int treat_packet(struct nfq_handle *qh, struct nfgenmsg *nfmsg,
 			/* we fail to send the packet so we free packet related to current */
 			pthread_mutex_lock(&packets_list.mutex);
 			/* search and destroy packet by packet_id */
-			sandf =
-			    psearch_and_destroy(q_pckt.packet_id,
+			sandf = psearch_and_destroy(q_pckt.packet_id,
 						&(q_pckt.mark));
 			pthread_mutex_unlock(&packets_list.mutex);
 
@@ -216,8 +222,6 @@ int packetsrv_open(void *data)
 	if (nfq_bind_pf(h, AF_INET) < 0) {
 		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_CRITICAL,
 				"[!] Error during nfq_bind_pf()");
-		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_CRITICAL,
-				"Maybe you need to compile NF_NETLINK* kernel options as modules (not built in the kernel!)");
 		return -1;
 	}
 
@@ -273,12 +277,6 @@ int packetsrv_open(void *data)
 
 void packetsrv_close(int smart)
 {
-	/* Clear all remaining packets, nfqueue will automatically drop 
-	 * queued packets when queue will be destroyed. */
-	pthread_mutex_lock(&packets_list.mutex);
-	clear_packet_list();
-	pthread_mutex_unlock(&packets_list.mutex);
-
 	log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_SERIOUS_MESSAGE,
 			"Destroy netfilter queue socket");
 	if (smart)
@@ -310,17 +308,17 @@ void packetsrv_ipq_process(unsigned char *buffer)
 	if (look_for_tcp_flags(msg_p->payload, msg_p->data_len)) {
 		auth_request_send(AUTH_CONTROL, &q_pckt);
 		IPQ_SET_VERDICT(msg_p->packet_id, NF_ACCEPT);
-		return;
+		RETURN_NO_LOG;
 	}
 
 	/* Create packet */
 	current = calloc(1, sizeof(packet_idl));
 	if (current == NULL) {
 		/* no more memory: drop packet and exit */
+		IPQ_SET_VERDICT(msg_p->packet_id, NF_DROP);
 		log_area_printf(DEBUG_AREA_MAIN | DEBUG_AREA_PACKET,
 				DEBUG_LEVEL_SERIOUS_WARNING,
 				"[+] Can not allocate packet_id (drop packet)");
-		IPQ_SET_VERDICT(msg_p->packet_id, NF_DROP);
 		return;
 	}
 	current->id = msg_p->packet_id;
@@ -447,12 +445,6 @@ void *packetsrv(void *void_arg)
 
 			if (err == EBADF) {
 				struct stat s;
-				int rebind;
-
-				if ((fstat(fd, &s)<0)) {
-					packetsrv_close(0);
-					rebind = 1;
-				}
 #ifdef HAVE_NFQ_INDEV_NAME
 				if ((fstat(if_fd, &s)<0)) {
 					iface_table_close(nlif_handle);
@@ -467,7 +459,8 @@ void *packetsrv(void *void_arg)
 					}
 				}
 #endif
-				if (rebind) {
+				if ((fstat(fd, &s)<0)) {
+					packetsrv_close(0);
 #ifdef HAVE_NFQ_INDEV_NAME
 					fd = packetsrv_open(nlif_handle);
 #else
@@ -526,11 +519,13 @@ void *packetsrv(void *void_arg)
 		pckt_rx++;
 	}
 
-	packetsrv_close(!fatal_error);
 #ifdef HAVE_NFQ_INDEV_NAME
 	iface_table_close(nlif_handle);
 #endif
-#else /* USE_NFQUEUE */
+
+
+	packetsrv_close(!fatal_error);
+#else				/* USE_NFQUEUE */
 	unsigned char buffer[BUFSIZ];
 	int size;
 
@@ -597,7 +592,7 @@ void *packetsrv(void *void_arg)
 		packetsrv_ipq_process(buffer);
 	}
 	ipq_destroy_handle(hndl);
-#endif /* else USE_NFQUEUE */
+#endif
 	log_area_printf(DEBUG_AREA_MAIN | DEBUG_AREA_PACKET,
 			DEBUG_LEVEL_WARNING,
 			"[+] Leave packet server thread");
@@ -637,72 +632,70 @@ void shutdown_tls()
  * and it's structure is ::nufw_to_nuauth_auth_message_t.
  *
  * \param type Type of request (::AUTH_REQUEST, ::AUTH_CONTROL, ...)
- * \param pckt_datas A pointer to a queued_pckt:: holding packet information
+ * \param pckt_data A pointer to a queued_pckt:: holding packet information
  * \return If an error occurs returns 0, else return 1.
  */
-int auth_request_send(uint8_t type, struct queued_pckt *pckt_datas)
+int auth_request_send(uint8_t type, struct queued_pckt *pckt_data)
 {
-	unsigned char datas[512];
+	unsigned char data[512];
 	nuv4_nufw_to_nuauth_auth_message_t *msg_header =
-	    (nuv4_nufw_to_nuauth_auth_message_t *) & datas;
+	    (nuv4_nufw_to_nuauth_auth_message_t *) & data;
 	unsigned char *msg_content =
-	    datas + sizeof(nuv4_nufw_to_nuauth_auth_message_t);
+	    data + sizeof(nuv4_nufw_to_nuauth_auth_message_t);
 	int msg_length;
 
 	/* Drop non-IPv(4|6) packet */
-	if ((((struct iphdr *) (pckt_datas->payload))->version != 4)
-	    && (((struct iphdr *) (pckt_datas->payload))->version != 6)) {
-		debug_log_printf(DEBUG_AREA_PACKET, DEBUG_LEVEL_DEBUG,
+	if ((((struct iphdr *) (pckt_data->payload))->version != 4)
+	    && (((struct iphdr *) (pckt_data->payload))->version != 6)) {
+		log_area_printf(DEBUG_AREA_PACKET, DEBUG_LEVEL_DEBUG,
 				 "Dropping non-IPv4/non-IPv6 packet (version %u)",
-				 ((struct iphdr *) (pckt_datas->payload))->
+				 ((struct iphdr *) (pckt_data->payload))->
 				 version);
 		return 0;
 	}
 
 	/* Truncate packet content if needed */
-	if (sizeof(datas) <
-	    sizeof(nuv4_nufw_to_nuauth_auth_message_t) +
-	    pckt_datas->payload_len) {
+	if (sizeof(data) <
+	    sizeof(nuv4_nufw_to_nuauth_auth_message_t) + pckt_data->payload_len) {
 		debug_log_printf(DEBUG_AREA_PACKET, DEBUG_LEVEL_DEBUG,
 				 "Very long packet: truncating!");
-		pckt_datas->payload_len =
-		    sizeof(datas) -
+		pckt_data->payload_len =
+		    sizeof(data) -
 		    sizeof(nuv4_nufw_to_nuauth_auth_message_t);
 	}
-	msg_length =
-	    sizeof(nuv4_nufw_to_nuauth_auth_message_t) +
-	    pckt_datas->payload_len;
+	msg_length = sizeof(nuv4_nufw_to_nuauth_auth_message_t) + pckt_data->payload_len;
 
 	/* Fill message header */
 	msg_header->protocol_version = PROTO_NUFW_VERSION;
 	msg_header->msg_type = type;
 	msg_header->msg_length = htons(msg_length);
-	msg_header->packet_id = htonl(pckt_datas->packet_id);
-	msg_header->timestamp = htonl(pckt_datas->timestamp);
+	msg_header->packet_id = htonl(pckt_data->packet_id);
+	msg_header->timestamp = htonl(pckt_data->timestamp);
 
 	/* Add info about interfaces */
-	msg_header->mark = pckt_datas->mark;
-	memcpy(msg_header->indev, pckt_datas->indev,
+	msg_header->mark = pckt_data->mark;
+	memcpy(msg_header->indev, pckt_data->indev,
 	       IFNAMSIZ * sizeof(char));
-	memcpy(msg_header->outdev, pckt_datas->outdev,
+	memcpy(msg_header->outdev, pckt_data->outdev,
 	       IFNAMSIZ * sizeof(char));
-	memcpy(msg_header->physindev, pckt_datas->physindev,
+	memcpy(msg_header->physindev, pckt_data->physindev,
 	       IFNAMSIZ * sizeof(char));
-	memcpy(msg_header->physoutdev, pckt_datas->physoutdev,
+	memcpy(msg_header->physoutdev, pckt_data->physoutdev,
 	       IFNAMSIZ * sizeof(char));
 
 	/* Copy (maybe truncated) packet content */
-	memcpy(msg_content, pckt_datas->payload, pckt_datas->payload_len);
+	memcpy(msg_content, pckt_data->payload, pckt_data->payload_len);
 
 	/* Display message */
 	log_area_printf(DEBUG_AREA_PACKET, DEBUG_LEVEL_DEBUG,
-			"Sending request for %lu", pckt_datas->packet_id);
+			"Sending request for %lu", (long)pckt_data->packet_id);
 
 	/* cleaning up current session : auth_server has detected a problem */
 	pthread_mutex_lock(&tls.mutex);
 	if ((tls.auth_server_running == 0) && tls.session != NULL) {
 		close_tls_session();
 	}
+
 	pthread_mutex_unlock(&tls.mutex);
 
 	/* negotiate TLS connection if needed */
@@ -710,28 +703,40 @@ int auth_request_send(uint8_t type, struct queued_pckt *pckt_datas)
 		log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_INFO,
 				"Not connected, trying TLS connection");
 		tls_connect();
+
 		if (tls.session) {
+			char buf[256];
+			buf[0] = '\0';
+			nussl_session_get_cipher(tls.session, buf, sizeof(buf));
 			log_area_printf(DEBUG_AREA_GW,
 					DEBUG_LEVEL_WARNING,
-					"[+] TLS connection to nuauth restored");
+					"[+] TLS connection to nuauth restored (%s:%d), cipher is %s",
+					authreq_addr, authreq_port,
+					(buf[0] != '\0') ? buf : "none" );
+
 		} else {
 			log_area_printf(DEBUG_AREA_GW,
-					DEBUG_LEVEL_CRITICAL,
-					"[!] TLS connection to nuauth can NOT be restored");
+					DEBUG_LEVEL_WARNING,
+					"[!] TLS connection to nuauth can NOT be restored (%s:%d)",
+					authreq_addr, authreq_port);
 			return 0;
 		}
 	}
 
 	/* send packet */
 	pthread_mutex_lock(&tls.mutex);
-	if (gnutls_record_send(*(tls.session), datas, msg_length) <= 0) {
-		log_area_printf(DEBUG_AREA_GW,
-				DEBUG_LEVEL_CRITICAL,
-				"[!] Error while sending TLS data");
+
+	if (nussl_write(tls.session, (char*)data, msg_length) < 0) {
+		debug_log_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_DEBUG,
+				 "Error during nussl_write (auth_request_send).");
 		shutdown_tls();
 		pthread_mutex_unlock(&tls.mutex);
+		log_area_printf(DEBUG_AREA_GW,
+				DEBUG_LEVEL_WARNING,
+				"[!] TLS send failure");
 		return 0;
 	}
 	pthread_mutex_unlock(&tls.mutex);
 	return 1;
 }
+
