@@ -1,13 +1,13 @@
 /*
-** Copyright (C) 2002-2007 INL
-** Written by ric Leblond <regit@inl.fr>
+** Copyright (C) 2002-2008 INL
+** Written by Eric Leblond <regit@inl.fr>
 ** INL http://www.inl.fr/
 **
-** $Id: structure.h 5312 2008-11-25 09:17:29Z pollux $
+** $Id$
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; version 2 of the License.
+** the Free Software Foundation; version 3 of the License.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -66,7 +66,7 @@ char *nuauth_cert_dn;		/*!< NuAuth certificate filename, default value: NULL */
 char authreq_addr[HOSTNAME_SIZE];
 
 /*! Port of NuAuth server address (::adr_srv), default value: #AUTHREQ_PORT */
-char authreq_port[20];
+unsigned int authreq_port;
 
 /*! Number of second before a packet is dropped, default value: #PACKET_TIMEOUT */
 int packet_timeout;
@@ -171,7 +171,7 @@ struct nufw_threadargument {
 };
 
 /* mutex */
-pthread_mutex_t hndl_mutex;
+extern pthread_mutex_t ipq_mutex;
 
 /** \def IPQ_SET_VERDICT(PACKETID, DECISION)
  * Set decision (NF_ACCEPT or NF_DROP) of a packet. Call nfq_set_verdict()
@@ -183,17 +183,34 @@ pthread_mutex_t hndl_mutex;
  * nfq_set_verdict_mark() or ipq_set_vwmark().
  */
 
-/* do some define to add mutex usage */
 #if USE_NFQUEUE
 #define IPQ_SET_VERDICT(PACKETID, DECISION) \
-	nfq_set_verdict(hndl, PACKETID, DECISION, 0 , NULL)
+	do { \
+		pthread_mutex_lock(&ipq_mutex); \
+		nfq_set_verdict(hndl, PACKETID, DECISION, 0 , NULL); \
+		pthread_mutex_unlock(&ipq_mutex); \
+	} while(0)
+
 #define IPQ_SET_VWMARK(PACKETID, DECISION, NFMARK) \
-	nfq_set_verdict_mark(hndl, PACKETID, DECISION, NFMARK, 0, NULL)
+	do { \
+		pthread_mutex_lock(&ipq_mutex); \
+		nfq_set_verdict_mark(hndl, PACKETID, DECISION, NFMARK, 0, NULL); \
+		pthread_mutex_unlock(&ipq_mutex); \
+	} while(0)
 #else
 #define	IPQ_SET_VERDICT(PACKETID, DECISION) \
-	ipq_set_verdict(hndl, PACKETID, DECISION,0,NULL)
+	do { \
+		pthread_mutex_lock(&ipq_mutex); \
+		ipq_set_verdict(hndl, PACKETID, DECISION,0,NULL); \
+		pthread_mutex_unlock(&ipq_mutex); \
+	} while(0)
+
 #define	IPQ_SET_VWMARK(PACKETID, DECISION, NFMARK) \
-	ipq_set_vwmark(hndl, PACKETID, DECISION, NFMARK,0,NULL)
+	do { \
+		pthread_mutex_lock(&ipq_mutex); \
+		ipq_set_vwmark(hndl, PACKETID, DECISION, NFMARK,0,NULL); \
+		pthread_mutex_unlock(&ipq_mutex); \
+	} while(0)
 #endif
 
 int pckt_tx;			/*!< Number of transmitted packets since NuFW is running */
