@@ -1,4 +1,4 @@
-#!/usr/bin/python2.4
+#!/usr/bin/python
 from unittest import TestCase, main
 from common import startNufw, connectClient
 from nuauth import Nuauth
@@ -15,12 +15,12 @@ class TestPlaintextAcl(TestCase):
         self.iptables = Iptables()
         self.users = USERDB
         self.config = NuauthConf()
-        self.config["xml_defs_periodfile"] = '"%s"' % os.path.join(os.getcwd(),"../conf/periods.xml")
+        self.config["xml_defs_periodfile"] = '"%s"' % os.path.abspath("../conf/periods.xml")
         self.acls = PlaintextAcl()
 
         # Start nuauth with new config
         self.users.install(self.config)
-        self.nufw = startNufw()
+        self.nufw = startNufw(["-s"])
 
     def tearDown(self):
         # Restore user DB and nuauth config
@@ -41,7 +41,7 @@ class TestPlaintextAcl(TestCase):
         self.nuauth = Nuauth(self.config)
 
         user = self.users[0]
-        client = user.createClient()
+        client = user.createClientWithCerts()
         testPort(self, self.iptables, client, VALID_PORT, False)
 
         self.acls.desinstall()
@@ -58,7 +58,7 @@ class TestPlaintextAcl(TestCase):
         self.nuauth = Nuauth(self.config)
 
         user = self.users[0]
-        client = user.createClient()
+        client = user.createClientWithCerts()
         testAllowPort(self, self.iptables, client)
 
         self.acls.desinstall()
