@@ -1,6 +1,6 @@
-#!/usr/bin/python2.4
+#!/usr/bin/python
 from unittest import TestCase, main
-from common import getNuauthConf, createClient, connectClient
+from common import getNuauthConf, createClientWithCerts, connectClient
 from nuauth import Nuauth
 from nuauth_conf import NuauthConf
 from logging import warning
@@ -8,6 +8,7 @@ from logging import warning
 class TestLog(TestCase):
     def setUp(self):
         config = getNuauthConf()
+        config["nuauth_tls_request_cert"] = "0"
         config["nuauth_user_logs_module"] = '"syslog"'
         config["nuauth_user_session_logs_module"] = '"syslog"'
         self.nuauth = Nuauth(config)
@@ -25,7 +26,7 @@ class TestLog(TestCase):
 
     def testLogin(self):
         # Client login
-        client = createClient()
+        client = createClientWithCerts()
         self.assert_(connectClient(client))
 
         # Check log output
